/****************************************************************************
** Leistungsorientierte Krankenanstaltenfinanzierung (LKF)                 **
** Scoring-C-Sourcen V2024.0                                               **
** (C) 2023 BMSGPK                                                         **
*****************************************************************************
** lkfstru5.h - Headerfile fuer zusaetzliche Satzarten ab 2004             **
**                                                                         **
**		 Alex.Holzer@chello.at                                             **
**       Bernhard.Pesec@dothealth.at                                       **
***************************************************************************/

#define LKF_FIELD_EMPTY -1
#define LKF_FIELD_ERROR -2

/* Generelle Feldnummern (reserved: 0 - 9) */

#define GEN_KANR					  0
#define GEN_AUFNR					  1
#define GEN_ABTPOS					  2
#define GEN_ERHEBDAT				  3
#define GEN_JAHR					  4
#define GEN_ABTFC					  5
#define GEN_LINEID					  6

/* Satzart X01 */
#define X01_DATENSATZID               2
#define X01_AUFNAHMEDATUM             3
#define X01_AUFNAHMEUHRZEIT           4
#define X01_BEHANDLUNGSART            5
#define X01_GEBURTSDATUM              6
#define X01_ALTERSGRUPPE              7
#define X01_GESCHLECHT                8
#define X01_STAATB                    9
#define X01_STAATW                   10
#define X01_PLZW                     11
#define X01_GEMEINDECODE             12
#define X01_AUFART1                  13
#define X01_AUFART2                  14
#define X01_TRANSPORTART             15
#define X01_ZUGEWIESEN_VON           16
#define X01_ZUGEWIESEN_AN            17
#define X01_ENTART                   18
#define X01_ENTDAT                   19
#define X01_ENTZEIT                  20
#define X01_LEIST_KOST               21
#define X01_ABRECHNUNGSRELEVANZ      22
#define X01_PLAUSKZ                  23
#define X01_UEBERMITTLUNG_KOST       24
#define X01_ABRECHQUARTAL            25

/* Satzart X02 */
#define X02_POSITIONSNUMMER           4
#define X02_ABTFC                     5
#define X02_AUFDAT                    6
#define X02_AUFZEIT                   7
#define X02_PHYSANW                   8
#define X02_ENTDAT                    9
#define X02_ENTZEIT                  10
#define X02_ENTART                   11
#define X02_FACH_FC                  12
#define X02_PFLEG_FC                 13
#define X02_NEONAT                   14
#define X02_ALTERSGRUPPE             15
#define X02_AUFNAHME_FUCO            16

/* Satzart X03 */
#define X03_DIAG                      4
#define X03_DIAGTYP                   5
#define X03_DIAGART                   6
#define X03_DIAGSTATIONAER            7
#define X03_AUFNAHME_FUCO             8

/* Satzart X04 */
#define X04_MEL                       4
#define X04_SEITE                     5
#define X04_ABRECHNUNGSRELEVANZ       6
#define X04_ANZAHL                    7
#define X04_DATUM                     8
#define X04_UHRZEIT                   9
#define X04_FCLEIST                  10
#define X04_KALEIST                  11
#define X04_AUFNAHME_FUCO            12

/* Satzart X05 */
#define X05_GRUPPE                    4
#define X05_KNOTEN                    5
#define X05_LEIST                     6
#define X05_TAG                       7
#define X05_LEIST_UNT                 8
#define X05_TAG_UNT                   9
#define X05_TAG_OBEN                 10
#define X05_INTENSIV                 11
#define X05_MELZU                    12
#define X05_SPEZ                     13
#define X05_SUMME                    14

/* Satzart X06 */
#define X06_WERT                      4
#define X06_CODE                      5
#define X06_KOMMENTAR                 6

/* Satzart X07 */
#define X07_KOMM1                     4
#define X07_KOMM2                     5
#define X07_KOMM3                     6
#define X07_KOMM4                     7
#define X07_KOMM5                     8
#define X07_KOMM6                     9

/* Satzart I01: Physiologische Daten */

#define I01_SI_INDIKATOR			 11
#define I01_SAUERSTOFFPART			 12
#define I01_ATMUNGSMODUS			 13
#define I01_BLUTBILD_LEUKOZYTEN		 14
#define I01_COMASCALE_MOTORISCH		 15
#define I01_COMASCALE_VERBAL		 16
#define I01_COMASCALE_VISUELL		 17
#define I01_HARN_VOLUMEN24H			 18
#define I01_HERZFREQUENZ			 19
#define I01_SAUERSTOFFKONZ_INSP		 20
#define I01_KOERPERTEMP				 21
#define I01_PULMONALISKATHETER		 22
#define I01_SERUM_BIKARBONAT		 23
#define I01_SERUM_GESAMT_BILIRUBIN   24
#define I01_SERUM_HARNSTOFF			 25
#define I01_SERUM_HARNSTOFF_N		 26
#define I01_SERUM_KALIUM			 27
#define I01_SERUM_NATRIUM			 28
#define I01_SYSTOL_ARTER_BLUTDRUCK	 29

/* Satzart I02: TISS28-Daten */

#define I02_STANDARD_MONITORING		 12
#define I02_LABOR					 13
#define I02_MEDIKATION_EINMALIG		 14
#define I02_MEDIKATION_MEHRFACH		 15
#define I02_ROUTINEVERBANDSWECHSEL	 16
#define I02_VERBANDSWECHSEL_HAEUFIG  17
#define I02_DRAINAGENPFLEGE			 18
#define I02_BEATMUNG_MECH			 19
#define I02_ATEMUNTERSTUETZUNG		 20
#define I02_TUBUSPFLEGE				 21
#define I02_ATEMTHERAPIE			 22
#define I02_MEDIKATION_VAS_EINMALIG  23
#define I02_MEDIKATION_VAS_MEHRFACH  24
#define I02_FLUESSIGERSATZ_MASSIV    25
#define I02_KATHETER_ARTERIELL		 26
#define I02_PULMONALISKATHETER		 27
#define I02_KATHETER_ZENTRALVENOES   28
#define I02_REANIMATION_KARDIO		 29
#define I02_NIERENERSATZTHERAPIE	 30
#define I02_HARNBILANZIERUNG		 31
#define I02_DIURESE_AKTIV			 32
#define I02_HIRNDRUCKMESSUNG		 33
#define I02_METABOL_ENTGLEISUNGEN	 34
#define I02_ERNAEHRUNG_ENTERAL		 35
#define I02_ERNAEHRUNG_PARENTERAL	 36
#define I02_INTERVENTION_EINFACH	 37
#define I02_INTERVENTION_MEHRFACH	 38
#define I02_DIAGTHER_AUSS_IBS		 39

/* Satzart I03: TRISS-Daten (DLB-Intensiv) */

#define I03_AIS_ABDOMEN				 11
#define I03_AIS_EXTERN				 12
#define I03_AIS_EXTREMITAETEN		 13
#define I03_AIS_GESICHT				 14
#define I03_AIS_HALS_KOPF			 15
#define I03_AIS_THORAX				 16
#define I03_AIS_TYP					 17
#define I03_COMASCALE_AUF_MOTREAK	 18
#define I03_COMASCALE_AUF_VERBREAK	 19
#define I03_COMASCALE_AUF_VISREAK	 20
#define I03_SYST_BLUTDRUCK_AUFNAHME  21
#define I03_ATEMFREQUENZ_AUFNAHME	 22

/* Satzart I11 - SAPS3 (V15.1 - Modell 2011) */

#define I11_AKUTEINF_AUFNAHME				30
#define I11_AUFN_ANDERER					31
#define I11_AUFN_BASISPFLEGE_OBSERVANZ		32
#define I11_AUFN_GASTROINTESTINAL			33
#define I11_AUFN_HAEMATOLOGISCH				34
#define I11_AUFN_KARDIOVASKULAER			35
#define I11_AUFN_LEBER						36
#define I11_AUFN_METABOLISCH				37
#define I11_AUFN_NEUROLOGISCH				38
#define I11_AUFN_RENAL						39
#define I11_AUFN_RESPIRATORISCH				40
#define I11_AUFN_SCHWERESTRAUMA				41
#define I11_AUFNAHMETYP						42
#define I11_BLUTBILD_PH						43
#define I11_BLUTBILD_THROMBO				44
#define I11_CHIRURGIE_EINGRIFF				45
#define I11_CHIRURG_STATUS					46
#define I11_KOEXIST_ERKRANKUNGEN			47
#define I11_MECHANISCHE_BEATMUNG			49
#define I11_SERUM_KREATININ					50
#define I11_THERAPEUTISCHE_MASSN			51
#define I11_ZUTRANSFERIERUNG				52

#define I11_DATENSATZID                     53
#define I11_AUFNAHMEDATUM                   54



/* Satzart I12 TISS-A-Daten (V15.1 - Modell 2011) */

#define I12_MEDIKAMENTE				I02_MEDIKATION_EINMALIG
#define I12_MEDIKAMENTE_VASOAKTIV	I02_MEDIKATION_VAS_EINMALIG
#define I12_VERBANDSWECHSEL			I02_ROUTINEVERBANDSWECHSEL	
#define I12_MONITORING_HAEMO		41
#define I12_HIRNDRUCKMESSUNG		44
#define I12_INTERVENTIONEN			45
#define I12_AGITATION_DELIRIUM		46
#define I12_ASSIST_KARDIAL			47
#define I12_ASSIST_PULMONAL			48
#define I12_HYPOTHERMIE_THERAP		49
#define I12_LEBERSUPPORT_EXTRAKORP	50
#define I12_KONT_INTRA_AKONVU_THERAP 51

#define I12_DATENSATZID                     I11_DATENSATZID
#define I12_AUFNAHMEDATUM                   I11_AUFNAHMEDATUM


/* Satzart M02: Aufenthalts-Daten fr Intensiv-Scoring (ab Position 38 des M02-Satzes) */
#define M02_ZUGANGSUHRZEIT			 11
#define M02_AUFART_INTENSIV			 12
#define M02_AUFART_SAPSII			 13
#define M02_COEXIST_ERKRANKUNGEN	 14
#define M02_ABGANGSDATUM			 15
#define M02_ABGANGSUHRZEIT			 16
#define M02_ABGANGSART				 17

/* Satzart S01: Prf- und Summensatz */

#define S01_PERIODE						12
#define S01_PERIODENNUMMER				13
#define S01_LFDNR						14
#define S01_ANZAHL_M					15
#define S01_ANZAHL_I					23
#define S01_ANZAHL_L					26
#define S01_ANZAHL_K					30
#define S01_ANZAHL_G					46
#define S01_ANZAHL_FEHLER				64
#define S01_ANZAHL_WARNINGS				49
#define S01_PUNKTE_LDF_LEIST			50
#define S01_PUNKTE_LDF_TAG				51
#define S01_PUNKTE_BELAG_UNTEN_LEIST	52
#define S01_PUNKTE_BELAG_UNTEN_TAG		53
#define S01_ZUSATZPUNKTE_BELAG_OBEN		54
#define S01_ZUSATZPUNKTE_INTENSIV		55
#define S01_ZUSATZPUNKTE_MEHRFACH_MEL	56
#define S01_PUNKTE_SPEZ					57
#define S01_PUNKTE_TOT_INNERHALB_LKF	58
#define S01_PUNKTE_TOT_AUSSERHALB_LKF	59
#define S01_KOMMENTAR					60
#define S11_E01                         61
#define S11_P01                         62
#define S11_P02                         63
#define S11_ANZAHL_B                    65

/* Satzart L01: Spezielle Leistungsbereiche */

#define L01_LEISTUNGSBEREICH		11
#define L01_GRUPPE_STUFE			12
#define L01_PUNKTE					13
#define L01_FILL					20

/* Satzart L02: Abrechnungsrelevante Kostentrger */

#define L02_KOSTENTRAEGER_CODE		11
#define L02_LKF_RELEVANZ			12

/* Satzart L03: Exklusionen medizinischer Einzelleistungen (DLB) */

#define L03_MEL						11

/* Satzart L04: Exklusionen tagesklinischer medizinischer Einzelleistungen */

#define L04_MEL						11

/* Satzart O02: Private OEMELKA */

#define O02_MEL         2
#define O02_GESAMT      3
#define O02_ERG_MVS     4
#define O02_ERG_MINK    5
#define O02_ERG_INTE    6
#define O02_ERG_KOZ     7
#define O02_ERG_HP      8
#define O02_ERG_BASIS   9
#define O02_ERG_INTNP  10
#define O02_UNUSED1    11
#define O02_UNUSED2    12
#define O02_ERG_MFS    13
#define O02_KRIT_MVS   14
#define O02_KRIT_MINK  15
#define O02_KRIT_INTE  16
#define O02_KRIT_KOZ   17
#define O02_KRIT_HP    18
#define O02_KRIT_BASIS 19
#define O02_KRIT_INTNP 20
#define O02_UNUSED3    21
#define O02_UNUSED4    22
#define O02_KRIT_MFS   23


/* Satzart K01: Kostenstellenplan */

#define K01_INT_KOSTNR				12
#define K01_INT_KOSTBEZ				13
#define K01_SPEZ_ORG				14

/* Satzart K02: KA-Stammdaten */

#define K02_BEZEICHNUNG				11
#define K02_STRASSE					12
#define K02_PLZ						13
#define K02_ORT						14
#define K02_TELEFON					15
#define K02_FAX						16
#define K02_HOMEPAGE				17
#define K02_RECHT_BEZEICHNUNG		18
#define K02_RECHT_STRASSE			19
#define K02_RECHT_PLZ				20
#define K02_RECHT_ORT				21
#define K02_RECHT_TELEFON			22
#define K02_RECHT_FAX				23
#define K02_RECHT_HOMEPAGE			24
#define K02_AERZTLICHE_LEITUNG		25
#define K02_VERWALTUNGSLEITUNG		26
#define K02_PFLEGEDIENSTLEITUNG		27

/* Satzart K03: KA-Statistik (Ressourcen und Inanspruchnahme) */

#define K03_SYST_BETTEN_SUM			11
#define K03_SYST_BETTEN_SOND		12
#define K03_TATS_BETTEN_INSG		13
#define K03_TATS_BETTEN_SOND		14
#define K03_BELAGSTAGE_SOND			15
#define K03_AUFNAHMEN_SOND			16
#define K03_VORJAHR_SOND			17
#define K03_ENTLASSUNGEN_SOND		18
#define K03_VERSTORBENE_SOND		19
#define K03_AUFNAHME_BEGLEIT		20
#define K03_BELAGSTAGE_BEGLEIT		21
#define K03_AMBULANTE_PAT			22
#define K03_OP_TISCHE				23
#define K03_ENTBINDUNGSPLAETZE		24
#define K03_POSTOP_UEB_PLAETZE		25
#define K03_DIALYSEPLAETZE			26
#define K03_OBDUK_ANSTPERS			27
#define K03_OBDUK_FREMDPERS			28
#define K03_ENTBIND_LEBEND			29
#define K03_ENTBIND_TOT				30

#define K03_AMBPLAETZE              31

/* Satzart K04: KA-Statistik (Personal-Vollzeitaequivalente nach Funktionsgruppen und Dienstverhaeltnis) */

#define K04_PERSONALGRUPPENINDEX    11
#define K04_VOLLZEITAEQUIVALENTE    12

/* Satzart K05: KA-Statistik (Personal des aerztlichen Dienstes) */

#define K05_FACHHAUPTBEREICHSIND	11
#define K05_FACHAERZTE_KOPF			12
#define K05_AERZTE_ALLGMED_KOPF		13
#define K05_FACHAERZTE_AUSB_KOPF	14
#define K05_AERZTE_ALLG_AUSB_KOPF	15
#define K05_FACHAERZTE_VZEQ			16
#define K05_AERZTE_ALLG_VZEQ		17
#define K05_FACHAERZTE_AUSB_VZEQ	18
#define K05_AERZTE_ALLG_AUSB_VZEQ	19

/* Satzart K06: KA-Statistik (Konsiliaraerztlicher Dienst) */

#define K06_FACHRICHTUNGSINDEX		11
#define K06_KONSILIARAERZTE_KOPF	12

/* Satzart K07: KA-Statistik (Personal der nicht-aerztlichen Gesundheitsberufe) */

#define K07_NICHTAERTZL_GESBERUF		11
#define K07_PERSONAL_MAENNLICH			12
#define K07_PERSONAL_WEIBLICH			13
#define K07_EU_AUSL_INSG				14
#define K07_NEU_AUSL_INSG				15
#define K07_MAENNL_VZEQ					16
#define K07_WEIBL_VZEQ					17
#define K07_EU_AUSL_VZEQ				18
#define	K07_NEU_AUSL_VZEQ				19

/* Satzart K08: Kostenstellen-Statistik und Kostennachweis (Bettenf. Hauptkostenstelle) */

#define K08_SYST_BETTEN					11
#define K08_TATS_BETTEN					12
#define K08_NETTOGRUNDRISSFLAECHE		13
#define K08_MLV1_1_VZEQ					14
#define K08_MLV1_2_VZEQ					15
#define K08_MLV1_3_VZEQ					16
#define K08_MLV1_4_VZEQ					17
#define K08_MLV1_5_VZEQ					18
#define K08_MLV1_6_VZEQ					19
#define K08_MLV1_7_VZEQ					20
#define K08_MLV1_8_VZEQ					21
#define K08_MLV1_9_VZEQ					22
#define K08_KOAGR01						23
#define K08_KOAGR02						24
#define K08_KOAGR03						25
#define K08_KOAGR04						26
#define K08_KOAGR05						27
#define K08_KOAGR06						28
#define K08_KOAGR07						29
#define K08_KOAGR08						30
#define K08_KOAGR11						31
#define K08_KOAGR12						32
#define K08_KOAGR13						33
#define K08_KOAGR14						34
#define K08_KOSTENMINDERUNGEN			35
#define K08_SUM_ABG_INNERBETR_LST		36
#define K08_UNTER_UEBERDECKUNG			37
#define K08_ENDKOSTEN					38

/* Satzart K09: Kostenstellenstatistik und Kostennachweis (nicht-bettenfuehrende HK) */

#define K09_AMBULANTE_PATIENTEN			11
#define K09_FREQ_AMB_PAT				12
/* 13 - 38 gleich wie K08 */
#define K09_FREQ_STAT_PAT				39
#define K09_FREQ_STAT_PAT_ANDKH			40

#define K09_AMBPLAETZE                  41

/* Satzart K10: Kostenstellenstatistik und Kostennachweis (Neben- und Hilfskostenstellen) */

#define K10_PERS_INSG_VZEQ				11

/* Satzart K11: Sammel-Kostennachweis Summenblatt */

#define K11_SUM_PRIMAERKOSTEN			11
#define K11_KALK_ABSCHREIBUNGEN			12
#define K11_KALK_ABSCHR_RESTW0			13
#define K11_KALK_ABSCHR_AUSG_ANL		14
#define K11_KALK_ZINSEN					15
#define K11_KALK_MIETEN					16
#define K11_ENDK_STAT_PATVERS			17
#define K11_ENDK_AMB_PATVERS			18
#define K11_ENDK_NEBENKOSTENST			19
/* KOAGR01 - KOAGR08, KOSTENMIND, ENDKOSTEN aus K08 */

/* Satzart K12: Sammel-Kostennachweis Detailblaetter */

#define K12_BLATTNUMMER					11
#define K12_MLVNUMMER					12
#define K12_AUFWAND_FINANZBUCHF			13
#define K12_NEUTRALER_AUFWAND			14
#define K12_KALK_KOSTEN					15
#define K12_KOSTEN						16

/* Satzart K13: Kalk. Anhang - Kostenminderungen */

#define K13_INDEX_KOSTENMIN_ART			11
#define K13_KOSTENMINDERUNGEN			12

/* Satzart K14: Kalk. Anhang - Kalk. Anlagenspiegel */

#define K14_ANLAGENINDEX				11
#define K14_ANSCHAFF_HERSTKOSTEN		12
#define K14_ZUGAENGE					13
#define K14_ABGAENGE					14
#define K14_UMBUCHUNGEN					15
#define K14_KUM_ABSCHREIBUNGEN			16
#define K14_BUCHW_ENDE_JAHR				17
#define K14_BUCHW_VORJAHR				18
#define K14_ABSCHR_LFD_JAHR				19

/* Satzart K15: Kalk. Anhang - Allg. Kostenbereiche */

#define K15_KENNZAHLENINDEX				11
#define K15_KOSTEN						12
#define K15_BEZUGSGROESSE				13

/* Satzart K16: Kalk. Anhang - Auswgewaehlte Kosten */

#define K16_KOSTENINDEX					11
#define K16_KOSTEN						12

/* Satzart G01: Grossgeraetebasisdaten */

#define G01_POSNR						11
#define G01_ART							12
#define G01_TYPE						13
#define G01_HERSTELLER					14
#define G01_BAUJAHR						15
#define G01_ANSCHAFFKOSTEN				16
#define G01_BETRIEBSZEITEN				17
#define G01_KOMMENTAR					18

/* Satzart G02: Grossgeraeteleistungsdaten */

#define G02_POSNR						11
#define G02_LSTINDEX					12
#define G02_AMB_FREQUENZEN				13
#define G02_STAT_FREQUENZEN				14
#define G02_AVG_DAUER_FREQ_MIN			15

/* Satzart R01: Gebarung laut Rechnungsabschlu */

#define R01_BER_BETRIEBSERG				11

/* Satzart R02: Einnahmenstruktur */

#define R02_FINANZTRAEGERINDEX			11
#define R02_EINNAHMENARTENINDEX			12
#define R02_EINNAHMEN					13

/* Satzarten B01 - B06 */
#define B01_BERICHTSTYP					11
#define B01_BERECH_TYP					12
#define B01_ZUORD_ART					13

#define B02_VKSINDEX					12
#define B02_WERT						13
#define B02_WERT_VORJAHR				14

#define B03_EVRINDEX					12
#define B04_QVAINDEX					12

#define B05_ERLOESINDEX					12
#define B05_WERT_GESAMT					13
#define B05_WERT_LGF					14

#define B06_FINANZIERUNGSINDEX			12
#define B06_ZUSCHUSSARTINDEX			13
#define B06_ZUSCHUSS					14

#define LKF_LEN_FOR_EXPANDEDLINE        98765

/* Argument Prototypes of lkfstru5.c */
const LKF_SATZART_DEFS *SatzartDef_Init (int year);

int Satzart_GetValueInt (const char *buf, unsigned int buflen, const SATZART_FELD *pFeld, int *pVal);
int Satzart_GetValueFloat (const char *buf, unsigned int buflen, const SATZART_FELD *pFeld, double *pValue, int modell);
unsigned int Satzart_GetValueString (const char *buf, unsigned int buflen, const SATZART_FELD *pFeld, char *value, unsigned int maxlen);
unsigned int Satzart_GetValueString2017 (const char *buf, unsigned int buflen, const SATZART_FELD *pFeld, unsigned int indexOfFieldNum, char *value, unsigned int maxlen); /* V21.1 */
int Satzart_GetValueDouble (const char *buf, unsigned int buflen, const SATZART_FELD *pFeld, unsigned int nachkommastellen, double *pVal);
int Satzart_GetValueDate (const char *ds, unsigned int buflen, const SATZART_FELD *pFeld);
int Satzart_IsEmpty (const char *buf, unsigned int len, const SATZART_FELD *pFeld);
int Satzart_IsNumeric (const char *buf, const SATZART_FELD *pFeld, int flags);
int Satzart_IsSatzart (const char *buf, const SATZART *pSatzart);

/* V21.0 - LKF_EXPANDED_LINE versions */
int Satzart_GetValueIntExp (const LKF_EXPANDED_LINE *expLine, const SATZART_FELD *pFeld, int *pVal);
int Satzart_GetValueFloatExp (const LKF_EXPANDED_LINE *expLine, const SATZART_FELD *pFeld, double *pVal, int modell);
unsigned int Satzart_GetValueStringExp (const LKF_EXPANDED_LINE *buf, const SATZART_FELD *pFeld, char *value, unsigned int maxlen);
int Satzart_GetValueDateExp (const LKF_EXPANDED_LINE *pLine, const SATZART_FELD *pFeld);
int Satzart_IsEmptyExp (const LKF_EXPANDED_LINE *expLine, const SATZART_FELD *pFeld);

const SATZART_INFO *Satzart_GetSatzart (const LKF_SATZART_DEFS *pDef, const char *buf);
unsigned int Satzart_GetLines (LKF_LINE_BUFFER *pLineBuffer, const SATZART *pSatzart, const char **ppLines, unsigned int max_lines);
unsigned int Satzart_GetLinesAlloc (LKF_LINE_BUFFER *pLineBuffer, const SATZART *pSatzart, const char ***pppLines);
unsigned int Satzart_GetLinesAllocExp (LKF_EXPANDED_LINES *pExpandedLines, const SATZART *pSatzart, const LKF_EXPANDED_LINE ***pppLines); /* V21.0 */
void Satzart_GetLinesFree  (const char **ppLines);
void Satzart_SyntaxError (LKF_CONTEXT *pContext, const char *ds, const SATZART_INFO *pSatzartInfo, const SATZART_FELD *pFeld, const LKF_SATZART_ERRORDEF *pErrorDef);
unsigned int Satzart_Check (LKF_CONTEXT *pContext, const char *ds, const SATZART_INFO *pSatzartInfo, LKF_CHECK_CONTEXT *pContextInfo);
unsigned int Satzart_CheckExp (LKF_CONTEXT *pContext, const LKF_EXPANDED_LINE *expLine, const SATZART_INFO *pSatzartInfo, LKF_CHECK_CONTEXT *pContextInfo);
void lkf_Satzart_Counter_Clear (LKF_CONTEXT *pContext);
