//
//  LKFTestCache.swift
//  lkfc
//
//  Created by Alexander Holzer on 10.06.18.
//  Copyright © 2018 Alexander Holzer. All rights reserved.
//

import Foundation
@_implementationOnly import LibLKF

public class LKFCompareCache {

    weak var context: LKFContext?
    var x05Lines = [String]()
    var x05TargetLines = [String]()
    
    init(context: LKFContext) {
        self.context = context
    }
    
    func sourceLineAdded (line: String) {
        if line.hasPrefix("X05") || line.hasPrefix("M06") {
            x05Lines.append(line)
        }
    }
    
    func targetLineAdded (line: String) {
        if line.hasPrefix("X05") || line.hasPrefix("M06") {
            x05TargetLines.append(line)
        }
    }
    
    func applyResult () {
        if let context = context {
            let count = LibLKF.lkf_get_aufenthalt_anz(context.ptr!)
            let lineBuffer = UnsafeMutablePointer<CChar>.allocate(capacity:1024)
            
            for i in 0..<count {
                let aufenthalt = LKFAufenthalt(context:context, index:i)
                _ = LibLKF.lkf_auf_line_scoring(aufenthalt.ptr, lineBuffer)
                targetLineAdded(line: String(cString:lineBuffer))
            }
            lineBuffer.deallocate()
        }
    }
    
    public func compare () -> [String]? {
        return compare(removeBM: false)
    }
    
    public func remove (part: Int, nullToBlank: Bool, from line: String) -> String {
        var parts = line.split(separator: "|", maxSplits: Int.max, omittingEmptySubsequences: false)
        
        if parts.count > part {
            parts[part] = ""
            
            for i in part..<parts.count {
                if parts[i] == "0" {
                    parts[i] = ""
                }
            }
            
            return parts.joined(separator: "|")
        }
        return line
    }
    
    public func compare (removeBM: Bool) -> [String]? {
        let sortedSource = x05Lines.sorted {
            $0.compare($1) == ComparisonResult.orderedAscending
        }
        
        let sortedTarget = x05TargetLines.sorted {
            $0.compare($1) == ComparisonResult.orderedAscending
        }

        var diffs = [String]()
        
        
        var sourceSlice = ArraySlice<String>(sortedSource)
        var targetSlice = ArraySlice<String>(sortedTarget)
        
        var source : String?
        var target : String?
        
        repeat {
            source = sourceSlice.first
            target = targetSlice.first
            
            
            if source != nil && target != nil {

                if removeBM {
                    source = remove (part: 3, nullToBlank: true, from: source!)
                }


                switch (source!.compare(target!)) {
                case .orderedAscending:
                    diffs.append("source: \(source!)")
                    sourceSlice = sourceSlice.dropFirst()
                    break
                case .orderedDescending:
                    diffs.append("target: \(target!)")
                    targetSlice = targetSlice.dropFirst()
                    break
                case .orderedSame:
                    sourceSlice = sourceSlice.dropFirst()
                    targetSlice = targetSlice.dropFirst()
                    break
                }
            }
            else if source != nil {
                diffs.append("source: \(source!)")
                sourceSlice = sourceSlice.dropFirst()
            }
            else if target != nil {
                diffs.append("target: \(target!)")
                targetSlice = targetSlice.dropFirst()
            }
            
        } while source != nil || target != nil
        
        return diffs.count > 0 ? diffs : nil
    }
}
