//
//  LKFMetaDataMel.swift
//  LKF
//
//  Created by Alexander Holzer on 14.01.22.
//  Copyright © 2022 Alexander Holzer. All rights reserved.
//

import Foundation
@_implementationOnly import LibLKF

public class LKFMetaDataMel {
    
    public enum Fields : UInt32 {
        case Mel = 12
        case StatGroup = 4
        case AmbGroup = 5
        case Seite = 7
        case Geschl = 8
        case Text = 20
        case WarnCount = 41
        case ErrorCount = 42
        case DayWarnCount = 43
        case DayErrorCount = 44
        case AmbWarnCount = 45
        case AmbErrorCount = 46
    }
    
    let context : LKFContext
    let melIndex: UInt32
    
    public init (context: LKFContext, melIndex: UInt32) {
        self.context = context
        self.melIndex = melIndex
    }
    
    public func value (for field:LKFMetaDataMel.Fields) -> String {
        if let ptr = context.ptr {
            let lineBuffer = UnsafeMutablePointer<CChar>.allocate(capacity:256)
            LibLKF.lkf_stamm_getMelField (ptr, melIndex, field.rawValue, lineBuffer, 256)
            let r = String(cString: lineBuffer).trimmingCharacters(in: .whitespaces)
            lineBuffer.deallocate()
            return r
        }
        return ""
     }

    public func value (for field:LKFMetaDataMel.Fields, encoding: String.Encoding) -> String {
        if let ptr = context.ptr {
            let lineBuffer = UnsafeMutablePointer<CChar>.allocate(capacity:256)
            LibLKF.lkf_stamm_getMelField (ptr, melIndex, field.rawValue, lineBuffer, 256)
            if let r = String(cString: lineBuffer, encoding: encoding) {
                lineBuffer.deallocate()
                return r.trimmingCharacters(in: .whitespaces)
            }
            else {
                lineBuffer.deallocate()
            }
        }
        return ""
     }

    
    public var Mel : String {
        return value (for: .Mel)
    }
    
    public var StatGroupName : String {
        return value (for: .StatGroup)
    }
    
    public var AmbGroupName : String {
        return value (for: .AmbGroup)
    }
    
    public var Text: String {
        return value (for: .Text, encoding: .ascii)
    }
    
    public lazy var ambGroup : LKFMetaDataGroup? = { () -> LKFMetaDataGroup? in
        let g = AmbGroupName
        if g.count > 0 {
            return LKFMetaDataGroup (context: context, group: g)
        }
        return nil
    }()

    public lazy var statGroup : LKFMetaDataGroup? = { () -> LKFMetaDataGroup? in
        let g = StatGroupName
        if g.count > 0 {
            return LKFMetaDataGroup (context: context, group: g)
        }
        return nil
    }()
    
    public func getMELDiags() -> [String] {
        guard let ptr = context.ptr else {
            return []
        }
        
        let lineBuffer = UnsafeMutablePointer<CChar>.allocate(capacity:16384)
        if LibLKF.lkf_stamm_getMelDiags (ptr, melIndex, lineBuffer, 16384) > 0 {
            if let r = String(cString: lineBuffer, encoding: .utf8) {
                lineBuffer.deallocate()
                return r.split(separator: ";").map { String($0) }
            }
        }
        lineBuffer.deallocate()
        return []
    }
    
    public func getMELMEL() -> [String] {
        guard let ptr = context.ptr else {
            return []
        }
        
        let lineBuffer = UnsafeMutablePointer<CChar>.allocate(capacity:16384)
        if LibLKF.lkf_stamm_getMelMel (ptr, melIndex, lineBuffer, 16384) > 0 {
            if let r = String(cString: lineBuffer, encoding: .utf8) {
                lineBuffer.deallocate()
                return r.split(separator: ";").map { String($0) }
            }
        }
        lineBuffer.deallocate()
        return []
    }

}
