/****************************************************************************
** Leistungsorientierte Krankenanstaltenfinanzierung (LKF)                 **
** Scoring-C-Sourcen V2024.0                                               **
** (C) 2023 BMSGPK                                                         **
*****************************************************************************
** scoring5.c - LKF-Punkteberechnung                                       **
**                                                                         **
**		 Alex.Holzer@chello.at                                             **
**       Bernhard.Pesec@dothealth.at                                       **
***************************************************************************/
#include "lkfint5.h"

typedef char SCORING_MELSTRING[LKF_AUFENTHALT_NEWMEL_LEN];
typedef char SCORING_STRING4[4];
typedef char SCORING_STRING2[2];

#define IS_TRACE_INFO(pContext) (lkf_isTraceInfo (pContext, LKF_FAC_MBDS))
#define IS_TRACE_DETAIL(pContext) (lkf_isTraceDetail (pContext, LKF_FAC_MBDS))
#define IS_TRACE_INPUT(pContext) (lkf_isTraceInput (pContext, LKF_FAC_MBDS))

#define TRACE_INFO(pAufenthalt,nAbtFC,sLine) (lkf_addTraceAufenthaltLine ((pAufenthalt)->pScoringContext, pAufenthalt, nAbtFC, sLine, LKF_FAC_MBDS, LKF_TRACE_INFO)) 
#define TRACE_DETAIL(pAufenthalt,nAbtFC,sLine) (lkf_addTraceAufenthaltLine ((pAufenthalt)->pScoringContext, pAufenthalt, nAbtFC, sLine, LKF_FAC_MBDS, LKF_TRACE_DETAIL))

static int isGrpMel (MEL_ARRAY_STRUC *pMel)
{

	if (!pMel)
		return 0;

	return !CompareUCase (pMel->sGruppe, "MEL", 3);
}

static int isGrpMelNr2009 (LKF_AUFENTHALT *pAufenthalt, int nMelId)
{
	MEL_ARRAY_STRUC *pMel;
	
	pMel = FindMel (pAufenthalt->pScoringContext, pAufenthalt->mel[nMelId].MEL);
	
	if (pMel)
		return isGrpMel (pMel);
	else
		return 0;
}

static int scoring_split_2009 (LKF_AUFENTHALT *pAufenthalt, const LKF_KNOTEN *pKnoten, int act_knoten, const char *act_subgruppe, int *agr01, const char *hd, int nMelId, int bTraceDetail)
{
	int val, val2;
	int i, j;

	int erfuellt;
	int agr01_bd;
	int agr01_found;

	int act_typ;
	int pat_grp;
	char tmp_spez_list[20];

	int splitmel;
	size_t comp_len;
	const SPEZ_LIST_STRING *pSpez;
    char c = 0;

	MEL_ARRAY_STRUC *pMel;
	ICD_LIST_STRING *pDiag;

	char act_grplgr[6];
	char sSplitDetail[128];

	char sMel[6];

	sSplitDetail[0] = 0;

	act_typ = pKnoten->merkmals_typ[act_knoten];
	erfuellt = 0;
	val = 0;

	if (act_typ < 100)
	{
		switch (act_typ)
		{
			case  1: /* SPEZMEL */
				splitmel = 1;
				memcpy (sMel, pKnoten->splitmerkmal[act_knoten], LKF_AUFENTHALT_NEWMEL_LEN);
				sMel[LKF_AUFENTHALT_NEWMEL_LEN] = 0;
				
				if (bTraceDetail) sprintf (sSplitDetail, "SPEZMEL: MEL=%s", sMel);

				for (i = 0; i < (int)pAufenthalt->mel_anz; i++) {

					pMel = FindMel (pAufenthalt->pScoringContext, pAufenthalt->mel[i].MEL);

					if (!pMel || (i != nMelId && isGrpMel (pMel)))
						continue;

					if (!memcmp (pAufenthalt->mel[i].MEL, sMel, LKF_AUFENTHALT_NEWMEL_LEN)) {
					
						for (j = 0; j < (int) pAufenthalt->pat_subgruppen; j++) {

							if (!memcmp (pAufenthalt->mel[i].MEL, pAufenthalt->subgruppe[j].gruppe + 8, LKF_AUFENTHALT_NEWMEL_LEN)) {
							
								if (memcmp (pAufenthalt->mel[i].MEL, act_subgruppe + 8, LKF_AUFENTHALT_NEWMEL_LEN)) splitmel = 0;
								j = (int) pAufenthalt->pat_subgruppen;
							
							}
						
						}
						if (splitmel) erfuellt = 1;
					}
				}
				break;
			case  2: /* SPEZHDG4 */
				if (bTraceDetail) sprintf (sSplitDetail, "SPEZHDG4: Diagnose=%5.5s", pKnoten->splitmerkmal[act_knoten]);
				if (!memcmp (pKnoten->splitmerkmal[act_knoten], hd, 5)) erfuellt = 1;
				break;
			case  3: /* SPEZHDG */
				if (bTraceDetail) sprintf (sSplitDetail, "SPEZHDG: Diagnose=%3.3s", pKnoten->splitmerkmal[act_knoten]);

				if (!memcmp (hd, pKnoten->splitmerkmal[act_knoten], 3)) erfuellt = 1;
				break;
			case  4: /* SPEZDG4 */
			case  5: /* SPEZDG */
				if (bTraceDetail) 
				{
					if (act_typ == 5)
					{
						sprintf (sSplitDetail, "SPEZDG: Diagnose=%3.3s", pKnoten->splitmerkmal[act_knoten]);
					}
					else
					{
						sprintf (sSplitDetail, "SPEZDG4: Diagnose=%5.5s", pKnoten->splitmerkmal[act_knoten]);
					}
				}

				if (act_typ == 5) comp_len = 3;
				else comp_len = 5;

				if (!memcmp (pAufenthalt->HD, pKnoten->splitmerkmal[act_knoten], comp_len)) erfuellt = 1;
				else {
					for (i = 0; !erfuellt && i < (int) pAufenthalt->diag_anz; i++) {
						if (!memcmp (pAufenthalt->diag[i].DIAG, pKnoten->splitmerkmal[act_knoten], comp_len)) erfuellt = 1;
					}
				}
				break;

			case  6: /* ALTER */
                GetInt (pKnoten->splitmerkmal[act_knoten], 5, &val);
				if (bTraceDetail) sprintf (sSplitDetail, "ALTER: Vergleichswert=%d", pKnoten->Intervall[0][((val<0)?-val:val) - 1]);

				if (val < 0) {
					if (pAufenthalt->Alter <= pKnoten->Intervall[0][-val - 1]) erfuellt = 1;
				}
				else if (pAufenthalt->Alter > pKnoten->Intervall[0][val - 1]) erfuellt = 1;

				break;
			case  7: /* AGR01 */
			case 12: /* AGR02 */
				if (bTraceDetail) sprintf (sSplitDetail, (act_typ == 7 ? "AGR01" : "AGR02"));

				agr01_bd = 0;
				agr01_found = 0;
				for (i = 0; i < (int) pAufenthalt->abt_anz; i++) {
					if (!IsBlank (pAufenthalt->abt[i].ABTFC, LKF_AUFENTHALT_ABTFC_LEN)) {
					
						sprintf (tmp_spez_list,".....%4.4s%8.8s", pAufenthalt->KANR, pAufenthalt->abt[i].ABTFC);
						
						pSpez = SpezInArray (pAufenthalt->pScoringContext, tmp_spez_list);
						if (pSpez && !ABTFC_Spez_Check (pAufenthalt, i, pSpez, 0)) pSpez = 0;
						if (pSpez) {
						
							switch ((*pSpez)[0]) {
							
								default:	pSpez = 0;
											break;

								case 'P':   if ((*pSpez)[27] != 'A') pSpez = 0;
											else if (act_typ == 7 && (*pSpez)[28] != '1') pSpez = 0;
											else if (act_typ == 12 && (*pSpez)[28] != '2') pSpez = 0;
											break;
								case 'J':   

											if (pAufenthalt->pScoringContext->modelljahr >= 2013)
											{ /* V17.0 */
												pSpez = 0;
											}
											else
											{
												if (act_typ == 12) pSpez = 0;
												else if ((*pSpez)[27] == ' ') c = (*pSpez)[28];
												else if ((*pSpez)[28] == ' ') c = (*pSpez)[27];
												else c = 0;
												if (pSpez && (c == 'I' || c == 'R')) pSpez = 0;
											}
											break;

								case 'O':   /* V15.0 */
                                            if ((*pSpez)[27] != '0') pSpez = 0;
                                            else if (act_typ == 7 && (*pSpez)[28] != '1') pSpez = 0;
                                            else if (act_typ == 12 && (*pSpez)[28] != '2') pSpez = 0;
											break;
								case 'A':	
											if (pAufenthalt->pScoringContext->modelljahr < 2010 || act_typ != 7)
												pSpez = 0;
											break;
							}
						}

						if (pSpez) {
							agr01_bd += pAufenthalt->abt[i].ABTVD;
							agr01_found = 1;
						}
					}
				}

				if (agr01_bd > pAufenthalt->GESVD / 2 ||
					(agr01_found && agr01_bd == 0 && pAufenthalt->GESVD == 0)) {
					*agr01 = 1;
					erfuellt = 1;
				}
				break;
		}
	}

	else if (act_typ >= 101 && act_typ <= 199)
	{
		sprintf (act_grplgr, "HGR%02d", act_typ - 100);
		pat_grp = 0;

		pDiag = FindDiag (pAufenthalt->pScoringContext->pStamm2001, hd);

		if (pDiag && !memcmp ((*pDiag) + 18, act_grplgr, 5)) pat_grp = 1;

		GetInt (pKnoten->splitmerkmal[act_knoten], 5, &val);
		if (bTraceDetail) sprintf (sSplitDetail, "%-8.8s: PATGRP: %d", act_grplgr, pat_grp); 
		if (val < 0) {
			if (pat_grp <= pKnoten->Intervall[1][-val - 1]) erfuellt = 1;
		}
		else if (pat_grp > pKnoten->Intervall[1][val - 1]) erfuellt = 1;
	}
	else if (act_typ <= 299)
	{
		sprintf (act_grplgr, "DGR%02d", act_typ - 200);
		pat_grp = 0;

		if (bTraceDetail) sprintf (sSplitDetail, "%s", act_grplgr);

		for (i = 0; i < (int) pAufenthalt->diag_anz + 1; i++) {
		
			if (!i) 
				pDiag = FindDiag (pAufenthalt->pScoringContext->pStamm2001, pAufenthalt->HD);
			else
				pDiag = FindDiag (pAufenthalt->pScoringContext->pStamm2001, pAufenthalt->diag[i-1].DIAG);


			if (pDiag && !memcmp ((*pDiag) + 24, act_grplgr, 5)) ++pat_grp;
			
		}

		GetInt (pKnoten->splitmerkmal[act_knoten], 5, &val2);

		if (val2 < 0) {
			if (pat_grp <= pKnoten->Intervall[1][-val2 -1]) erfuellt = 1;
		}
		else {
			if (pat_grp > pKnoten->Intervall[1][val2 - 1]) erfuellt = 1;
		}
	}
	else if (act_typ <= 399)
	{
		sprintf (act_grplgr, "LGR%02d", act_typ - 300);

		pat_grp = 0;

		for (i = 0; i < (int) pAufenthalt->mel_anz; i++)
		{
			pMel = FindMel (pAufenthalt->pScoringContext, pAufenthalt->mel[i].MEL);	
			
			if (!pMel)
				continue; 

			if (i != nMelId && isGrpMel (pMel))
				continue;

			if (!memcmp (pMel->sLGR, act_grplgr, 5)) {
				for (j = 0; j < i && (isGrpMelNr2009 (pAufenthalt, j) || memcmp (pAufenthalt->mel[i].MEL, pAufenthalt->mel[j].MEL, LKF_AUFENTHALT_NEWMEL_LEN)); j++) ; /* V14.2 */
				if (j == i) pat_grp++;
			}
		}

		GetInt (pKnoten->splitmerkmal[act_knoten], 5, &val2);
		if (bTraceDetail) sprintf (sSplitDetail, "%-8.8s: PATGRP: %d", act_grplgr, pat_grp); 
		if (val2 < 0) {
			if (pat_grp <= pKnoten->Intervall[1][-val2 - 1]) erfuellt = 1;
		}
		else {
			if (pat_grp > pKnoten->Intervall[1][val2 - 1]) erfuellt = 1;
		}
	}

	return erfuellt;
}

static int scoring_split_isMinus (const char *buf) {
    return (buf[0]=='-' || buf[1]=='-' || buf[2]=='-' || buf[3]=='-');
        
}

static int scoring_afg_split(LKF_AUFENTHALT *pAufenthalt, const char *grp) 
{
	unsigned int i;

    if (pAufenthalt->abt_anz) {
        for (i = 0; i < pAufenthalt->abt_anz; i++)
        {
            FUNK_LIST_STRING *funk = FunkInArray(pAufenthalt->pScoringContext->pStamm2001, pAufenthalt->abt[i].ABTFC);
            if (!funk || memcmp(((const char *)funk) + 107, grp, 5))
                return 0;
        }
        return 1;
    }
    return 0; /* V2021.2 - Quickscore ohne Abt trifft Kriterium nicht */
}

static int scoring_split_2017 (LKF_AUFENTHALT *pAufenthalt, const LKF_KNOTEN *pKnoten, int act_knoten, const char *act_subgruppe, int *agr01, const char *hd, int nMelId, int bTraceDetail)
{ /* V21.0 - neue Funktion fuer Modelle 2017+ */
    int val, val2;
    int i, j;
    
    int erfuellt;
    int agr01_bd;
    int agr01_found;
    
    int act_typ;
    int pat_grp;
    char tmp_spez_list[20];
    
    int splitmel;
    size_t comp_len;
    const SPEZ_LIST_STRING *pSpez;
    char c = 0;
    
    MEL_ARRAY_STRUC *pMel;
    ICD_LIST_STRING *pDiag;
    
    char act_grplgr[6];
    char sSplitDetail[128];
    
    char sMel[6];
    
    sSplitDetail[0] = 0;
    
    act_typ = pKnoten->merkmals_typ[act_knoten];
    erfuellt = 0;
    val = 0;
    
    if (act_typ < 100)
    {
        switch (act_typ)
        {
            case  1: /* SPEZMEL */
                splitmel = 1;
                memcpy (sMel, pKnoten->splitmerkmal[act_knoten], LKF_AUFENTHALT_NEWMEL_LEN);
                sMel[LKF_AUFENTHALT_NEWMEL_LEN] = 0;
                
                if (bTraceDetail) sprintf (sSplitDetail, "SPEZMEL: MEL=%s", sMel);
                
                for (i = 0; i < (int)pAufenthalt->mel_anz; i++) {
                    
                    pMel = FindMel (pAufenthalt->pScoringContext, pAufenthalt->mel[i].MEL);
                    
                    if (!pMel || (i != nMelId && isGrpMel (pMel)))
                        continue;
                    
                    if (!memcmp (pAufenthalt->mel[i].MEL, sMel, LKF_AUFENTHALT_NEWMEL_LEN)) {
                        
                        for (j = 0; j < (int) pAufenthalt->pat_subgruppen; j++) {
                            
                            if (!memcmp (pAufenthalt->mel[i].MEL, pAufenthalt->subgruppe[j].gruppe + 8, LKF_AUFENTHALT_NEWMEL_LEN)) {
                                
                                if (memcmp (pAufenthalt->mel[i].MEL, act_subgruppe + 8, LKF_AUFENTHALT_NEWMEL_LEN)) splitmel = 0;
                                j = (int) pAufenthalt->pat_subgruppen;
                                
                            }
                            
                        }
                        if (splitmel) erfuellt = 1;
                    }
                }
                break;
            case  2: /* SPEZHDG4 */
                if (bTraceDetail) sprintf (sSplitDetail, "SPEZHDG4: Diagnose=%5.5s", pKnoten->splitmerkmal[act_knoten]);
                
                
                for (i = 0; !erfuellt && i < (int) pAufenthalt->diag_anz; i++) {
                    if (!pAufenthalt->diag[i].isHD)
                        continue;
                    if (!memcmp (pKnoten->splitmerkmal[act_knoten], pAufenthalt->diag[i].DIAG, 5)) erfuellt = 1;
                }
                
                break;
            case  3: /* SPEZHDG */
                if (bTraceDetail) sprintf (sSplitDetail, "SPEZHDG: Diagnose=%3.3s", pKnoten->splitmerkmal[act_knoten]);

                for (i = 0; !erfuellt && i < (int) pAufenthalt->diag_anz; i++) {
                    if (!pAufenthalt->diag[i].isHD)
                        continue;
                    if (!memcmp (pAufenthalt->diag[i].DIAG, pKnoten->splitmerkmal[act_knoten], 3)) erfuellt = 1;
                }

                break;
            case  4: /* SPEZDG4 */
            case  5: /* SPEZDG */
                if (bTraceDetail)
                {
                    if (act_typ == 5)
                    {
                        sprintf (sSplitDetail, "SPEZDG: Diagnose=%3.3s", pKnoten->splitmerkmal[act_knoten]);
                    }
                    else
                    {
                        sprintf (sSplitDetail, "SPEZDG4: Diagnose=%5.5s", pKnoten->splitmerkmal[act_knoten]);
                    }
                }
                
                if (act_typ == 5) comp_len = 3;
                else comp_len = 5;
             
                for (i = 0; !erfuellt && i < (int) pAufenthalt->diag_anz; i++) {
                    if (!memcmp (pAufenthalt->diag[i].DIAG, pKnoten->splitmerkmal[act_knoten], comp_len)) erfuellt = 1;
                }
                break;
                
            case  6: /* ALTER */
                GetInt (pKnoten->splitmerkmal[act_knoten], 5, &val);
                if (bTraceDetail) sprintf (sSplitDetail, "ALTER: Vergleichswert=%d", val);
                
                if (val == 0) {
                    const char *merkmal = pKnoten->splitmerkmal[act_knoten];
                    if (scoring_split_isMinus(merkmal)) {
                        if (pAufenthalt->Alter == 0) erfuellt = 1;
                    }
                    else if (pAufenthalt->Alter > 0) erfuellt = 1;
                }
                else if (val < 0) {
                    if (pAufenthalt->Alter <= -val) erfuellt = 1;
                }
                else if (pAufenthalt->Alter > val) erfuellt = 1;
                
                break;
            case  7: /* AGR01 */
            case 12: /* AGR02 */
                if (bTraceDetail) sprintf (sSplitDetail, (act_typ == 7 ? "AGR01" : "AGR02"));
                
                agr01_bd = 0;
                agr01_found = 0;
                for (i = 0; i < (int) pAufenthalt->abt_anz; i++) {
                    if (!IsBlank (pAufenthalt->abt[i].ABTFC, LKF_AUFENTHALT_ABTFC_LEN)) {
                        
                        sprintf (tmp_spez_list,".....%4.4s%8.8s", pAufenthalt->KANR, pAufenthalt->abt[i].ABTFC);
                        
                        pSpez = SpezInArray (pAufenthalt->pScoringContext, tmp_spez_list);
                        if (pSpez && !ABTFC_Spez_Check (pAufenthalt, i, pSpez, 0)) pSpez = 0;
                        if (pSpez) {
                            
                            switch ((*pSpez)[0]) {
                                    
                                default:	pSpez = 0;
                                    break;
                                    
                                case 'P':   if ((*pSpez)[27] != 'A') pSpez = 0;
                                else if (act_typ == 7 && (*pSpez)[28] != '1') pSpez = 0;
                                else if (act_typ == 12 && (*pSpez)[28] != '2') pSpez = 0;
                                    break;
                                case 'J':
                                    
                                    if (pAufenthalt->pScoringContext->modelljahr >= 2013)
                                    { /* V17.0 */
                                        pSpez = 0;
                                    }
                                    else
                                    {
                                        if (act_typ == 12) pSpez = 0;
                                        else if ((*pSpez)[27] == ' ') c = (*pSpez)[28];
                                        else if ((*pSpez)[28] == ' ') c = (*pSpez)[27];
                                        else c = 0;
                                        if (pSpez && (c == 'I' || c == 'R')) pSpez = 0;
                                    }
                                    break;
                                    
                                case 'O':   /* V15.0 */
                                    if (pAufenthalt->pScoringContext->modelljahr >= 2022) {
                                        pSpez = 0;
                                    }
                                    else {
                                        if ((*pSpez)[27] != '0') pSpez = 0;
                                        else if (act_typ == 7 && (*pSpez)[28] != '1') pSpez = 0;
                                        else if (act_typ == 12 && (*pSpez)[28] != '2') pSpez = 0;
                                    }
                                    break;
                            }
                        }
                        
                        if (pSpez) {
                            agr01_bd += pAufenthalt->abt[i].ABTVD;
                            agr01_found = 1;
                        }
                    }
                }
                
                if (agr01_bd > pAufenthalt->GESVD / 2 ||
                    (agr01_found && agr01_bd == 0 && pAufenthalt->GESVD == 0)) {
                    *agr01 = 1;
                    erfuellt = 1;
                }
                break;
        }
    }
    
    else if (act_typ >= 101 && act_typ <= 199)
    {
        sprintf (act_grplgr, "HG%03d", act_typ);
        pat_grp = 0;
        
        pDiag = FindDiag (pAufenthalt->pScoringContext->pStamm2001, hd);
        
        if (pDiag && !memcmp ((*pDiag) + 18, act_grplgr, 5)) pat_grp = 1;
        
        GetInt (pKnoten->splitmerkmal[act_knoten], 5, &val);
        if (bTraceDetail) sprintf (sSplitDetail, "%-8.8s: PATGRP: %d", act_grplgr, pat_grp); 
        if (val == 0) {
            if (pat_grp == 0) erfuellt = 1;
        }
        else {
            if (pat_grp > 0) erfuellt = 1;
        }
    }
    else if (act_typ > 300 && act_typ <= 400)
    { /* V21.0 - Ambulant */
        if (nMelId >= 0) {
            sprintf (act_grplgr, "ALG%02d", act_typ - 300);
            pMel = FindMel (pAufenthalt->pScoringContext, pAufenthalt->mel[nMelId].MEL);
            if (!memcmp (pMel->sAmbLeistGrp, act_grplgr, 5)) {
                erfuellt = 1;
            }
        }
    }
    else if (act_typ > 400 && act_typ <= 500)
    { /* V21.0 - Ambulant */
        sprintf (act_grplgr, "AFG%02d", act_typ - 400);
		erfuellt = scoring_afg_split(pAufenthalt, act_grplgr);
    }
    else if (act_typ <= 699)
    {
        sprintf (act_grplgr, "LG%03d", act_typ - 400);
        
        pat_grp = 0;
        
        for (i = 0; i < (int) pAufenthalt->mel_anz; i++)
        {
            pMel = FindMel (pAufenthalt->pScoringContext, pAufenthalt->mel[i].MEL);	
            
            if (!pMel)
                continue; 
            
            if (i != nMelId && isGrpMel (pMel))
                continue;
            
            if (!memcmp (pMel->sLGR, act_grplgr, 5)) {
                for (j = 0; j < i && (isGrpMelNr2009 (pAufenthalt, j) || memcmp (pAufenthalt->mel[i].MEL, pAufenthalt->mel[j].MEL, LKF_AUFENTHALT_NEWMEL_LEN)); j++) ;
                if (j == i) pat_grp++;
            }
        }
        
        GetInt (pKnoten->splitmerkmal[act_knoten], 5, &val2);
        if (bTraceDetail) sprintf (sSplitDetail, "%-8.8s: PATGRP: %d", act_grplgr, pat_grp); 

        if (val2 == 0) {
            if (pat_grp == 0) erfuellt = 1;
        }
        else {
            if (pat_grp > 0) erfuellt = 1;
        }
    }
    
    return erfuellt;
}

static int compare_subgruppe_meldat (const void *pv1, const void *pv2)
{
	LKF_AUFENTHALT_SUBGRUPPE *p1, *p2;
	p1 = (LKF_AUFENTHALT_SUBGRUPPE *) pv1;
	p2 = (LKF_AUFENTHALT_SUBGRUPPE *) pv2;

    if (p1->lMelDat == p2->lMelDat) {
        if (p1->mel_id == p2->mel_id) return 0;
        return p1->mel_id > p2->mel_id ? 1 : -1;
    }
		
	else
		return p1->lMelDat > p2->lMelDat ? 1 : -1;

}

static void SortSubgruppeMelDat (unsigned int nCount, LKF_AUFENTHALT_SUBGRUPPE *pSubgruppe)
{
	qsort (pSubgruppe, nCount, sizeof(LKF_AUFENTHALT_SUBGRUPPE), compare_subgruppe_meldat);
}

_SCORING_EXPORT int lkf_scoring_abortable (LKF_CONTEXT *pContext, LKFEXECUTION_DELEGATE cb, unsigned int stepCount) {
    unsigned int i;

    pContext->scor_work_count = 0;
    pContext->scor_count = 0;

        
    for (i = 0; i < pContext->patient_anz; i++) {
        // error handling!
        i += lkf_context_scoring(pContext, stepCount);
        if (cb(pContext, "scoring", i)) {
            return -99;
        }
    }
        
    return (int) i;

}

_SCORING_EXPORT int lkf_context_scoring2 (LKF_CONTEXT *pContext, unsigned int anzahl, int reset) {
    if (reset) {
        pContext->scor_work_count = 0;
        pContext->scor_count = 0;
    }
    return lkf_context_scoring(pContext, anzahl);
}


_SCORING_EXPORT int lkf_context_scoring (LKF_CONTEXT *pContext, unsigned int anzahl)
{
	LKF_AUFENTHALT *pAufenthalt;
	int errorcode;
	unsigned int i;

    if (anzahl == 0) pContext->scor_work_count = 0;
	
    if (!pContext->scor_count) {
		for (i = 0; i < pContext->ka_gesamt; i++) lkf_init_ka (pContext, pContext->ka + i, 0, 0);
		
		lkf_context_sort_private_stamm (pContext);
	}
		
	for (i = 0; (!anzahl || i < anzahl) && pContext->scor_work_count < pContext->patient_anz; i++) { 

		pAufenthalt = pContext->ppAufenthalt[pContext->scor_work_count];
		++pContext->scor_work_count;
		if (!(pAufenthalt->mark & 1)) continue;

		errorcode = lkf_scoring (pContext, pAufenthalt);

		++pContext->scor_count;
		lkf_add_ka_stats (pContext, pAufenthalt, 4);


		if (errorcode) return errorcode;
		
	}
	return (int) i;

}

static void lkf_scoring_tkl_subgruppe (LKF_CONTEXT *pContext, LKF_AUFENTHALT *pAufenthalt, LKF_AUFENTHALT_SUBGRUPPE *pSubgruppe)
{ /* Modell 2012 - 2016 */
	unsigned int i;
	double tk_faktor;
	MEL_ARRAY_STRUC *pMel;
	int TKLMel;
	char tmp_mel_ka[12];
	long orig_scutk;

	TKLMel = 0;
	pSubgruppe->TKLModus = 0;
	if (pAufenthalt->GESVD == 0 && pAufenthalt->ENTART != 'S' && pAufenthalt->ENTART != 'T' && pAufenthalt->ENTART != LKF_ENTART_REVERS /* V18.0 */ &&
		pAufenthalt->SCSPEZ == 0 && memcmp (pSubgruppe->gruppe, "MEL22", 5)) {

		for (i = 0; i < pAufenthalt->mel_anz && !TKLMel; i++) {
		
			pMel = FindMel (pContext, pAufenthalt->mel[i].MEL);

			if (pMel && pMel->cTKL== 'J') {
				sprintf (tmp_mel_ka, "%4.4s %5.5s", pAufenthalt->KANR, pAufenthalt->mel[i].MEL);
				if (!GetMELKATKLContext (pContext, tmp_mel_ka)) TKLMel = 1;	
			}
		}
		if (TKLMel)
		{
			tk_faktor = 2;
			pSubgruppe->TKLModus = 2;
		}
		else
		{
            tk_faktor = 0.05;
            pSubgruppe->TKLModus = 1;
		}
	}
	else tk_faktor = 1;

	orig_scutk = pSubgruppe->SCUTK;
	pSubgruppe->SCUTK = simCLng (pSubgruppe->SCUTK_DOUBLE * tk_faktor);
	pSubgruppe->SCGES += pSubgruppe->SCUTK;
	pSubgruppe->SCGES -= orig_scutk;
}



static void lkf_scoring_tkl (LKF_CONTEXT *pContext, LKF_AUFENTHALT *pAufenthalt)
{
	unsigned int i;
	double tk_faktor;
	MEL_ARRAY_STRUC *pMel;
	LKF_AUFENTHALT_SUBGRUPPE *pSubgruppe;
	int TKLMel;
	char tmp_mel_ka[12];

	TKLMel = 0;
	pAufenthalt->TKLModus = 0;

	if (pAufenthalt->GESVD == 0 && pAufenthalt->ENTART != 'S' && pAufenthalt->ENTART != 'T' && pAufenthalt->ENTART != LKF_ENTART_REVERS &&
		pAufenthalt->SCSPEZ == 0 && memcmp (pAufenthalt->GRP, "MEL22", 5)) {

        for (i = 0; i < pAufenthalt->mel_anz && !TKLMel; i++) {
            
            pMel = FindMel (pContext, pAufenthalt->mel[i].MEL);
            
            if (pMel && pMel->cTKL== 'J') {
                sprintf (tmp_mel_ka, "%4.4s %5.5s", pAufenthalt->KANR, pAufenthalt->mel[i].MEL);
                if (!GetMELKATKLContext (pContext, tmp_mel_ka)) TKLMel = 1;
            }
        }

        if (TKLMel)
		{
            tk_faktor = 2;
			pAufenthalt->TKLModus = 2;
		}
		else
		{
			switch (pContext->modelljahr)
			{
				case 2010:
					tk_faktor = 0.05;	
					break;	
				case 2009:
					tk_faktor = 0.1;
					break;
				default:
					tk_faktor = 0.5;
					break;
			}
			pAufenthalt->TKLModus = 1;
		}
	}
	else tk_faktor = 1;

	pAufenthalt->SCUTK_TKL = simCLng (pAufenthalt->SCUTK_DOUBLE * tk_faktor);

	pAufenthalt->SCGES_TKL = pAufenthalt->LDFPTK + pAufenthalt->LDFPLK + pAufenthalt->SCUTK_TKL +
				  pAufenthalt->SCULK + pAufenthalt->VDZU + pAufenthalt->INTZU + pAufenthalt->MELZU + pAufenthalt->SCSPEZ;

	if (tk_faktor != 1)
	{
	
		long orig_scutk;

		for (i = 0; i < pAufenthalt->pat_subgruppen; i++) {
			pSubgruppe = pAufenthalt->subgruppe+i;
			orig_scutk = pSubgruppe->SCUTK;
			if (tk_faktor != 1) 
			{
                pSubgruppe->SCUTK = simCLng (pSubgruppe->SCUTK_DOUBLE * tk_faktor);
				pSubgruppe->SCGES += pSubgruppe->SCUTK;
				pSubgruppe->SCGES -= orig_scutk;
			}

		}
	}
}

int lkf_quickscore_compute_abtdats (LKF_AUFENTHALT *pAufenthalt)
{
	int gesvd, i, abtvd, limit, abtsum;

	if (pAufenthalt->abt_anz) {

		gesvd = (int) pAufenthalt->GESVD;
		abtsum = 0;
			
		for (i = (int) pAufenthalt->abt_anz - 1; i >= 0 && pAufenthalt->abt[i].ABTDAT[0] == '#'; i--) {
			GetInt (pAufenthalt->abt[i].ABTDAT + 1, 7, &abtvd);
			pAufenthalt->abt[i].ABTVD = abtvd;
			abtsum += abtvd;
		}

		if (i >= 0) {

			if (gesvd < 0) limit = i + 1;
			else limit = i;

			for (i = 0; i < limit; i++) {
				pAufenthalt->abt[i].ABTVD = Util_Diff_Tage(pAufenthalt, (short) i);
				abtsum += pAufenthalt->abt[i].ABTVD;
			}
			if (gesvd < 0) pAufenthalt->GESVD = abtsum;
			else {
				gesvd -= abtsum;
				if (gesvd < 0) {
					pAufenthalt->abt[limit].ABTVD = 0;
					pAufenthalt->GESVD -= gesvd;
				}
				else pAufenthalt->abt[limit].ABTVD = gesvd;
			}
		}
		else if (abtsum > pAufenthalt->GESVD) pAufenthalt->GESVD = abtsum;
	}
	return 0;
}

static void InactivateMel (LKF_AUFENTHALT *pAufenthalt, unsigned int i)
{
    memcpy (pAufenthalt->mel[i].MEL, "00000", LKF_AUFENTHALT_NEWMEL_LEN);
	memcpy (pAufenthalt->mel[i].MELANZ, "   0", 4);
}

static int isDuring19Aufenthalt (LKF_AUFENTHALT *pAufenthalt, int meldat)
{
	unsigned int i;
	int TageSeitAUFDAT;
	
	if (!meldat || !pAufenthalt->lAufDat) return 0; 
	TageSeitAUFDAT = (int) (meldat - pAufenthalt->lAufDat);


	for (i = 0; i < pAufenthalt->abt_anz; i++) {
		if (TageSeitAUFDAT < pAufenthalt->abt[i].ABTVD) {
			if (!memcmp (pAufenthalt->abt[i].ABTFC, "19", 2)) return 1;
		}
		else if (TageSeitAUFDAT == pAufenthalt->abt[i].ABTVD) {
			/* am Verlegungstag zhlen zwei Abteilungen */
			if (memcmp (pAufenthalt->abt[i].ABTFC, "19", 2)) return 0;
		}
		else TageSeitAUFDAT -= pAufenthalt->abt[i].ABTVD;
	}
	return 0;
}

static int mel_during_abt19 (LKF_AUFENTHALT *pAufenthalt, int id)
{
	MEL_ARRAY_STRUC *pMel;

    pMel = FindMel (pAufenthalt->pScoringContext, pAufenthalt->mel[id].MEL);

	if (pMel && isDuring19Aufenthalt (pAufenthalt, pAufenthalt->mel[id].meldat_nr))
		return 1;
	return 0;
}


_SCORING_EXPORT int lkf_scoring (LKF_CONTEXT *pContext, LKF_AUFENTHALT *pAufenthalt)
{

	int scoring_return;
	
	if (!pAufenthalt) {
		pContext->scor_count = 0;
		pContext->scor_work_count = 0;
		return lkf_context_scoring (pContext, 0);
	}

	scoring_return = lkf_scoring_compute (pContext, pAufenthalt);
	
	lkf_trace_finished (pContext);

	return scoring_return;
}

static int compute_dates (LKF_AUFENTHALT *pAufenthalt)
{
	int  i;
	long nEntDat;
	long nAufDat;
    long nCurDat = 0;
	long nNextDat;

    nAufDat = lkf_dat2int (pAufenthalt->AUFDAT);

    if (pAufenthalt->isAmb) {
        nEntDat = nAufDat;
    }
	else if (pAufenthalt->ENTART == 'V')
	{
		char sLastDate[10];
		sprintf (sLastDate, "%4d1231", pAufenthalt->pScoringContext->cmb_jahr);
		nEntDat = lkf_dat2int (sLastDate);
	}
	else
		nEntDat = lkf_dat2int (pAufenthalt->ENTDAT);

	
	if (!nEntDat || !nAufDat)
		return 0;

	if (pAufenthalt->abt_anz > 0)
	{
		nCurDat = lkf_dat2int (pAufenthalt->abt[0].ABTDAT);
		if (nCurDat != nAufDat)
			return 0;
	}

	pAufenthalt->GESVD = nEntDat - nAufDat;
	pAufenthalt->GESVD_KORR = 0;

	for (i = 0;i < (int)pAufenthalt->abt_anz; i++)
		pAufenthalt->abt[i].ABTVD_KORR = 0;

	for (i = 0; i < (int) pAufenthalt->abt_anz; i++)
	{
		if (i + 1 < (int) pAufenthalt->abt_anz)
			nNextDat = lkf_dat2int (pAufenthalt->abt[i+1].ABTDAT);
		else 
			nNextDat = nEntDat;
		if (nNextDat < nCurDat)
		{
			for (;i>=0;i--) pAufenthalt->abt[i].ABTVD = 0;
			pAufenthalt->GESVD = 0;
			return 0;
		}
		pAufenthalt->abt[i].ABTVD = nNextDat - nCurDat;
		if (pAufenthalt->pScoringContext->modelljahr >= 2010 && !memcmp (pAufenthalt->abt[i].ABTFC, UNTERBRECHUNG_ABTFC, UNTERBRECHUNG_LEN))
			pAufenthalt->GESVD -= pAufenthalt->abt[i].ABTVD; /* V14.0 */
		nCurDat = nNextDat;
	}
	return 1;

}

static unsigned int lkf_get_melreduziert_bonus (LKF_AUFENTHALT *pAufenthalt, LKF_AUFENTHALT_SUBGRUPPE *pSubgruppe, int nMELIndices, const int *pMelIndices, int nMELRelevant, const int *pRelevantIndices, MELMEL_OPT_DATA *pOptData, MELMEL_BOOST *pBoost)
{
	int i, k;
	int nNext;
	MELZU_DATA *pMelZuData;
	unsigned int nMelZuBonus;
	unsigned int *pMelValid;
    int nMaxIndex = 0;
	long nMaxZuschlag;
	nNext = 0;
	nMelZuBonus = 0;

	for (i = 0; i < nMELIndices; i++)
	{
		if (nNext < nMELRelevant && pRelevantIndices[nNext] == i)
		{
			++nNext;
			continue;
		}

		pMelZuData = pSubgruppe->pMelZu + pMelIndices[i];
		if (!pMelZuData->nDynMelId || pMelZuData->lDynMelDiff > 0)
			continue;

		nMaxZuschlag = 0;

		pMelValid = (unsigned int *) LKF_CALLOC (sizeof(unsigned int), pAufenthalt->mel_anz);
				
		for (k = 0; k < (int) pAufenthalt->mel_anz; k++)
			pMelValid[k] = (pSubgruppe->pMelZu[k].nDynMelId == pMelZuData->nDynMelId) ? 1 : 0;
					
        for (k = 0; k < nMELIndices; k++) {
            pMelValid[pMelIndices[k]] |= 2;
        
            if (pBoost && pBoost[k].pAlternateIndices) {
                unsigned int l;
                for (l = 0; l < pBoost[k].alternateCount; l++) {
                    pMelValid[pBoost[k].pAlternateIndices[l]] |= 2;
                }
            }
        
        }

		for (k = 0; k < nMELRelevant; k++)
			pMelValid[pMelIndices[pRelevantIndices[k]]] |= 4;

		for (k = 0; k < (int) pAufenthalt->mel_anz; k++)
		{
			if (pSubgruppe->pMelZu[k].nDynMelId == pMelZuData->nDynMelId)
			{
				if ((pMelValid[k] & 1) && (!(pMelValid[k]&2) || (pMelValid[k]&4)))
				{
					if (pSubgruppe->pMelZu[k].lDynMelDiff > (long) nMaxZuschlag)
					{
						nMaxIndex = k;
						nMaxZuschlag = pSubgruppe->pMelZu[k].lDynMelDiff;
					}
				}
			}
		}
		if (nMaxZuschlag > 0)
		{
			pOptData[i].lDynMelBonus = nMaxZuschlag;
			pOptData[i].lDynMelBonusId = nMaxIndex;
			nMelZuBonus += pOptData[i].lDynMelBonus;
		}

		LKF_FREE (pMelValid);		
	}
    if (nMelZuBonus > 0) {
        return nMelZuBonus;
    }
	return nMelZuBonus;
}


void lkf_setMelMelInfo (LKF_AUFENTHALT *pAufenthalt, unsigned int melIndex) {
    unsigned int melgroupIndex = pAufenthalt->mel[melIndex].nMelMelIndexSep;
    if (melgroupIndex > 0 && !pAufenthalt->mel[melIndex].punkteRelevant) {
        
        if (pAufenthalt->mel[melIndex].melmelInfo[0] &&
            memcmp (pAufenthalt->mel[melIndex].melmelInfo, pAufenthalt->pMelMelInfo[melgroupIndex], 4)) {
            strcpy (pAufenthalt->mel[melIndex].melmelInfo, pAufenthalt->pMelMelInfo[melgroupIndex]);
            return;
        }
        if (!pAufenthalt->pMelMelInfo[melgroupIndex][0]) {
 /*
            unsigned int f, i;
            f = 0;
            for (i = 0; i < pAufenthalt->mel_anz; i++) {
                
                if (pAufenthalt->mel[i].nMelMelIndex == melgroupIndex) {
                    
                    if (pAufenthalt->isAmb && !IsBlank(pAufenthalt->mel[i].ambGruppe, 8)) {
                        f += 1;
                    }
                }
            }
            if (f < 2) {
                return;
            }
*/
            return;
        }
        
        strcpy (pAufenthalt->mel[melIndex].melmelInfo, pAufenthalt->pMelMelInfo[melgroupIndex]);
    }
}

/*
static void printrel (int melCount, int *relIndex, int curMelCount) {
    
    int i;
    printf("%3d -> ", curMelCount);
    for (i = 0; i < melCount; i++) {
        if (i < curMelCount)
            printf ("%3d ", relIndex[i]);
        else
            printf ("%3d*", relIndex[i]);
    }
    printf("\n");
}

static void printcomp (int index, int melIndex, int melCount, unsigned char *relIndex) {
    
    int i;
    printf("pr(%d/%d):", index, melIndex);
    for (i = 0; i < melCount; i++) {
        printf ("%c ", relIndex[i] ? '1' : '0');
    }
    printf("\n");
}
*/
static int lkf_aufenthalt_optimize_melmel (LKF_AUFENTHALT *pAufenthalt, LKF_AUFENTHALT_SUBGRUPPE *pSubgruppe, int nMelMelGroup, int bStaticOnly, int subgruppe_index, long *plCurMELZU)
{
	LKF_AUFENTHALT_SUBGRUPPE **ppSubgruppen;
    LKF_SCORINGDETAILS_GROUP *pScoringDetailsGroup;
	int *pMelIndices;
	int *pMelRelevantIndices;
    int i, j, nMelCount = 0;
    int nMelRelevant;
	char sTraceLine[4096];
	int bTraceDetail;
    MELMEL_BOOST *pBoost;

    bTraceDetail = IS_TRACE_DETAIL (pAufenthalt->pScoringContext);

    if (pAufenthalt->pScoringDetails) {
        pScoringDetailsGroup = pAufenthalt->pScoringDetails->pCurrentGroup;
    }
    else {
        pScoringDetailsGroup = 0;
    }
    
	/* Anzahl Subgruppen in der MEL-Gruppe ermitteln */
	for (i = 0; i < (int)pAufenthalt->pat_subgruppen; i++)
	{
		if (pAufenthalt->subgruppe[i].nMelMelGrp == nMelMelGroup) ++nMelCount;
	}

    /* Wenn weniger als 2 Subgruppen, ist keine Optimierung ntig */
    if (nMelCount < 2)
        return 1;

	ppSubgruppen = (LKF_AUFENTHALT_SUBGRUPPE **) LKF_CALLOC((size_t)nMelCount, sizeof (LKF_AUFENTHALT_SUBGRUPPE *));
	pMelIndices = (int *) LKF_CALLOC ((size_t)nMelCount, sizeof (int));
	pMelRelevantIndices = (int *) LKF_CALLOC ((size_t)nMelCount, sizeof (int));

	nMelCount = 0;

	/* Ermitteln der MEL-Indices und der betreffenden Subgruppen */
	for (i = 0; i < (int)pAufenthalt->pat_subgruppen; i++)
	{
		if (pAufenthalt->subgruppe[i].nMelMelGrp == nMelMelGroup)
		{
			pMelIndices[nMelCount] = pAufenthalt->subgruppe[i].mel_id;
			ppSubgruppen[nMelCount++] = pAufenthalt->subgruppe + i;
		}
	}

	if (bTraceDetail)
	{
		sprintf (sTraceLine, "MELMEL-OPTIMIERUNG #%d: %5.5s %8.8s %c (MELMEL: %d)", subgruppe_index, pAufenthalt->mel[pSubgruppe->mel_id].MEL, pSubgruppe->gruppe, pSubgruppe->knoten_char, nMelMelGroup);
        TRACE_DETAIL(pAufenthalt, -1, sTraceLine);
        TRACE_DETAIL(pAufenthalt,-1,"MELGRP   MEL     Anz    MELZU");
		for (i = 0; i < nMelCount; i++)
		{
			sprintf (sTraceLine, "%2d:      %5.5s %3d %8ld", i, pAufenthalt->mel[ppSubgruppen[i]->mel_id].MEL,
									 			       ppSubgruppen[i]->anz,
													   pSubgruppe->pMelZu[ppSubgruppen[i]->mel_id].lMelZu);
			TRACE_DETAIL(pAufenthalt,-1,sTraceLine);
        }
	}

	/* Alle Gruppen entfernen, die zumindest gegen eine andere Gruppe statisch "verlieren" */ 
	nMelRelevant = 0;

	for (i = 0; i < nMelCount; i++)
	{
		nMelRelevant++;
		pMelRelevantIndices[i] = 1;
		for (j = 0; j < nMelCount; j++)
		{
			if (i != j && pAufenthalt->pMelMel[AUFENTHALT_MELMELINDEX(pAufenthalt,pMelIndices[i], pMelIndices[j])] == MELMELGROUP_LOSE)
			{

				if (bTraceDetail)
				{
					sprintf (sTraceLine, "MEL #%d (%5.5s) wird statisch ignoriert!", i, pAufenthalt->mel[pMelIndices[i]].MEL);
					TRACE_DETAIL (pAufenthalt, -1, sTraceLine);
				}
				if (bStaticOnly)
					pSubgruppe->pMelZu[pMelIndices[i]].lMelZu = -1;
				pMelRelevantIndices[i] = 0;
				--nMelRelevant;
                
                lkf_setMelMelInfo (pAufenthalt, pMelIndices[i]);
				break;
			}
		}
	}

	/* Wenn alle Subgruppen zumindest einmal verlieren haben (statische Kreisdefinition), so werden alle statischen Definitionen verworfen */
	if (nMelRelevant == 0)
	{
		if (bTraceDetail)
		{
			TRACE_DETAIL (pAufenthalt, -1, "Zirkulaere statische MEL-MEL-Referenz -> statische Definition wird ignoriert");
		}
		nMelRelevant = nMelCount;
		for (i = 0; i < nMelCount; i++)
		{
			pMelRelevantIndices[i] = 1;
			if (bStaticOnly)
				pSubgruppe->pMelZu[pMelIndices[i]].lMelZu = 0;
		}
	}

	for (i = 0; i < nMelCount; i++)
	{
		if (ppSubgruppen[i] == pSubgruppe)
		{
			/* wenn die aktuelle Subgruppe nicht relevant ist -> ignoriere Gruppe */
			if (!pMelRelevantIndices[i])
			{
				if (bTraceDetail)
				{
					TRACE_DETAIL (pAufenthalt, -1, "Aktuelle Gruppe ungltig, wird durch statische Referenz unterdrckt");
				}
                if (pScoringDetailsGroup) {
                    lkf_scoringdetails_addComment(pAufenthalt, "Aktuelle Gruppe ungueltig (statische MEL-MEL-Unterdrueckung)");
                }

				LKF_FREE (pMelIndices);
				LKF_FREE (pMelRelevantIndices);
				LKF_FREE (ppSubgruppen);
				return 0; /* Gruppe ignorieren */
			}
			else /* V14.0 if (!bStaticOnly) */
			{
				/* wenn die aktuelle Subgruppe relevant ist -> eliminiere alle Subgruppen, die dynamisch mit dieser verbunden sind */
				for (j = 0; j < nMelCount; j++)
				{
					if (i != j && pMelRelevantIndices[j] && pAufenthalt->pMelMel[AUFENTHALT_MELMELINDEX(pAufenthalt,pMelIndices[i], pMelIndices[j])])
					{
						if (bTraceDetail)
						{
							sprintf (sTraceLine, "MEL #%d (%5.5s) wird dynamisch ignoriert (gegen aktuelle Subgruppe)!", j, pAufenthalt->mel[ppSubgruppen[j]->mel_id].MEL);
							TRACE_DETAIL (pAufenthalt, -1, sTraceLine);
						}
						pMelRelevantIndices[j] = 0;
						if (bStaticOnly)
							pSubgruppe->pMelZu[pMelIndices[j]].lMelZu = -1;
						else
						{
							pSubgruppe->MELZU -= pSubgruppe->pMelZu[pMelIndices[j]].lMelZu;
							pSubgruppe->pMelZu[pMelIndices[j]].lMelZu = 0;
                            if (plCurMELZU) plCurMELZU[pMelIndices[j]] -= pSubgruppe->pMelZu[pMelIndices[j]].lMelZu;
						}
					}
				}
			}
		}
	}

	if (bStaticOnly)
	{
		LKF_FREE (pMelIndices);
		LKF_FREE (pMelRelevantIndices);
		LKF_FREE (ppSubgruppen);
		return 1;
	}

    if (pAufenthalt->pScoringContext->modelljahr >= LKF_MIN_MELMEL_BOOST_MODEL) {
        pBoost = (MELMEL_BOOST *) LKF_CALLOC (sizeof(MELMEL_BOOST), nMelCount);
    }
    else {
        pBoost = 0;
    }
        
	/* Entferne alle Gruppen, die nicht mehr relevant sind, die restlichen werden in pMelIndices verdichtet */
	nMelRelevant = 0;
	for (i = 0; i < nMelCount; i++)
	{
		if (!pMelRelevantIndices[i])
		{
			pSubgruppe->MELZU -= pSubgruppe->pMelZu[pMelIndices[i]].lMelZu;
            if (plCurMELZU) plCurMELZU[pMelIndices[i]] -= pSubgruppe->pMelZu[pMelIndices[i]].lMelZu;

			pSubgruppe->pMelZu[pMelIndices[i]].lMelZu = 0;
		}
        else if (ppSubgruppen[i] != pSubgruppe) {
            
            if (pBoost) {
                int j;
                int found = 0;
                MELZU_DATA *pCur;
                
                unsigned char *pMelMelForMel = pAufenthalt->pMelMel + (pAufenthalt->mel_anz * pMelIndices[i]);
                pCur = pSubgruppe->pMelZu + pMelIndices[i];
     /*          printcomp(i, pMelIndices[i], nMelCount, pMelMelForMel); */
                
                for (j = 0; j < nMelRelevant; j++) {
                    
                    if (!memcmp (pAufenthalt->pMelMel + (pAufenthalt->mel_anz * pBoost[j].mainMelId), pMelMelForMel, pAufenthalt->mel_anz)) {
                        if (pBoost[j].pMelzu->lDynMelDiff == pCur->lDynMelDiff) {
                            if (pBoost[j].pAlternateIndices == 0) {
                                pBoost[j].pAlternateIndices = (unsigned int *) LKF_CALLOC (nMelCount, sizeof (unsigned int));
                            }
                            pBoost[j].pAlternateIndices[pBoost[j].alternateCount++] = pMelIndices[i];
                            found = 1;
                            break;
                        }
                    }
                }
                
                if (!found) {
                    pBoost[nMelRelevant].mainMelId = pMelIndices[i];
                    pBoost[nMelRelevant].alternateCount = 0;
                    pBoost[nMelRelevant].pAlternateIndices = 0;
                    pBoost[nMelRelevant].pMelzu = pSubgruppe->pMelZu + pMelIndices[i];
                    pMelIndices[nMelRelevant++] = pMelIndices[i];
                }
            }
            else {
                pMelIndices[nMelRelevant++] = pMelIndices[i];
            }
            
        }
			
	}


	/* Wenn noch immer mehr als eine MEL brig ist -> dynamische Optimierung */
	if (nMelRelevant > 1)
	{
		int bSkip;
		unsigned int nMelZuMax;          /* maximaler MELZU-Wert */
		unsigned int nMelZu;             /* aktueller MEL-Zuschlagswert */
		int nCurrMelIndex;               /* aktueller MEL-Index */
		int nCurrMelCount;               /* aktuell gltige MEL-Anzahl in pMelRelevantIndices */
		int nCurrMelCountMax;            /* Anzahl MELs bei aktuell maximalem Zuschlagswert */
		int *pMelMax;           /* aktuell maximale MEL-Kombination */
		MELMEL_OPT_DATA *pMelMelOptData; /* V14.0: Dynamische MEL-Indices der einzelnen MELs */
		unsigned int *pMelBonus;         /* V14.0: MEL-Bonus */

        unsigned int nMelZuBonus;             /* V14.0: MEL-Zuschlag der sich aus dynamischen MEL-Reduktionen ergibt */
		unsigned int nMaxMelZuBonus;          /* V14.0 */
		int traceDetail;
        int tracePos = 0;

		pMelMax = 0;

        traceDetail = LKF_IS_TRACE_DETAIL (pAufenthalt->pScoringContext->nTraceAufenthalt);

		nCurrMelIndex = 0; 
		nCurrMelCount = 0;
        nCurrMelCountMax = 0;
		nMelZu = 0;        
		nMelZuMax = 0;     
		nMaxMelZuBonus = 0;

        pMelMelOptData = (MELMEL_OPT_DATA *) LKF_CALLOC(sizeof(MELMEL_OPT_DATA), (size_t) nMelRelevant);
		pMelBonus = 0;
        
		if (traceDetail) {
			tracePos = sprintf (sTraceLine, "DYNAMISCHE MEL-MEL-OPTIMIERUNG (%d MELs): ", nMelRelevant);
		}
		for (i = 0; i < nMelRelevant; i++)
		{
			MELZU_DATA *pMelZuData = pSubgruppe->pMelZu + pMelIndices[i];
			pMelMelOptData[i].nDynMelId = pMelZuData->nDynMelId;

			if (traceDetail) {
				tracePos += sprintf (sTraceLine + tracePos, "%5.5s(%d),", pAufenthalt->mel[pMelIndices[i]].MEL, pMelZuData->nDynMelId);
			}
		}
		if (traceDetail) {
 			TRACE_DETAIL (pAufenthalt, -1, sTraceLine);
			for (i = 0; i < (int) pAufenthalt->mel_anz; i++) {
				tracePos = sprintf (sTraceLine, "%5.5s: ", pAufenthalt->mel[i].MEL);
				for (j = 0; j < (int) pAufenthalt->mel_anz; j++) {
					tracePos += sprintf (sTraceLine + tracePos, "%2d ", pAufenthalt->pMelMel[AUFENTHALT_MELMELINDEX(pAufenthalt,i,j)]);
				}
				TRACE_DETAIL (pAufenthalt, -1, sTraceLine);
			}
		}

		for (i = 0; i < nMelRelevant; i++) pMelRelevantIndices[i] = 0;
		pMelMax = (int *) LKF_CALLOC ((size_t)nMelRelevant, sizeof (unsigned int));

		while (nCurrMelIndex < nMelRelevant || nCurrMelCount > 0)
		{
			/* Solange MELs vorhanden sind, untersuchen, ob sie hinzufgbar sind */
			while (nCurrMelIndex < nMelRelevant)
			{
				bSkip = 0;
				for (i = 0; !bSkip && i < nCurrMelCount; i++)
				{
					/* Wenn MEL aufgrund bereits selektierter MEL ausgeschlossen wird -> ignorieren */
					if (pAufenthalt->pMelMel[AUFENTHALT_MELMELINDEX(pAufenthalt,pMelIndices[pMelRelevantIndices[i]],pMelIndices[nCurrMelIndex])])
						bSkip = 1;
				}
				/* MEL hinzufgen, wenn nicht ausgeschlossen */
				if (!bSkip)
				{
					pMelRelevantIndices[nCurrMelCount++] = nCurrMelIndex;
					nMelZu += pSubgruppe->pMelZu[pMelIndices[nCurrMelIndex]].lMelZu;
                    
                    if (pBoost && pBoost[nCurrMelIndex].alternateCount > 0) {
                        unsigned int k;
                        MELMEL_BOOST *pCurBoost = pBoost + nCurrMelIndex;
                        for (k = 0; k < pCurBoost->alternateCount; k++) {
                            nMelZu += pSubgruppe->pMelZu[pCurBoost->pAlternateIndices[k]].lMelZu;
                        }
                    }
                    
				}
				++nCurrMelIndex; /* naechste MEL untersuchen */
			}

        /*    printrel (nMelCount, pMelRelevantIndices, nCurrMelCount); */


			for (j = 0; j < nMelRelevant; j++) pMelMelOptData[j].lDynMelBonus = 0;

			nMelZuBonus = lkf_get_melreduziert_bonus (pAufenthalt, pSubgruppe, nMelRelevant, pMelIndices, nCurrMelCount, pMelRelevantIndices, pMelMelOptData, pBoost);
			if (traceDetail) {
				tracePos = sprintf (sTraceLine, "KOMBINATION: ");
				for (j = 0; j < nCurrMelCount; j++) tracePos += sprintf(sTraceLine + tracePos, "%5.5s,", pAufenthalt->mel[pMelIndices[pMelRelevantIndices[j]]].MEL);
				sprintf (sTraceLine + tracePos, " melzu=%ld", (long) nMelZuBonus);
				TRACE_DETAIL (pAufenthalt,-1, sTraceLine);
			}

			/* Aktuell gefundene MEL-Kombination mit bisher bester Kombination vergleichen */
			if (nMelZu + nMelZuBonus >= nMelZuMax)
			{
				nMelZuMax = nMelZu + nMelZuBonus;
				nCurrMelCountMax = nCurrMelCount;
				memcpy (pMelMax, pMelRelevantIndices, (size_t) (sizeof(unsigned int) * (unsigned int)nCurrMelCount));
			
				nMaxMelZuBonus = nMelZuBonus;

				if (nMaxMelZuBonus > 0)
				{
					if (!pMelBonus)
						pMelBonus = (unsigned int *) LKF_CALLOC(sizeof(unsigned int), pAufenthalt->mel_anz);

					for (j = 0; j < (int) pAufenthalt->mel_anz; j++)
						pMelBonus[j] = 0;
					for (j = 0; j < nMelRelevant; j++)
					{
						if (pMelMelOptData[j].lDynMelBonus > 0)
							pMelBonus[pMelMelOptData[j].lDynMelBonusId] = (unsigned int) pMelMelOptData[j].lDynMelBonus;
					}
				}
			}

			/* hchsten MEL-Index entfernen */
			nMelZu -= pSubgruppe->pMelZu[pMelIndices[pMelRelevantIndices[--nCurrMelCount]]].lMelZu;
			nCurrMelIndex = pMelRelevantIndices[nCurrMelCount];
            
            if (pBoost && pBoost[nCurrMelIndex].alternateCount > 0) {
                unsigned int k;
                MELMEL_BOOST *pCurBoost = pBoost + nCurrMelIndex;
                for (k = 0; k < pCurBoost->alternateCount; k++) {
                    nMelZu -= pSubgruppe->pMelZu[pCurBoost->pAlternateIndices[k]].lMelZu;
                }
            }
            
			/* wenn hchster MEL-Index, weitere MEL-Indices entfernen */
			if (nCurrMelIndex + 1 == nMelRelevant)
			{
				while (nCurrMelCount > 0 && pMelRelevantIndices[nCurrMelCount-1]+1 == pMelRelevantIndices[nCurrMelCount])
				{	
					nMelZu -= pSubgruppe->pMelZu[pMelIndices[pMelRelevantIndices[--nCurrMelCount]]].lMelZu;			
					nCurrMelIndex = pMelRelevantIndices[nCurrMelCount];
				}
			}
			++nCurrMelIndex; /* mit nchster MEL weitermachen */
		}

		/* fuer alle MELs, die nicht im MEL-Relevant-Array sind, MELZU entfernen */
		nCurrMelIndex = 0;
		for (i = 0; i < nMelRelevant; i++)
		{
			if (nCurrMelIndex < nCurrMelCountMax && pMelMax[nCurrMelIndex] == i)
				nCurrMelIndex++;
			else
			{
				pSubgruppe->MELZU -= pSubgruppe->pMelZu[pMelIndices[i]].lMelZu;
                if (plCurMELZU) plCurMELZU[pMelIndices[i]] -= pSubgruppe->pMelZu[pMelIndices[i]].lMelZu;
				pSubgruppe->pMelZu[pMelIndices[i]].lMelZu = 0;
                
                if (pBoost && pBoost[nCurrMelIndex].alternateCount > 0) {
                    unsigned int k;
                    MELMEL_BOOST *pCurBoost = pBoost + nCurrMelIndex;
                    for (k = 0; k < pCurBoost->alternateCount; k++) {
                        pSubgruppe->MELZU -= pSubgruppe->pMelZu[pCurBoost->pAlternateIndices[k]].lMelZu;
                        if (plCurMELZU) plCurMELZU[pMelIndices[i]] -= pSubgruppe->pMelZu[pCurBoost->pAlternateIndices[k]].lMelZu;
                        pSubgruppe->pMelZu[pCurBoost->pAlternateIndices[k]].lMelZu = 0;
                    }
                }
			}
		}

		if (nMaxMelZuBonus > 0)
		{
			for (i = 0; i < (int) pAufenthalt->mel_anz; i++)
			{
				if (pMelBonus[i])
				{
					pSubgruppe->MELZU += pMelBonus[i];
                    if (plCurMELZU) plCurMELZU[i] += pMelBonus[i];

					pSubgruppe->pMelZu[i].lMelZu += pMelBonus[i];
				}
			}
		}

		LKF_FREE (pMelMax);
		if (pMelMelOptData)
			LKF_FREE (pMelMelOptData);
		if (pMelBonus)
			LKF_FREE (pMelBonus);
	}
    
    if (pBoost) {
        int i;
        for (i = 0; i < nMelRelevant; i++) {
            if (pBoost[i].pAlternateIndices) {
                LKF_FREE (pBoost[i].pAlternateIndices);
            }
        }
        LKF_FREE (pBoost);
    }

	LKF_FREE (pMelIndices);
	LKF_FREE (pMelRelevantIndices);
	LKF_FREE (ppSubgruppen);
	return 1; /* Gruppe gltig */

}

typedef struct {
	long afg_punkte;
	long alter;
	long alter0_punkte;
	long alter1_punkte;
} LKF_AMB_MINIMUM_OPTION;

typedef struct 
{
	long min_punkte;
	LKF_AMB_MINIMUM_OPTION apg0000;
	LKF_AMB_MINIMUM_OPTION apg0010;
} LKF_AMB_MINIMUM;

typedef struct {
    int  valid;
    unsigned int groupCount;
    unsigned int abtCount;
    char grp[LKF_MAX_AMB_GRP * 10];
    float mapApg[LKF_MAX_AMB_ABT * LKF_MAX_AMB_GRP];
    int apgcount[LKF_MAX_AMB_GRP];
    double lpg[LKF_MAX_AMB_GRP];
    double lpa[LKF_MAX_AMB_ABT];
    double lps[LKF_MAX_AMB_ABT];
    double lpall;
    double spall;
    
    long   ppl[LKF_MAX_AMB_GRP];
    long   pBonus[LKF_MAX_AMB_GRP];
    
} LKF_AUFENTHALT_SCORING_ABT;

static int lkf_scoring_amb_minimum_check(LKF_AUFENTHALT *pAufenthalt, const char *basisGruppe) 
{ /* V21.1: Minimum-Check APG00.00/APG00.10 */
	static LKF_AMB_MINIMUM minimum2017 = { 50, {5, 14, 40, 25}, {10, 14, 50, 30} };
	LKF_AMB_MINIMUM *pMinimum = &minimum2017;
	LKF_AMB_MINIMUM_OPTION *pOption;
	long min_punkte;
	char knoten;

	if (pAufenthalt->SCGES - pAufenthalt->SCSPEZ >= pMinimum->min_punkte) {
		return 0;
	}
	pOption = memcmp(basisGruppe, "APG00.00", 8) ? &pMinimum->apg0010 : &pMinimum->apg0000;

	if (scoring_afg_split(pAufenthalt, "AFG01")) {
		min_punkte = pOption->afg_punkte;
		knoten = 'A';
	}
	else if (pAufenthalt->Alter <= pOption->alter) {
		min_punkte = pOption->alter0_punkte;
		knoten = 'B';
	}
	else {
		min_punkte = pOption->alter1_punkte;
		knoten = 'C';
	}

	if (pAufenthalt->SCGES - pAufenthalt->SCSPEZ < min_punkte) {
        pAufenthalt->KNT[0] = knoten;
		pAufenthalt->KNT[1] = ' ';
		pAufenthalt->LDFPLK = 0;
        pAufenthalt->SCULK = 0;
		pAufenthalt->MELZU = 0;
		pAufenthalt->LDFPTK = min_punkte;
		pAufenthalt->SCGES = min_punkte + pAufenthalt->SCSPEZ; /* V2019.2 */
		memcpy(pAufenthalt->GRP, basisGruppe, 8);
        pAufenthalt->maxMelIndex = -1;
        
        if (IS_TRACE_DETAIL(pAufenthalt->pScoringContext)) {
            char sTraceLine[1024];
            sprintf (sTraceLine, "GRP=%8.8s, KNT=%c, MINPUNKTE=%ld, SPEZ=%ld, GES=%ld", basisGruppe, knoten, min_punkte, pAufenthalt->SCSPEZ, pAufenthalt->SCGES);
            TRACE_DETAIL(pAufenthalt, -1, sTraceLine);
        }

        
		return 1;
	}
	return 0;
}

static void lkf_scoring_amb_adjust (LKF_AUFENTHALT *pAufenthalt, LKF_AUFENTHALT_SCORING_ABT *pScoringAbt, long sumLst)
{
    unsigned int i, j;
    double allLP;
    long abt100[LKF_MAX_AMB_ABT];
    long sumP = sumLst * 100;
    long sumadd = 0;
    
    for (i = 0; i < pScoringAbt->groupCount; i++) {
        if (pScoringAbt->lpg[i] > 0) {
            unsigned int groupOffset = i * LKF_MAX_AMB_ABT;
            int count = pScoringAbt->apgcount[i];

            if (count > 0) {
                double part = (pScoringAbt->lpg[i] / count);
                
                for (j = 0; j < pScoringAbt->abtCount; j++) {
                    if (pScoringAbt->mapApg[j + groupOffset] > 0.001) {
                        pScoringAbt->lpa[j] += part * pScoringAbt->mapApg[j + groupOffset];
                    }
                }
            }
        }
    }
    
    allLP = pScoringAbt->lpall / pScoringAbt->abtCount;
    
    for (i = 0; i < pScoringAbt->abtCount; i++) {
        sumP -= pAufenthalt->abt[i].lp100;
        abt100[i] = simCLng((pScoringAbt->lpa[i] + allLP)*100);
        pAufenthalt->abt[i].lp100 += abt100[i] + (long) (pScoringAbt->lps[i] * 100);
        sumadd += abt100[i];
    }
    
    if (sumadd < sumP) {
        for (i = 0; i < pScoringAbt->abtCount; i++) {
            if (abt100[i] > 0) {
                pAufenthalt->abt[i].lp100++;
                if (++sumadd == sumP)
                    break;
            }
        }
    }
    else if (sumadd > sumP) {
        for (i = pScoringAbt->abtCount; i > 0; i--) {
            if (abt100[i-1] > 0) {
                pAufenthalt->abt[i-1].lp100--;
                if (--sumadd == sumP)
                    break;
            }
        }
    }
}


static void lkf_scoring_amb_divide (LKF_AUFENTHALT *pAufenthalt, long tp, long lp)
{
    
    long lpabt, tpabt, tdiff, ldiff, i, abtanz;
    
    abtanz = (long) pAufenthalt->abt_anz;
    
    lp *= 100;
    tp *= 100;
    
    lpabt = lp / abtanz;
    tpabt = tp / abtanz;
    
    ldiff = lp - lpabt * abtanz;
    tdiff = tp - tpabt * abtanz;
    
    for (i = 0; i < abtanz; i++) {
        pAufenthalt->abt[i].lp100 += (i < ldiff) ? lpabt + 1 : lpabt;
        pAufenthalt->abt[i].tp100 += (i < tdiff) ? tpabt + 1 : tpabt;
    }
}

static void lkf_scoring_tkl_adjust (LKF_AUFENTHALT *pAufenthalt, long sumlst) {
    unsigned int i;
    long lp;
    LKF_AUFENTHALT_SUBGRUPPE *pSubgruppe, *pMaxSubgruppe;
    LKF_AUFENTHALT_MEL *pMel;
    
    if (pAufenthalt->max_index >= 0 && (unsigned int) pAufenthalt->max_index < pAufenthalt->pat_subgruppen) {
        pMaxSubgruppe = pAufenthalt->subgruppe + pAufenthalt->max_index;
    }
    else {
        pMaxSubgruppe = 0;
    }
    
    for (i = 0; i < pAufenthalt->pat_subgruppen; i++) {
        pSubgruppe = pAufenthalt->subgruppe + i;
        
        if (i == pAufenthalt->max_index) {
            lp = pSubgruppe->SCULK + pAufenthalt->LDFPLK + pSubgruppe->MELZU_DIRECT;
        }
        else if (pMaxSubgruppe && pMaxSubgruppe->pMelZu && pSubgruppe->mel_id >= 0) {
            lp = pMaxSubgruppe->pMelZu[pSubgruppe->mel_id].lMelZu; /* V2020.3 */
        }
        else {
            lp = pSubgruppe->MELZU;
        }
        if (pSubgruppe->mel_id >= 0)
        {
            pMel = pAufenthalt->mel + pSubgruppe->mel_id;
        }
        else { /* V2022.2 */
            pMel = 0;
        }

        if (pMel && (IsBlank (pMel->KALEIST, 6) || !memcmp (pAufenthalt->KANR, pMel->KALEIST, 4))) {
            if (!IsBlank(pMel->FCLEIST, 8)) {
                unsigned int k;
                unsigned int melabt;
                LKF_AUFENTHALT_ABT *pAbt;
                melabt = 0;
                for (k = 0; k < pAufenthalt->abt_anz; k++) {
                    if (!memcmp (pAufenthalt->abt[k].ABTFC, pMel->FCLEIST, 8)) {
                        ++melabt;
                    }
                }
                if (melabt > 0) {
                    long lpabt, plus1count;
                    lpabt = lp * 100 / melabt;
                    plus1count = lp * 100 - lpabt * melabt;
                    for (k = 0; k < pAufenthalt->abt_anz; k++) {
                        pAbt = pAufenthalt->abt + k;
                        if (!memcmp (pAbt->ABTFC, pMel->FCLEIST, 8)) {
                            pAbt->lp100 += lpabt;
                            if (plus1count > 0) {
                                pAbt->lp100++;
                                plus1count--;
                            }
                        }
                    }
                }
                else {
                    lkf_scoring_amb_divide(pAufenthalt, 0, lp);
                }
            }
            else {
                lkf_scoring_amb_divide(pAufenthalt, 0, lp);
            }
        }
        else {
            lkf_scoring_amb_divide(pAufenthalt, 0, lp);
        }
    }
}

static void lkf_scoring_divide_lst (LKF_AUFENTHALT_SCORING_ABT *pScoringAbt, unsigned int melabt, const char *ambMap, double lp)
{
    unsigned int i;
    
    double ambP = lp / melabt;
    
    for (i = 0; i < pScoringAbt->abtCount; i++) {
        if (ambMap[i]) {
            pScoringAbt->lpa[i] += ambP;
        }
    }
}

static void lkf_scoring_divide_special (LKF_AUFENTHALT_SCORING_ABT *pScoringAbt, unsigned int melabt, const char *ambMap, double lp)
{ /* V2019.0 */
    unsigned int i;
    
    double ambP = lp / melabt;
    
    for (i = 0; i < pScoringAbt->abtCount; i++) {
        if (ambMap[i]) {
            pScoringAbt->lps[i] += ambP;
        }
    }
}

static void lkf_adjust_melzu_pkt (LKF_AUFENTHALT *pAufenthalt) {
    
    unsigned int i, j;
    unsigned int hauptmelindex;
    LKF_AUFENTHALT_SUBGRUPPE *pSubGruppe;
    if (pAufenthalt->subgruppe && pAufenthalt->max_index >= 0) {
        pSubGruppe = pAufenthalt->subgruppe + pAufenthalt->max_index;
        hauptmelindex  = (unsigned int) pSubGruppe->mel_id;
    }
    else {
        pSubGruppe = 0;
        hauptmelindex = 0;
    }
    
    
    for (i = 0; i < pAufenthalt->mel_anz; i++) {
        if (pAufenthalt->mel[i].punkteIndex != i)
            pAufenthalt->mel[i].punkteIndex = pAufenthalt->mel[pAufenthalt->mel[i].punkteIndex].punkteIndex;
    }
    
    for (i = 0; i < pAufenthalt->mel_anz; i++) {
        
        if (pAufenthalt->mel[i].punkteIndex != i) {
            
            long lcount = 0, lmelzu, lmelzuSum = 0, lmelzuBonus;
            unsigned int sumPunkteIndex = pAufenthalt->mel[i].punkteIndex;
            int melanz;
            
            for (j = 0; j < pAufenthalt->mel_anz; j++) {
                if (pAufenthalt->mel[j].punkteIndex == sumPunkteIndex) {
                    if (GetInt (pAufenthalt->mel[j].ORIG_MELANZ, LKF_AUFENTHALT_MELANZ_LEN, &melanz)) {
                        lmelzuSum += pAufenthalt->mel[j].melzu;
                        lcount += melanz;
                        //if (pSubGruppe && j == hauptmelindex) --lcount;
                    }
                }
            }
            
            if (lcount > 1) {
                lmelzu = lmelzuSum / lcount;
                lmelzuBonus = lmelzuSum - lmelzu * lcount;
                
                for (j = 0; j < pAufenthalt->mel_anz; j++) {
                    if (pAufenthalt->mel[j].punkteIndex == sumPunkteIndex) {
                        
                        if (GetInt (pAufenthalt->mel[j].ORIG_MELANZ, LKF_AUFENTHALT_MELANZ_LEN, &melanz)) {
                            //if (pSubGruppe && j == hauptmelindex) --melanz;
                            pAufenthalt->mel[j].melzu = melanz * lmelzu;
                            if (lmelzuBonus >= melanz) {
                                pAufenthalt->mel[j].melzu += melanz;
                                lmelzuBonus -= melanz;
                            }
                            else if (lmelzuBonus > 0) {
                                pAufenthalt->mel[j].melzu += lmelzuBonus;
                                lmelzuBonus = 0;
                            }
                            pAufenthalt->mel[j].punkteIndex = j;
                        }
                    }
                }
            }
        }
    }
}


//define LKF_SCORING_AMB_ABT_CHECK 1


#ifdef LKF_SCORING_AMB_ABT_CHECK
static int lkf_scoring_amb_check (LKF_AUFENTHALT *pAufenthalt) {
    
    long lp = 0;
    long tp = 0;
    char abtpos[10];
    long lpabt100, tpabt100;
	unsigned int i;
    int ok = 1;
    
    for (i = 0; i < pAufenthalt->abt_anz; i++) {
        lkf_auf_get_abtscor100(pAufenthalt, i, abtpos, &tpabt100, &lpabt100);
        lp += lpabt100;
        tp += tpabt100;
    }
    
    if (tp != pAufenthalt->LDFPTK * 100) {
        ok = 0;
    }
    
    if (lp != (pAufenthalt->LDFPLK + pAufenthalt->MELZU) * 100) {
        ok = 0;
    }
    
    return ok;
    
}
#endif

static int lkf_has_amb_melmel (LKF_AUFENTHALT *pAufenthalt) {
    if (pAufenthalt->nMelMelCount == 0)
        return 0;
    return 1;
}

typedef struct {
    double mel_kp;
    double mel_lp;
    char gruppe[8];
    char knoten;
    int melanz;
} AMB_MELMEL;


typedef struct {
    unsigned int melAnz;
    unsigned int melAnzOpt;
    AMB_MELMEL *pAmbMelMel;
    char *melmel_relevant;
    unsigned int *melOptIndices;
    double points;
} AMB_MELMEL_OPT;

static int lkf_optimize_amb_melmel (LKF_AUFENTHALT *pAufenthalt, AMB_MELMEL_OPT *pOpt) {
    
    unsigned int i, j;
    unsigned int melIndexLeft, melIndexRight;
    
    double LP = 0;
    double KP = 0;
    double maxLP = 0;
    double maxGrpKP = 0;
    
	double mel_lp, mel_kp;

    char maxGrp[10];
    LKF_AUFENTHALT_SCORING_ABT scoringAbt;

	memset (&scoringAbt, 0, sizeof (LKF_AUFENTHALT_SCORING_ABT));
    
    for (i = 0; i < pOpt->melAnzOpt; i++) {
        
        melIndexLeft = pOpt->melOptIndices[i];
        if (!pOpt->melmel_relevant[melIndexLeft])
            continue;
        
        for (j = 0; j < pOpt->melAnzOpt; j++) {
            
            if (i == j)
                continue;
            
            melIndexRight = pOpt->melOptIndices[j];
            if (!pOpt->melmel_relevant[melIndexRight])
                continue;
        
            if (pAufenthalt->pMelMel[melIndexLeft*pOpt->melAnz + melIndexRight]) {
                
                AMB_MELMEL_OPT left;
                AMB_MELMEL_OPT right;
                char *melmel_relevant_left, *melmel_relevant_right;
                
                melmel_relevant_left = (char *) LKF_MALLOC(pOpt->melAnz);
                melmel_relevant_right = (char *) LKF_MALLOC(pOpt->melAnz);
                
                memcpy (melmel_relevant_left, pOpt->melmel_relevant, pOpt->melAnz);
                memcpy (melmel_relevant_right, pOpt->melmel_relevant, pOpt->melAnz);
                
                memcpy (&left, pOpt, sizeof (AMB_MELMEL_OPT));
                memcpy (&right, pOpt, sizeof (AMB_MELMEL_OPT));
                
                left.melmel_relevant = melmel_relevant_left;
                right.melmel_relevant = melmel_relevant_right;
                
                melmel_relevant_left[melIndexRight] = 0;
                melmel_relevant_right[melIndexLeft] = 0;
                
                lkf_optimize_amb_melmel(pAufenthalt, &left);
                lkf_optimize_amb_melmel(pAufenthalt, &right);
                
                if (left.points >= right.points) {
                    pOpt->points = left.points;
                    memcpy (pOpt->melmel_relevant, melmel_relevant_left, pOpt->melAnz);
                }
                else {
                    pOpt->points = right.points;
                    memcpy (pOpt->melmel_relevant, melmel_relevant_right, pOpt->melAnz);
                }
                
                LKF_FREE (melmel_relevant_left);
                LKF_FREE (melmel_relevant_right);
                return 0;
            }
        }
    }
    
    KP = 0;
    LP = 0;
    memset (maxGrp, ' ', 8);
    
    for (i = 0; i < pOpt->melAnz; i++) {
        
        if (!pOpt->melmel_relevant[i] || !pOpt->pAmbMelMel[i].gruppe[0])
            continue;
        
        
        if (i == 0 || KP < pOpt->pAmbMelMel[i].mel_kp)
        {
            KP = pOpt->pAmbMelMel[i].mel_kp;
        }
        
        mel_lp = pOpt->pAmbMelMel[i].mel_lp;
        mel_kp = pOpt->pAmbMelMel[i].mel_kp;
        
        if (i == 0 || mel_lp > maxLP || (mel_lp == maxLP && mel_kp > maxGrpKP ) || (mel_lp == maxLP && mel_kp == maxGrpKP  && memcmp (maxGrp, pOpt->pAmbMelMel[i].gruppe,8)>0)) {
            maxLP = mel_lp;
            maxGrpKP = mel_kp;
            memcpy (maxGrp, pOpt->pAmbMelMel[i].gruppe, 8);
        }

        
        if (!memcmp (pOpt->pAmbMelMel[i].gruppe, "APG", 3)) {
            int found = 0;
            char cur[10];
            unsigned int g;

            sprintf (cur, "%8.8s%c", pOpt->pAmbMelMel[i].gruppe, pOpt->pAmbMelMel[i].knoten);
            
            for (g = 0; g < scoringAbt.groupCount; g++) {
                if (!memcmp (scoringAbt.grp + 10 * g, cur, 9)) {
                    found = 1;
                    break;
                }
            }
            
            if (!found) {
                if (scoringAbt.groupCount < LKF_MAX_AMB_GRP - 1) {
                    memcpy (scoringAbt.grp + scoringAbt.groupCount++ * 10, cur, 10);
                }
                LP += pOpt->pAmbMelMel[i].mel_lp;
            }
        }
        else
        {
            double v = pOpt->pAmbMelMel[i].mel_lp * pOpt->pAmbMelMel[i].melanz;
            LP += v;
        }
    }
    
    pOpt->points = KP + LP;
    
    
//    printf ("%12.12s/%8.8s: %8.2lf %8.2lf %8.2lf\n", pAufenthalt->PATID, pAufenthalt->AUFDAT, KP, LP, pOpt->points);
    
    return 1;
}

static int lkf_compute_amb_melmel (LKF_AUFENTHALT *pAufenthalt, char *melmel_relevant) {
    
    unsigned int i, j;
    unsigned int *melmel_opt;
    unsigned int optCount = 0;
    char errbuf[1024];
    AMB_MELMEL *ambMelMel;
    LKF_CONTEXT *pContext;
    MEL_ARRAY_STRUC *pMelDef;
    const LKF_KNOTEN *pKnoten;
    short act_knoten;
    int nSplitSteps;
    int scoring_split_result;
    int isAgr;
    int success;
    unsigned int melAnz;
    int modelljahr;
    AMB_MELMEL_OPT melmelOpt;
    unsigned int nMelMelGroup;
    success = 1;
    
    melAnz = pAufenthalt->mel_anz;
    pContext = pAufenthalt->pScoringContext;
    modelljahr = pContext->modelljahr;
    melmel_opt = (unsigned int*) LKF_CALLOC (sizeof (unsigned int), melAnz);
    memset (melmel_relevant, 1, melAnz);
    
    ambMelMel = (AMB_MELMEL *) LKF_CALLOC(sizeof (AMB_MELMEL), melAnz);
    
    for (i = 0; i < pAufenthalt->mel_anz; i++) {

        LKF_AUFENTHALT_MEL *pMel = pAufenthalt->mel + i;
        if (pMel->MEL_RELEVANT == 'N' || pMel->MEL_RELEVANT == 'K') {
            melmel_relevant[i] = 0;
            continue;
        }
        
        if (modelljahr >= 2021 && pMel->nIgnoreMel) {
            melmel_relevant[i] = 0;
            continue;
        }
        
        if (pMel->nTagMelAnzRelevant == 0) /* V22.2 */
        {
            melmel_relevant[i] = 0;
            continue;
        }
        
        pMelDef = FindMel(pContext, pMel->MEL);
        if (pMelDef && pMelDef->ambFixPunkte > 0)
        { /* V2019.0 */
            melmel_relevant[i] = 0;
            continue;
        }

        nMelMelGroup = pAufenthalt->mel[i].nMelMelIndex;
        if (nMelMelGroup > 0) {
            
            for (j = 0; j < melAnz; j++) {
                
				LKF_AUFENTHALT_MEL *pOtherMel;
                if (i == j)
                    continue;
                
                pOtherMel = pAufenthalt->mel + j;
                if (pOtherMel->MEL_RELEVANT == 'N' || pOtherMel->MEL_RELEVANT == 'K')
                    continue;
                
                if (modelljahr >= 2021 && pOtherMel->nIgnoreMel)
                    continue;
                
                if (pAufenthalt->pMelMel[AUFENTHALT_MELMELINDEX(pAufenthalt,i, j)] == MELMELGROUP_LOSE) {
                    melmel_relevant[i] = 0;
                    lkf_setMelMelInfo (pAufenthalt, i);
                    break;
                }
            }
            
            if (melmel_relevant[i])
                melmel_opt[optCount++] = i;
            else
                continue;
        }
        
        if (pAufenthalt->psych_flag) {
        
            
            // TODO: check relevant fr MELMEL ?
            
            int melindex = pMel->nMelIndex;
            
            if (IsConstMEL (pContext, CONSTMEL_AM060_7596, melindex)) {
                if (!(pAufenthalt->psych_flag & 2)) continue;
            }
            else if (IsConstMEL (pContext, CONSTMEL_AM070_7597, melindex)) {
                if (!(pAufenthalt->psych_flag & 4)) continue;
            }
            else if (IsConstMEL (pContext, CONSTMEL_AM080_7598, melindex)) {
                if (!(pAufenthalt->psych_flag & 8)) continue;
            }
            else if (IsConstMEL (pContext, CONSTMEL_AM090_7601, melindex)) {
                if (!(pAufenthalt->psych_flag & 16)) continue;
            }
            else if (IsConstMEL (pContext, CONSTMEL_AM110, melindex)) {
                if (!(pAufenthalt->psych_flag & 32)) continue;
            }
            else if (modelljahr >= 2022 && IsConstMEL (pContext, CONSTMEL_AM150, melindex)) {
                if (!(pAufenthalt->psych_flag & 256)) continue;
            }
            else if (IsConstMEL (pContext, CONSTMEL_AM120, melindex)) {
                if (!(pAufenthalt->psych_flag & 64)) continue;
            }
            else if (IsConstMEL (pContext, CONSTMEL_AM130, melindex)) {
                if (pContext->modelljahr >= 2017) {
                    if (!(pAufenthalt->psych_flag & 192)) continue;
                }
                else if (!(pAufenthalt->psych_flag & 128)) continue;
            }
            else
                continue;
            
        }
        
        if (!pMelDef || !pMelDef->sAmbGruppe[0] || IsEmpty(pMelDef->sAmbGruppe, 8)) {
            memcpy (ambMelMel[i].gruppe, "APG00.10", 8);
            continue;
        }
        
        pKnoten = GetAMBKnoten (pContext->pStamm2001, pMelDef->sAmbGruppe, errbuf);
        if (!pKnoten)
        {
            success = -5;
            sprintf (pContext->errortext, "FEHLER beim Einlesen des Files '%8.8s': %s",
                     pMelDef->sAmbGruppe, errbuf);
            break;
        }
        
        act_knoten = 0;
        
        nSplitSteps = 0;
        
        while (pKnoten->nfl[act_knoten] != -99)
        {
            if (++nSplitSteps > 20)
            {
                /* Nach 20 Schritten abbrechen - vermutlich Fehler im Stammdaten-File */
                sprintf (pContext->errortext, "FEHLER beim Scoring-Split von '%8.8s', Abbruch nach 20 Schritten, vermutlich Fehler im Stammdaten-File",
                         pMelDef->sAmbGruppe);
                success = -6;
                break;
            }
            
            scoring_split_result = scoring_split_2017(pAufenthalt, pKnoten, act_knoten, 0, &isAgr, pAufenthalt->HD, (int) i, 0);
            if (scoring_split_result)
                act_knoten = pKnoten->nfl[act_knoten]-1;
            else act_knoten = pKnoten->nfl[act_knoten];
            
        }
        
        ambMelMel[i].mel_kp = pKnoten->tk_anteil[act_knoten];
        ambMelMel[i].mel_lp = pKnoten->lk_anteil[act_knoten];
        
        if (IsBlank(pMelDef->sAmbGruppe, 8))
            ambMelMel[i].gruppe[0] = 0;
        else {
            memcpy (ambMelMel[i].gruppe, pMelDef->sAmbGruppe, 8);
            pMel->ambKnoten = pKnoten->knoten_char[act_knoten];
            strcpy (pMel->ambGruppe, pMelDef->sAmbGruppe);
        }
        if (pContext->modelljahr >= 2017)
            ambMelMel[i].melanz = pMel->nTagMelAnzRelevant;
        else
            ambMelMel[i].melanz = atoi(pMel->MELANZ);
        ambMelMel[i].knoten = pKnoten->knoten_char[act_knoten];
        
    }
    
    if (success < 0) {
        LKF_FREE (melmel_opt);
        LKF_FREE (ambMelMel);
        return success;
    }
    
    melmelOpt.melAnz = melAnz;
    melmelOpt.pAmbMelMel = ambMelMel;
    melmelOpt.melOptIndices = melmel_opt;
    melmelOpt.melAnzOpt = optCount;
    melmelOpt.melmel_relevant = melmel_relevant;
    
    lkf_optimize_amb_melmel(pAufenthalt, &melmelOpt);
    
    for (j = 0; j < pAufenthalt->mel_anz; j++) {
        lkf_setMelMelInfo(pAufenthalt, j);
    }
    
    
    LKF_FREE (melmel_opt);
    LKF_FREE (ambMelMel);
    return 1;
}

static unsigned int lkf_scoring_check_fcleist_index (LKF_AUFENTHALT_SCORING_ABT *pScoringAbt, const LKF_AUFENTHALT *pAufenthalt, const LKF_AUFENTHALT_MEL *pMel, char *ambabtmap)
{ /* V2019.0 */
    unsigned int melabt = 0;
    if (pScoringAbt->valid) {
        melabt = 0;
        if (IsBlank (pMel->KALEIST, 6) || !memcmp (pAufenthalt->KANR, pMel->KALEIST, 4)) {
            if (!IsBlank(pMel->FCLEIST, 8)) {
                unsigned int k;
                memset (ambabtmap, 0, pScoringAbt->abtCount);
                for (k = 0; k < pScoringAbt->abtCount; k++) {
                    if (!memcmp (pAufenthalt->abt[k].ABTFC, pMel->FCLEIST, 8)) {
                        ambabtmap[k] = 1;
                        ++melabt;
                    }
                }
            }
        }
    }
    return melabt;
}

static int lkf_scoring_compute_amb (LKF_AUFENTHALT *pAufenthalt)
{
    int bTraceInfo;
    int bTraceDetail;
    
    unsigned int countedMELs = 0;
    
    double LP = 0;
    double KP = 0;
    double SP = 0; /* V2019.0 */
    double maxLP = 0;
    double maxGrpKP = 0;
    unsigned int maxMelIndex = 0;
    
    LKF_AUFENTHALT_SCORING_ABT scoringAbt;
    
    short act_knoten;
    int nSplitSteps;
    int scoring_split_result;
    int isAgr;
    
    char sTraceLine[1024];
    
    const char *basisGruppe;
    
    MEL_ARRAY_STRUC *pMelDef;
    const LKF_KNOTEN *pKnoten;
    
    LKF_CONTEXT *pContext = pAufenthalt->pScoringContext;
    STAMM2001 *pStamm = pContext->pStamm2001;
    
    char ambabtmap[LKF_MAX_AMB_ABT];
    unsigned int melabt = 0;
    
    char *melmel_relevant = 0;
    unsigned int i;
    int *melgrpIndex;
    
    int modelljahr;
    
    modelljahr = pContext->modelljahr;
    
    bTraceInfo = IS_TRACE_INFO (pContext);
    bTraceDetail = IS_TRACE_DETAIL (pContext);
    
    if (bTraceInfo)
        TRACE_INFO(pAufenthalt, -1, "Scoring Punkteberechnung (spitalsambulant)");

    basisGruppe = "APG00.00";
    
    memset (&scoringAbt, 0, sizeof (LKF_AUFENTHALT_SCORING_ABT));
    scoringAbt.abtCount = pAufenthalt->abt_anz;
    scoringAbt.valid = 1; // (scoringAbt.abtCount > 1);
    if (scoringAbt.valid) {
        scoringAbt.groupCount = 0;
        if (scoringAbt.abtCount > LKF_MAX_AMB_ABT) scoringAbt.abtCount = LKF_MAX_AMB_ABT;
    }
    
    melgrpIndex = 0;
    if (pAufenthalt->mel_anz) {
        unsigned int i;
        double mel_kp, mel_lp;

        if (lkf_has_amb_melmel(pAufenthalt)) {
            melmel_relevant = (char *) LKF_MALLOC (pAufenthalt->mel_anz);
            lkf_compute_amb_melmel(pAufenthalt, melmel_relevant);
            
        }

        melgrpIndex = (int *) LKF_CALLOC(sizeof(int), pAufenthalt->mel_anz);
        
        for (i = 0; i < pAufenthalt->mel_anz; i++) {
         
            LKF_AUFENTHALT_MEL *pMel = pAufenthalt->mel + i;
            melgrpIndex[i] = -1;
            if (pMel->MEL_RELEVANT == 'N' || pMel->MEL_RELEVANT == 'K')
                continue;
            
            if (pMel->nIgnoreMel && modelljahr >= 2021)
                continue; /* V2021.2 */
            
            if (modelljahr >= 2017 && pMel->nTagMelAnzRelevant == 0)
                continue; /* V22.1 */
        
            pMelDef = FindMel(pContext, pMel->MEL);
            
            if (!pMelDef)
                continue;
            
            if (bTraceDetail) {
                sprintf (sTraceLine, "  mel: %5.5s (%4.4s/%8.8s)", pMel->MEL, pMel->MELANZ, pMel->MELDAT);
                TRACE_DETAIL(pAufenthalt, -1, sTraceLine);
            }
            
            if (pMelDef->ambFixPunkte > 0)
            { /* V2019.0 */
				double fixPunkte;
                pMel->ambFixPunkte = pMelDef->ambFixPunkte * pMel->nTagMelAnzRelevant;
                fixPunkte = pMel->ambFixPunkte;
                SP += fixPunkte;
                melabt = lkf_scoring_check_fcleist_index (&scoringAbt, pAufenthalt, pMel, ambabtmap);
                                
                if (bTraceDetail) {
                    sprintf (sTraceLine, "  fixpunkte: %.2f, melabt: %d", fixPunkte, melabt);
                    TRACE_DETAIL(pAufenthalt, -1, sTraceLine);
                }
                
                if (melabt > 0) {
                    lkf_scoring_divide_special (&scoringAbt, melabt, ambabtmap, fixPunkte);
                }
                else {
                    scoringAbt.spall += fixPunkte;
                }
                pMel->punkteRelevant = 1;
                continue;
            }
            
            if (melmel_relevant && !melmel_relevant[i])
                continue;
            
            if (pAufenthalt->psych_flag) {
                
 
                int melindex = pMel->nMelIndex;
 
				if (bTraceDetail) {
					sprintf(sTraceLine, "  psych=%d (melindex=%d)", pAufenthalt->psych_flag, melindex);
					TRACE_DETAIL(pAufenthalt, -1, sTraceLine);
				}

                if (IsConstMEL (pContext, CONSTMEL_AM060_7596, melindex)) {
                    if (!(pAufenthalt->psych_flag & 2)) continue;
                }
                else if (IsConstMEL (pContext, CONSTMEL_AM070_7597, melindex)) {
                    if (!(pAufenthalt->psych_flag & 4)) continue;
                }
                else if (IsConstMEL (pContext, CONSTMEL_AM080_7598, melindex)) {
                    if (!(pAufenthalt->psych_flag & 8)) continue;
                }
                else if (IsConstMEL (pContext, CONSTMEL_AM090_7601, melindex)) {
                    if (!(pAufenthalt->psych_flag & 16)) continue;
                }
                else if (IsConstMEL (pContext, CONSTMEL_AM110, melindex)) {
                    if (!(pAufenthalt->psych_flag & 32)) continue;
                }
                else if (modelljahr >= 2022 && IsConstMEL (pContext, CONSTMEL_AM150, melindex)) {
                    if (!(pAufenthalt->psych_flag & 256)) continue;
                }
                else if (IsConstMEL (pContext, CONSTMEL_AM120, melindex)) {
                    if (!(pAufenthalt->psych_flag & 64)) continue;
                }
                else if (IsConstMEL (pContext, CONSTMEL_AM130, melindex)) {
                    if (modelljahr >= 2017) {
                        if (!(pAufenthalt->psych_flag & 192)) continue;
                    }
                    else if (!(pAufenthalt->psych_flag & 128)) continue;
                }
                else
                    continue;
                
            }
            
            countedMELs++;
            
            if (!pMelDef)
                continue; /* sollte schon zuvor einen Fehler ausgeloest */
            
            if (!pMelDef->sAmbGruppe[0] || IsEmpty(pMelDef->sAmbGruppe, 8)) {
                if (bTraceDetail) {
                    sprintf (sTraceLine, "  keine AMBGRUPPE => Basisgruppe auf APG00.10");
                    TRACE_DETAIL(pAufenthalt, -1, sTraceLine);
                }
                basisGruppe = "APG00.10";
                continue;
            }
            
            pKnoten = GetAMBKnoten (pStamm, pMelDef->sAmbGruppe, sTraceLine);
            if (!pKnoten)
            {
                sprintf (pContext->errortext, "FEHLER beim Einlesen des Files '%8.8s': %s",
                             pMelDef->sAmbGruppe, sTraceLine);
                if (melmel_relevant)
                    LKF_FREE (melmel_relevant);
                if (bTraceInfo) {
                    TRACE_INFO(pAufenthalt, -1, pContext->errortext);
                }
                return -2;
            }
            else {
                if (bTraceDetail) {
                    sprintf (sTraceLine, "  AmbGruppe='%8.8s'", pMelDef->sAmbGruppe);
                    TRACE_DETAIL(pAufenthalt, -1, sTraceLine);
                }
            }
            
            act_knoten = 0;
            
            nSplitSteps = 0;
            
            while (pKnoten->nfl[act_knoten] != -99)
            {
                if (++nSplitSteps > 20)
                {
                    /* Nach 20 Schritten abbrechen - vermutlich Fehler im Stammdaten-File */
                    sprintf (pContext->errortext, "FEHLER beim Scoring-Split von '%8.8s', Abbruch nach 20 Schritten, vermutlich Fehler im Stammdaten-File",
                             pMelDef->sAmbGruppe);
                    if (melmel_relevant)
                        LKF_FREE (melmel_relevant);
                    if (bTraceInfo) {
                        TRACE_INFO(pAufenthalt, -1, pContext->errortext);
                    }
                    return -3;
                }
                
                scoring_split_result = scoring_split_2017(pAufenthalt, pKnoten, act_knoten, 0, &isAgr, pAufenthalt->HD, (int) i, bTraceDetail);
                
                if (bTraceDetail)
                {
                    sprintf (sTraceLine, "Scoring-AmbSplit, res=%d, neuer Knoten=%d", scoring_split_result, act_knoten);
                    TRACE_DETAIL (pAufenthalt, -1, sTraceLine);
                }
                if (scoring_split_result)
                    act_knoten = pKnoten->nfl[act_knoten]-1;
                else act_knoten = pKnoten->nfl[act_knoten];
                
            }

            strcpy (pAufenthalt->mel[i].ambGruppe, pMelDef->sAmbGruppe);
            pAufenthalt->mel[i].ambKnoten = pKnoten->knoten_char[act_knoten];
            
            mel_kp = pKnoten->tk_anteil[act_knoten];
            mel_lp = pKnoten->lk_anteil[act_knoten];
            
            if (i == 0 || KP < mel_kp)
            {
                KP = mel_kp;
            }
            
            if (i == 0 || mel_lp > maxLP || (mel_lp == maxLP && mel_kp > maxGrpKP ) || (mel_lp == maxLP && mel_kp == maxGrpKP  && memcmp (pAufenthalt->GRP, pMelDef->sAmbGruppe,8)>0)) {
                maxLP = mel_lp;
                maxGrpKP = mel_kp;
                maxMelIndex = i;
                pAufenthalt->maxMelIndex = (int) i;
                memcpy (pAufenthalt->GRP, pMelDef->sAmbGruppe, 8);
                pAufenthalt->KNT[0] = pKnoten->knoten_char[act_knoten];
                pAufenthalt->KNT[1] = ' ';
            
                if (bTraceDetail)
                {
                    sprintf (sTraceLine, "  #%d neuer Max-Index: lp = %5.1f, kp = %5.1f, KNT=%2.2s", i, mel_lp, mel_kp, pAufenthalt->KNT);
                    TRACE_DETAIL (pAufenthalt, -1, sTraceLine);
                }

            }
            else {
                if (bTraceDetail)
                {
                    sprintf (sTraceLine, "  #%d nicht MAX:       lp = %5.1f, kp = %5.1f, KNT=%c", i, mel_lp, mel_kp, pKnoten->knoten_char[act_knoten]);
                    TRACE_DETAIL (pAufenthalt, -1, sTraceLine);
                }
            }
            
            melabt = lkf_scoring_check_fcleist_index (&scoringAbt, pAufenthalt, pMel, ambabtmap);

            if (!memcmp (pMelDef->sAmbGruppe, "APG", 3)) {
                int found = 0;
                char cur[10];
                unsigned int g, k;
                int melanz;
                float hittingShare;

				if (modelljahr >= 2017)
                    melanz = pMel->nTagMelAnzRelevant;
                else
                    melanz = atoi(pMel->MELANZ);
                
                sprintf (cur, "%8.8s%c", pMelDef->sAmbGruppe, pKnoten->knoten_char[act_knoten]);
                
                for (g = 0; g < scoringAbt.groupCount; g++) {
                    if (!memcmp (scoringAbt.grp + 10 * g, cur, 9)) {
                        found = 1;
                        break;
                    }
                }
                
                if (!found) {
                    if (scoringAbt.groupCount < LKF_MAX_AMB_GRP - 1) {
                        if (scoringAbt.valid) {
                            memset (scoringAbt.mapApg + scoringAbt.groupCount * LKF_MAX_AMB_ABT, 0, sizeof(float) * scoringAbt.abtCount);

                            scoringAbt.apgcount[scoringAbt.groupCount] = melanz;
                            scoringAbt.lpg[scoringAbt.groupCount] = mel_lp;

                            if (melabt) {
                                
                                hittingShare = (float) melanz / melabt;
                                
                                for (k = 0; k < scoringAbt.abtCount; k++) {
                                    if (ambabtmap[k]) {
                                        scoringAbt.mapApg[scoringAbt.groupCount * LKF_MAX_AMB_ABT + k] = hittingShare;
                                    }
                                }
                            }
                            else if (pAufenthalt->abt_anz > 0){
                                hittingShare = (float) melanz / pAufenthalt->abt_anz;
                                
                                for (k = 0; k < scoringAbt.abtCount; k++) {
                                    scoringAbt.mapApg[scoringAbt.groupCount * LKF_MAX_AMB_ABT + k] = hittingShare;
                                }
                            }
                        }
                        melgrpIndex[i] = (int) scoringAbt.groupCount;
                        memcpy (scoringAbt.grp + scoringAbt.groupCount++ * 10, cur, 10);
                    }
                    LP += mel_lp;
          //          pAufenthalt->mel[i].melzu = mel_lp;
                }
                else if (scoringAbt.valid)
                {
                    melgrpIndex[i] = (int) g;
                    scoringAbt.apgcount[g] += melanz;

                    if (melabt) {
                        hittingShare = (float) melanz / melabt;

                        for (k = 0; k < scoringAbt.abtCount; k++) {
                            if (ambabtmap[k]) {
                                scoringAbt.mapApg[g * LKF_MAX_AMB_ABT + k] += hittingShare;
                            }
                        }
                    }
                    else if (pAufenthalt->abt_anz) {
                        hittingShare = (float) melanz / pAufenthalt->abt_anz;
                        for (k = 0; k < scoringAbt.abtCount; k++) {
                            scoringAbt.mapApg[g * LKF_MAX_AMB_ABT + k] += hittingShare;
                        }
                    }
                }
                pMel->punkteRelevant = 1;
            }
            else
            {
                double v;
                
                if (modelljahr >= 2017)
                    v = mel_lp * pMel->nTagMelAnzRelevant;
                else
                    v = mel_lp * atoi(pMel->MELANZ);
                
                LP += v;
                pAufenthalt->mel[i].melzu = (long) v;
                if (scoringAbt.valid) {
                    if (melabt > 0) {
                        lkf_scoring_divide_lst (&scoringAbt, melabt, ambabtmap, v);
                    }
                    else {
                        scoringAbt.lpall += v;
                    }
                }
                pMel->punkteRelevant = 1;
            }
        }
    }
    
    if (!countedMELs) {
        char errbuf[1024];
        pKnoten = GetAMBKnoten (pStamm, basisGruppe, errbuf);
        if (!pKnoten)
        {
            sprintf (pContext->errortext, "FEHLER beim Einlesen des Files '%8.8s': %s", basisGruppe, errbuf);
            if (bTraceInfo)
            {
                TRACE_INFO (pAufenthalt, -1, pContext->errortext);
            }
            return -2;
        }
        if (bTraceDetail) {
            sprintf (sTraceLine, "keine Count-MEL => knoten '%8.8s'", basisGruppe);
            TRACE_DETAIL(pAufenthalt, -1, sTraceLine);
        }
        
        act_knoten = 0;
        
        nSplitSteps = 0;
        
        while (pKnoten->nfl[act_knoten] != -99)
        {
            if (++nSplitSteps > 20)
            {
                /* Nach 20 Schritten abbrechen - vermutlich Fehler im Stammdaten-File */
                sprintf (pContext->errortext, "FEHLER beim Scoring-Split von '%8.8s', Abbruch nach 20 Schritten, vermutlich Fehler im Stammdaten-File",
                         basisGruppe);
                if (bTraceInfo)
                {
                    TRACE_INFO (pAufenthalt, -1, pContext->errortext);
                }
                return -3;
            }
            
            scoring_split_result = scoring_split_2017(pAufenthalt, pKnoten, act_knoten, 0, &isAgr, pAufenthalt->HD, -1, bTraceDetail);
            
            if (bTraceDetail)
            {
                sprintf (sTraceLine, "Scoring-AmbSplit, res=%d, neuer Knoten=%d", scoring_split_result, act_knoten);
                TRACE_DETAIL (pAufenthalt, -1, sTraceLine);
            }
            if (scoring_split_result)
                act_knoten = pKnoten->nfl[act_knoten]-1;
            else act_knoten = pKnoten->nfl[act_knoten];
            
        }

        KP = pKnoten->tk_anteil[act_knoten];
        LP = pKnoten->lk_anteil[act_knoten];
        memcpy (pAufenthalt->GRP, basisGruppe, 8);
        pAufenthalt->maxMelIndex = -1;
        pAufenthalt->KNT[0] = pKnoten->knoten_char[act_knoten];
        pAufenthalt->KNT[1] = ' ';
        pAufenthalt->LDFPLK = simCLng(LP);
        pAufenthalt->MELZU = 0;
		pAufenthalt->LDFPTK = simCLng(KP);
        pAufenthalt->SCSPEZ = simCLng(SP); /* V2019.0 */
        pAufenthalt->SCGES = pAufenthalt->LDFPLK + pAufenthalt->LDFPTK + pAufenthalt->MELZU + pAufenthalt->SCSPEZ;
        
        if (bTraceDetail) {
            sprintf (sTraceLine, "GRP=%8.8s, KNT=%c, PLK=%ld, PTK=%ld, MELZU=%ld, SPEZ=%ld, GES=%ld", basisGruppe, pKnoten->knoten_char[act_knoten], pAufenthalt->LDFPLK, pAufenthalt->LDFPTK, pAufenthalt->SCSPEZ, pAufenthalt->MELZU, pAufenthalt->SCGES );
            TRACE_DETAIL(pAufenthalt, -1, sTraceLine);
        }

        if (pAufenthalt->mel_anz) {
            int melanz = 0;
            for (i = 0; i < pAufenthalt->mel_anz; i++) {
                melanz += pAufenthalt->mel[i].nTagMelAnzRelevant;
            }
            if (melanz > 0) {
                for (i = 0; i < pAufenthalt->mel_anz; i++) {
                    pAufenthalt->mel[i].melzu = pAufenthalt->LDFPLK * pAufenthalt->mel[i].nTagMelAnzRelevant / melanz;
                }
            }
        }
	}
    else
    {
        int cma;
        pAufenthalt->MELZU = simCLng(LP-maxLP);
        pAufenthalt->LDFPLK = simCLng(maxLP);
		pAufenthalt->LDFPTK = simCLng(KP);
        pAufenthalt->SCSPEZ = simCLng(SP); /* V2019.0 */
        pAufenthalt->SCGES = pAufenthalt->LDFPLK + pAufenthalt->LDFPTK + pAufenthalt->MELZU + pAufenthalt->SCSPEZ;

        if (bTraceDetail) {
            sprintf (sTraceLine, "GRP=%8.8s, KNT=%c, PLK=%ld, PTK=%ld, MELZU=%ld, SPEZ=%ld, GES=%ld", pAufenthalt->GRP, pAufenthalt->KNT[0], pAufenthalt->LDFPLK, pAufenthalt->LDFPTK, pAufenthalt->SCSPEZ, pAufenthalt->MELZU, pAufenthalt->SCGES);
            TRACE_DETAIL(pAufenthalt, -1, sTraceLine);
        }

        
        if (lkf_scoring_amb_minimum_check(pAufenthalt, basisGruppe)) {
            if (scoringAbt.valid) {
                scoringAbt.valid = 0;
                for (i = 0; i < pAufenthalt->abt_anz; i++) pAufenthalt->abt[i].lp100 = (long) (scoringAbt.lps[i] * 100);
            }
        }
        else {
            for (i = 0; i < scoringAbt.groupCount; i++) {
                scoringAbt.ppl[i] = (long) (scoringAbt.lpg[i] / scoringAbt.apgcount[i]);
                scoringAbt.pBonus[i] = (long) (scoringAbt.lpg[i] - scoringAbt.ppl[i] * scoringAbt.apgcount[i]);
            }
        }
        
        for (i = 0; i < pAufenthalt->mel_anz; i++) {
            if (melgrpIndex[i] >= 0) {
                long v;
                cma = pAufenthalt->mel[i].nTagMelAnzRelevant;
                
                v = scoringAbt.ppl[melgrpIndex[i]] * cma;
                if (cma <= scoringAbt.pBonus[melgrpIndex[i]]) {
                    v += cma;
                    scoringAbt.pBonus[melgrpIndex[i]] -= cma;
                }
                else if (scoringAbt.pBonus[melgrpIndex[i]] > 0) {
                    v += scoringAbt.pBonus[melgrpIndex[i]];
                    scoringAbt.pBonus[melgrpIndex[i]] = 0;
                }
                
                pAufenthalt->mel[i].melzu = (long) v;
            }
        }
        
        pAufenthalt->mel[maxMelIndex].punkteRelevant = 2;
	}
   
    lkf_adjust_melzu_pkt (pAufenthalt);
/*
    mz = 0;
    for (i = 0; i < pAufenthalt->mel_anz; i++) {
        mz += pAufenthalt->mel[i].melzu;
    }
    
    if (pAufenthalt->LDFPLK + pAufenthalt->MELZU != mz) {
        printf ("%12.12s %10ld %10ld %10d %2d %8.8s\n", pAufenthalt->PATID, pAufenthalt->LDFPLK, pAufenthalt->MELZU, mz, pAufenthalt->mel_anz, pAufenthalt->GRP);
        mz += 0;
    }
*/
    if (melgrpIndex)
        LKF_FREE (melgrpIndex);
    
    
    pAufenthalt->SCGES_TKL = pAufenthalt->SCGES;
    
    if (pAufenthalt->abt_anz == 1)
    {
        pAufenthalt->abt[0].lp100 = (pAufenthalt->LDFPLK + pAufenthalt->MELZU + pAufenthalt->SCSPEZ /* V2019.2 */) * 100;
        pAufenthalt->abt[0].tp100 = pAufenthalt->LDFPTK * 100;
    }
    else if (pAufenthalt->abt_anz > 1) {
        if (scoringAbt.valid)
        {
            lkf_scoring_amb_divide(pAufenthalt, pAufenthalt->LDFPTK, 0);
            lkf_scoring_amb_adjust (pAufenthalt, &scoringAbt, pAufenthalt->LDFPLK + pAufenthalt->MELZU);
        }
        else {
            lkf_scoring_amb_divide(pAufenthalt, pAufenthalt->LDFPTK, pAufenthalt->LDFPLK + pAufenthalt->MELZU);
        }
    }
    
#ifdef LKF_SCORING_AMB_ABT_CHECK
    lkf_scoring_amb_check(pAufenthalt);
#endif
    
    if (melmel_relevant)
        LKF_FREE (melmel_relevant);

    return 0;
}


static LKF_SCORINGDETAILS_GROUP *diagDetailsForSubgruppe (LKF_SCORINGDETAILS *pDetails, int diagSubgruppeIndex) {
    unsigned int i, count;
    count = pDetails->pAufenthalt->diag_anz;
    for (i = 0; i < count; i++) {
        if (pDetails->pDiagGroup[i].subgruppeIndex == diagSubgruppeIndex)
            return pDetails->pDiagGroup + i;
    }
    return 0;
}

int lkf_scoring_compute (LKF_CONTEXT *pContext, LKF_AUFENTHALT *pAufenthalt)
{
    char errbuf[1024];
	unsigned int i, j;
	char act_ambulant;
	short diag_ambulant;
	short m7501_ok, m7506_ok, m7401_ok;
	short m7316_ok, m7321_ok;
	short m1255_ok;
	short mDB020_ok;
	short mAJ140_ok, mAK010_ok, mLH010_ok, mLH021_ok;
	short mAM100_ok; /* V15.0 */
	short mAM120_ok; /* V17.0 */
    short mAM130_ok; /* V21.1 */
    short mAM140_ok; /* V2022.0 */
	int m7591_pkt, m7592_pkt;

	short teil_akutpat;
	long spezial_tage;
	const SPEZ_LIST_STRING *pSpez;
	char kjnp_typ[2];
	int tmp_add;
	int tmp_found_spez;
	char tmp_spez_buf[20];
	long ngj_tagsatz;

	short stroke_missing_fc;                         
	short kjnp_missing_fc;
	short  psych_missing_fc; 
	short alk_missing_fc;

	short act_knoten;

	int psych_7591, psych_7592;
	
	int quickscore_enabled;

	SCORING_MELSTRING *tmp_mel = 0;
	SCORING_STRING4 *tmp_melanz = 0;

	ICD_LIST_STRING *pICD;

	LKF_AUFENTHALT_SUBGRUPPE *pSubgruppe;

	const LKF_KNOTEN *pKnoten;

	int melanz;
	MEL_ARRAY_STRUC *pMel = 0;

	const char *diag;

	double calc;
	double day_mean;
	int is_agr01;                     

	long max_score;
	long max_scges;

	unsigned int palliativ_tage;

	unsigned int nRemobTage;
	unsigned int nAkutGeriatrieTage;

	int scoring_split_result;

	int auf19_ok;

    int bTraceInfo;
    int bTraceDetail;
    
	int nSplitSteps;

	int bMelZuModus;
	unsigned int lAktDatum;
	long lAktMaxZuschlag;
	long lAktMaxZuschlagOp;
	unsigned int lGrpMelDat;
	char sTraceLine[1024];

	int nCurrMelGrpIndex;

	int bIgnoreSubgruppe;
    
    long *plMaxMELZU, *plCurMELZU; /* V22.1 */

    int modelljahr = pContext->modelljahr;

    int bModell2017p = (modelljahr >= 2017); /* V21.0 */
    int bSubgruppeTKL = (modelljahr >= 2011) ? !bModell2017p : 0;
    
    int bAmb = (pAufenthalt->BEHANDLUNGSART == 'I') && pAufenthalt->isAmb != 2;
    
	STAMM2001 *pStamm = pContext->pStamm2001;
    LKF_SCORINGDETAILS *pScoringDetails;
    LKF_SCORINGDETAILS_ABT *pScoringAbtDetails;

    if (pContext->noStamm) {
        compute_dates(pAufenthalt); /* V2023.2 */
        return 0;
    }
        
    
	pAufenthalt->pScoringContext = pContext;
	pContext->errortext[0] = 0;		

	nAkutGeriatrieTage = 0;
	nRemobTage = 0;
    
	if (pContext->pTraceSettings && pContext->pTraceSettings->nMBDSSeverity)
	{
 		if (pContext->pTraceSettings->sPATID[0])
		{
			unsigned int nPatIdLen = (unsigned int) strlen (pContext->pTraceSettings->sPATID);

			if (!memcmp (pContext->pTraceSettings->sPATID, pAufenthalt->PATID, nPatIdLen))
				pContext->nTraceAufenthalt = pContext->pTraceSettings->nMBDSSeverity;
		}
		else
			pContext->nTraceAufenthalt = pContext->pTraceSettings->nMBDSSeverity;
	}
    
    if (pContext->nTraceAufenthalt) {
        lkf_scoringdetails_alloc (pAufenthalt);
        pScoringDetails = pAufenthalt->pScoringDetails;
    }
    else {
        pScoringDetails = 0;
    }

	pAufenthalt->LDFPTK = 0;
	pAufenthalt->LDFPLK = 0;

	pAufenthalt->SCUTK = 0;
    pAufenthalt->SCUTK_DOUBLE = 0.0;
	pAufenthalt->SCULK = 0;
	pAufenthalt->VDZU = 0;
	pAufenthalt->INTZU = 0;
	pAufenthalt->MELZU = 0;
	pAufenthalt->SCSPEZ = 0;
	pAufenthalt->SCGES = 0;

	pAufenthalt->SCGES_TKL = 0;
	pAufenthalt->SCUTK_TKL = 0;

	memset (pAufenthalt->GRP, ' ', sizeof(pAufenthalt->GRP));
    
    bTraceInfo = IS_TRACE_INFO (pContext);
    bTraceDetail = IS_TRACE_DETAIL (pContext);
    
    if (bTraceInfo) {
        if (bAmb)
            TRACE_INFO(pAufenthalt, -1, "Scoring Punkteberechnung (spitalsambulant)");
        else
            TRACE_INFO(pAufenthalt, -1, "Scoring Punkteberechnung");
    }
    
    pAufenthalt->status |= 2;

	quickscore_enabled = (pContext->flags & LKFCONTEXT_ALLOWQUICKSCORE);

    if (quickscore_enabled && pAufenthalt->GEBDAT[0] == '#') {
		int alter;
		GetInt (pAufenthalt->GEBDAT + 1, 7, &alter);
		pAufenthalt->Alter = (short) alter;
	}
    else if (pAufenthalt->pScoringContext->isBM) {
        pAufenthalt->Alter = (short) atoi (pAufenthalt->ALTERSGRUPPE);
    }
	else pAufenthalt->Alter = (short) Util_Alter(pAufenthalt);

	memset (pAufenthalt->KNT, ' ', sizeof (pAufenthalt->KNT));

	if (pAufenthalt->nStatus == 0)
	{
		pAufenthalt->GESVD = 0;
		pAufenthalt->GESVD_KORR = 0;

		memcpy(pAufenthalt->GRP, "        ", 8);

		if (bTraceInfo)
			TRACE_INFO (pAufenthalt, -1, "Programmfehler - fuer den Aufenthalt wurde kein Plaus-Check aufgerufen!");

		return -1;
	}
	else
		pAufenthalt->nStatus = 2;

	compute_dates (pAufenthalt);

	if (!bAmb && pAufenthalt->ENTART == 'V')
	{
		if (bTraceDetail)
		{
			TRACE_DETAIL(pAufenthalt, -1, "Entlassungsart 'V' -> keine Punkte");
		}

		memcpy(pAufenthalt->GRP, "VERBLEIB", 8);
		return 0;		
	}


	/* Immer Tagesklinik, aber in normalen Werten gespeichert */
	pAufenthalt->berechnung_tagesklinik = 0;	

	pAufenthalt->psych_mode = 0;
	if (!bAmb && pAufenthalt->AUFART == 'H') {

		sprintf (tmp_spez_buf, ".....%4.4s%8.8s", pAufenthalt->KANR, pAufenthalt->abt[0].ABTFC);
		
		pSpez = SpezInArray (pAufenthalt->pScoringContext, tmp_spez_buf);
		if (pSpez && !ABTFC_Spez_Check (pAufenthalt, 0, pSpez, 0)) pSpez = 0;
		if (pSpez) {
			if ((*pSpez)[0] == 'P') {
				if (!memcmp ((*pSpez) + 27, "TK", 2)) pAufenthalt->psych_mode = 1;
				else if (!memcmp ((*pSpez) + 27, "TS", 2)) {
					if (modelljahr == 2013)
					{ /* V17.1 */
						if (!memcmp (pAufenthalt->AUFDAT, "2013", 4)) pAufenthalt->psych_mode = 23;
						else
						{
							if (!memcmp (pAufenthalt->abt[0].ABTFC + 4, "85", 2)) pAufenthalt->psych_mode = 2;
							else if (!memcmp (pAufenthalt->abt[0].ABTFC + 4, "86", 2)) pAufenthalt->psych_mode = 3;
							else pAufenthalt->psych_mode = 23;
						}
					} 
					else if (modelljahr > 2013) pAufenthalt->psych_mode = 23; /* V17.1 */
					else if (!memcmp (pAufenthalt->abt[0].ABTFC + 4, "85", 2)) pAufenthalt->psych_mode = 2;
					else pAufenthalt->psych_mode = 3;
				}
			}
			if ((*pSpez)[0] == 'G') {
				if (!memcmp ((*pSpez) + 27, "TK", 2)) pAufenthalt->psych_mode = 4;
			}
							
			if (modelljahr >= 2011 && (*pSpez)[0] == 'O' && !memcmp ((*pSpez) + 27, "TK", 2))
				pAufenthalt->psych_mode = 5; /* V15.0: PSYCHOSOMATIK */

			if (modelljahr >= 2013 && (*pSpez)[0] == 'J' && !memcmp ((*pSpez) + 27, "TK", 2))
				pAufenthalt->psych_mode = 6; /* V17.0 */

            if (modelljahr >= 2017 && (*pSpez)[0] == 'J' && !memcmp ((*pSpez) + 27, "TS", 2))
                pAufenthalt->psych_mode = 7; /* V21.1 */
        }

		if (bTraceDetail)
		{
			strcpy (sTraceLine, "Aufnahmeart HALBSTATIONAER");
			switch (pAufenthalt->psych_mode)
			{
				default:
				case 0:
					strcat (sTraceLine, ", aber keine spezielle halbstaetionaere Betreuung.");
					break;
				case 1:
					strcat (sTraceLine, ", tagesklinische Psych-Abteilung.");
					break;
				case 2:
					strcat (sTraceLine, ", tagesstrukturierende Psych-Abteilung (85).");
					break;
				case 3:
				case 23:
					strcat (sTraceLine, ", tagesstrukturierende Psych-Abteilung.");
					break;
				case 4:
					strcat (sTraceLine, ", AG/R-tagesklinische Abteilung.");
					break;
				case 5: /* V15.0 */
					strcat (sTraceLine, ", tagesklinische psychosomatische Abteilung.");
					break;

			}
			TRACE_DETAIL (pAufenthalt, 0, sTraceLine);
		}
	}

	if (quickscore_enabled) {
		int vd;

		if (pAufenthalt->AUFDAT[0] == '#') GetInt (pAufenthalt->AUFDAT + 1, 7, &vd);
		else {
			long lAufDat, lEntDat;
			lAufDat = lkf_dat2int (pAufenthalt->AUFDAT);
			lEntDat = lkf_dat2int (pAufenthalt->ENTDAT);

			if (lAufDat && lEntDat) vd = (int) (lEntDat - lAufDat);
			else vd = -1;
		}
		pAufenthalt->GESVD = (short) vd;
		pAufenthalt->GESVD_KORR = 0;

		lkf_quickscore_compute_abtdats (pAufenthalt);

		if (pAufenthalt->abt_anz > 0) {
			auf19_ok = 0;
			for (i = 0; i < pAufenthalt->abt_anz; i++) {
				if (!memcmp (pAufenthalt->abt[i].ABTFC, "19", 2)) {
					pAufenthalt->GESVD -= pAufenthalt->abt[i].ABTVD;
					pAufenthalt->GESVD_KORR += pAufenthalt->abt[i].ABTVD;
					pAufenthalt->abt[i].ABTVD_KORR = pAufenthalt->abt[i].ABTVD;
					pAufenthalt->abt[i].ABTVD = 0;
				}
				else auf19_ok = 1;

				if (modelljahr >= 2010 && !memcmp (pAufenthalt->abt[i].ABTFC, UNTERBRECHUNG_ABTFC, UNTERBRECHUNG_LEN))
				{
					pAufenthalt->GESVD -= pAufenthalt->abt[i].ABTVD;
					pAufenthalt->abt[i].ABTVD_KORR = pAufenthalt->abt[i].ABTVD;
					pAufenthalt->abt[i].ABTVD = 0;
				}
                pAufenthalt->abt[i].lp100 = pAufenthalt->abt[i].tp100 = 0;

			}
		}
		else auf19_ok = 1;

	}
	else {
		pAufenthalt->GESVD = 0;
		pAufenthalt->GESVD_KORR = 0;
		auf19_ok = 0;

        if (pAufenthalt->isAmb && pContext->modelljahr >= 2020) {
            for (i = 0; i < pAufenthalt->abt_anz; i++)
            {
                pAufenthalt->abt[i].lp100 = pAufenthalt->abt[i].tp100 = 0;
            }
            auf19_ok = 1;
        }
        else {
            for (i = 0; i < pAufenthalt->abt_anz; i++)
            {
                if (memcmp (pAufenthalt->abt[i].ABTFC, "19",2) && memcmp(pAufenthalt->abt[i].ABTFC, "        ", 8))
                {

                    if (pAufenthalt->pScoringContext->modelljahr >= 2010 && !memcmp (pAufenthalt->abt[i].ABTFC, UNTERBRECHUNG_ABTFC, UNTERBRECHUNG_LEN))
                    {
                        pAufenthalt->abt[i].ABTVD_KORR = pAufenthalt->abt[i].ABTVD;
                        pAufenthalt->abt[i].ABTVD = 0;
                    }
                    else
                    {
                        auf19_ok = 1;
                        pAufenthalt->abt[i].ABTVD = Util_Diff_Tage(pAufenthalt, (short) i);
                        pAufenthalt->GESVD += pAufenthalt->abt[i].ABTVD;
                    }
                }
                else
                {
                    pAufenthalt->GESVD_KORR += pAufenthalt->abt[i].ABTVD;
                    pAufenthalt->abt[i].ABTVD_KORR = pAufenthalt->abt[i].ABTVD;
                    pAufenthalt->abt[i].ABTVD = 0;
                }
                pAufenthalt->abt[i].lp100 = pAufenthalt->abt[i].tp100 = 0;
            }
        }
    }
    
    if (pScoringDetails) {
        for (i = 0; i < pAufenthalt->abt_anz; i++) {
            pScoringDetails->pAbt[i].abtVD = pAufenthalt->abt[i].ABTVD + pAufenthalt->abt[i].ABTVD_KORR;
        }
    }
    
    if (pAufenthalt->isAmb && pAufenthalt->pScoringContext->modelljahr >= 2018)
        pAufenthalt->GESVD = pAufenthalt->GESVD_KORR = 0;

	if (pAufenthalt->FLAG >= '4' && (!pAufenthalt->akerr_count || pAufenthalt->plaus_err)) {

		if (bTraceDetail)
		{
			sprintf (sTraceLine, "Plaus-Flag '%c', keine Punkte wegen Plaus-Fehlers", pAufenthalt->FLAG);
			TRACE_DETAIL (pAufenthalt, -1, sTraceLine);
		}

		memmove(pAufenthalt->GRP, "FEHLER  ", 8);
		return 0;
	}
    
    if (pContext->modelljahr >= 2018 && !memcmp (pAufenthalt->KOST, "BN", 2)) {
        memmove (pAufenthalt->GRP, "HKLE    ", 8);
        return 0;
        
    }

    if (bAmb) {
        return lkf_scoring_compute_amb (pAufenthalt);
    }
    
	if (auf19_ok == 0) {
		if (bTraceDetail)
		{
			TRACE_DETAIL (pAufenthalt, -1, "PFLEGE-Aufenthalt, keine Punkte");
		}
		memmove (pAufenthalt->GRP, "PFLEGE  ", 8);
		return 0;
	}
	
	if (pAufenthalt->FLAG > '1' && pAufenthalt->isAmb != 2) {

		ICD_LIST_STRING *pDiaglist;
        pDiaglist = FindDiag (pStamm, pAufenthalt->HD);

        act_ambulant = pDiaglist ? (*pDiaglist)[42] : ' ';
		if (act_ambulant == 'J') {
			diag_ambulant = 1;

            if (pAufenthalt->mel_anz) {
                if (modelljahr >= 2017) {
                    for (i = 0; i < pAufenthalt->mel_anz; i++) {
                        
                        MEL_ARRAY_STRUC *pMel;
                        if (pAufenthalt->mel[i].nMelIndex < 0)
                            continue;
                        
                        pMel = GetMELArrayFromIndex(pContext, pAufenthalt->mel[i].nMelIndex);
                        if (pMel->cStatVerpflichtend) {
                            diag_ambulant = 0;
                            break;
                        }
                    }
                }
                else {
                    diag_ambulant = 0;
                }
            }
			if (diag_ambulant) {
			
				for (i = 0; diag_ambulant && i < pAufenthalt->diag_anz; i++) {
                    pDiaglist = FindDiag (pStamm, pAufenthalt->diag[i].DIAG);

					if (pDiaglist) 
					{
						act_ambulant = (*pDiaglist)[42];

						if (act_ambulant != 'J' && !memcmp ((*pDiaglist) + 9, "HDG", 3))
							diag_ambulant = 0;
					}
				}
				
				if (diag_ambulant) {
					if (bTraceDetail)
					{
						TRACE_DETAIL(pAufenthalt,-1, "nur ambulante Diagnosen, daher keine Punkte (DIAG0PKT)");
					}
					memcpy (pAufenthalt->GRP, "DIAG0PKT", 8);
					return 0;
				}
			
			}
		}
	}

	/* Gltiger Patient mit speziellen Aufnahmearten H, P, L -> Beenden von Scoring_Pat */

	switch (pAufenthalt->AUFART) {

		case 'P':
			
			if (bTraceDetail)
			{
				TRACE_DETAIL(pAufenthalt, -1, "Aufnahmeart PFLEGE, keine weitere Scoring-Berechnung");
			}
			
			memcpy(pAufenthalt->GRP, "PFLEGE  ", 8);
			lkf_scoring_tkl (pContext, pAufenthalt);
			return 0;

		case 'L':
			if (bTraceDetail)
			{
				TRACE_DETAIL(pAufenthalt, -1, "Aufnahmeart LANGZEIT, keine weitere Scoring-Berechnung");
			}
			memcpy(pAufenthalt->GRP, "LANGZEIT", 8);
			lkf_scoring_tkl (pContext, pAufenthalt);
			return 0;
		default : ;/* alle anderen Aufnahmearten -> weitermachen */
	}

	/* Neurorehab/Geriatrie/KJNP-Patienten */
	/* LKF99: PsychMEL-Patienten */

	teil_akutpat = 0;
	spezial_tage = 0;
	m7501_ok = 0;
	m7506_ok = 0;
	m7401_ok = 0;
	m7316_ok = m7321_ok = 0;
	m1255_ok = (modelljahr < 2012) ? 0 : -1; /* V16.0 - anderer Check ab Modell 2012 */
	mDB020_ok= 0;
	mAM100_ok = (modelljahr >= 2011) ? 0 : -1;
    mAM120_ok = (modelljahr >= 2013)  ? 0 : -1;
    mAM130_ok = (modelljahr >= 2017)  ? 0 : -1;
    mAM140_ok = (modelljahr >= 2022) ? 0 : -1;
    pSpez = 0;
	tmp_found_spez = 0;

	palliativ_tage = 0;

	mAJ140_ok = mAK010_ok = mLH010_ok = mLH021_ok = mDB020_ok = (modelljahr == 2010 || modelljahr == 2011) ? 0 : (-1);

	if (SpezAnz (pContext)) for (i = 0; i < pAufenthalt->abt_anz; i++) {

        pScoringAbtDetails =  pScoringDetails ? pScoringDetails->pAbt + i : 0;
        
		tmp_add = 1;

		if (!IsEmpty(pAufenthalt->abt[i].ABTFC, 8)) {

			sprintf (tmp_spez_buf, ".....%4.4s%8.8s", pAufenthalt->KANR, pAufenthalt->abt[i].ABTFC);
			pSpez = SpezInArray (pContext, tmp_spez_buf);

			if (pSpez && !ABTFC_Spez_Check (pAufenthalt, (int) i, pSpez, 0)) pSpez = 0;
            if (pSpez) {
				switch ((*pSpez)[0]) {

					case 'N': /* (1) Neurorehab-Patient */
						tmp_found_spez = 1;
						memcpy (pAufenthalt->GRP, "NEURO   ", 8);
                        if (pScoringAbtDetails) {
                            strcpy (pScoringAbtDetails->spezBereich, "NEURO");
                        }
						break;

					case 'G': /* (2) Geriatrie-Patient */
                        
                        if (!memcmp ((*pSpez) + 27, "TK", 2) && modelljahr >= 2018) {
                            pSpez = 0;
                            break;
                        }
                        
						if (pAufenthalt->psych_mode == 4) { pSpez = 0; break; }
						memcpy (pAufenthalt->GRP, "GERIAT  ", 8);
						tmp_found_spez = 1;
                        if (pScoringAbtDetails) {
                            strcpy (pScoringAbtDetails->spezBereich, "GERIAT");
                        }
						break;

					case 'J': /* (3) KJNP-Patient */

                        if (modelljahr >= 2017)
                        { /* V17.0 */
                            if (!memcmp ((*pSpez) + 27, "TK", 2) || !memcmp ((*pSpez) + 27, "TS", 2)) {
                                pSpez = 0;
                                break;
                            }
                        }
                        else if (modelljahr >= 2013)
						{ /* V17.0 */
							if (!memcmp ((*pSpez) + 27, "TK", 2)) {
								pSpez = 0;
								break;
							}
						}

                        if (modelljahr >= 2010)
							memcpy (pAufenthalt->GRP, "KJP     ", 8);
                        else
							memcpy (pAufenthalt->GRP, "KJNP    ", 8);
                        if (pScoringAbtDetails) {
                            strcpy (pScoringAbtDetails->spezBereich, "KJP");
                        }

						tmp_found_spez = 1;

						if (modelljahr < 2013) {
							kjnp_typ[0] = (*pSpez)[27];
							kjnp_typ[1] = (*pSpez)[28];
							
							if (((kjnp_typ[0] != 'I' && kjnp_typ[0] != 'R') || kjnp_typ[1] != ' ') &&
							((kjnp_typ[1] != 'I' && kjnp_typ[1] != 'R') || kjnp_typ[0] != ' ')) {
						
								tmp_add = 0;
								if (kjnp_typ[0] == '0') switch (kjnp_typ[1]) {
									case '3': m7506_ok = -1; 
									case '1': m7501_ok = -1; break;
									case '2': m7506_ok = -1; break;
									default: break;
								}
							}
						}
						break;						

					case 'P': /* (4) PsychMEL-Patient */
						
						if (memcmp ((*pSpez) + 27, "TK", 2) && memcmp ((*pSpez) + 27, "TS", 2)) {
							tmp_add = 0;
						}
						else pSpez = 0;

						if (!memcmp (pAufenthalt->abt[i].ABTFC + 2, "6215", 4)) {
							m7316_ok = m7321_ok = -1;			
						}

						break;

					case 'O': /* V15.0: Psychosomatik */
                        
                        if (modelljahr >= 2022) {
                            if (!memcmp ((*pSpez) + 27, "ET", 2)) {
                                pSpez = 0;
                                break;
                            }

                            if (pAufenthalt->psych_mode == 5) { pSpez = 0; break; }
                            
                            if (!memcmp ((*pSpez) + 27, "ES", 2))
                            {
                                tmp_add = 0;
                                mAM100_ok = 1;
                                break;
                            }
                            else if (!memcmp ((*pSpez) + 27, "KS", 2))
                            {
                                tmp_add = 0;
                                mAM140_ok = 1;
                                break;
                            }
                            memcpy (pAufenthalt->GRP, "       ", 8);
                            tmp_found_spez = 0;
                        }
                        else {
                            if (!memcmp ((*pSpez) + 27, "TK", 2) && modelljahr >= 2018) {
                                pSpez = 0;
                                break;
                            }

                            if (pAufenthalt->psych_mode == 5) { pSpez = 0; break; }
                            
                            if ( (*pSpez)[27] == '0' && ( (*pSpez)[28] == '1' || (*pSpez)[28] == '2'))
                            {
                                tmp_add = 0;
                                mAM100_ok = 1;
                                break;
                            }
                            memcpy (pAufenthalt->GRP, "       ", 8);
                            tmp_found_spez = modelljahr >= 2012 ? 0 : 1; /* V17.1 */
                        }

						break;


					case 'S': /* (5) stroke-unit Patient */
						tmp_add = 0;

						if (modelljahr >= 2010)
						{
							if ((pAufenthalt->abt[i].ABTVD > 0 || pAufenthalt->ENTART == 'S' || pAufenthalt->ENTART == 'T' || pAufenthalt->ENTART == LKF_ENTART_REVERS) && (*pSpez)[27] == '0' && (*pSpez)[28] == '1') m7401_ok = (-1);
						}
						else if ((*pSpez)[27] == '0' && (*pSpez)[28] == '1') m7401_ok = (-1);
						break;
				
					case 'M':
						tmp_add = 0;
						if (modelljahr >= 2010)
						{ /* Mehrfachnennungen in der Spezlist mglich */
							tmp_spez_buf[0] = 'M';
							pSpez = FirstSpezInArray (pContext, pSpez, tmp_spez_buf); 

							do
							{
								if ((*pSpez)[27] == '0')
								{
									switch ((*pSpez)[28])
									{
									case '1':
										m1255_ok = (-1);
										break;
									case '2':
										mAJ140_ok = -1;
										break;
									case '3':
										mAK010_ok = -1;
										break;
									case '4':
										mDB020_ok = -1;
										break;
									case '5':
										mLH010_ok = -1;
										break;
									case '6':
										mLH021_ok = -1;
										break;
									default:
										break;
									}
								}
								pSpez = NextSpezInArray (pContext, pSpez, tmp_spez_buf);
							} while (pSpez);
						}
						else 
							if ((*pSpez)[27] == '0' && (*pSpez)[28] == '1') m1255_ok = (-1);
						break;

					case 'L': /* (6) Palliativmedizinische Einrichtungen */
						memcpy (pAufenthalt->GRP, "PALLIAT ", 8);
                        if (pScoringAbtDetails) {
                            strcpy (pScoringAbtDetails->spezBereich, "PALLIAT");
                        }
						tmp_found_spez = 1;
						tmp_add = 2; /* Spezialrechnung fr degressiven Zuschlag */
						break;

					case  'A':
						tmp_add = 0;
						if ((*pSpez)[27] == '0') switch ((*pSpez)[28]) {
							case '3': m7316_ok = -1;
							case '2': m7321_ok = -1; break;
							case '1': m7316_ok = -1; break;
							 default: break;
						}
						break;

					case 'R':
						if (!memcmp ((*pSpez) + 27, "RN", 2))
						{
							memcpy (pAufenthalt->GRP, "REMOB   ", 8); /* V18.0: 5 -> 8 */
                            if (pScoringAbtDetails) {
                                strcpy (pScoringAbtDetails->spezBereich, "REMOB");
                            }

                            tmp_add = 4;
							tmp_found_spez = 1;
						}
						break;

					default: pSpez = 0; break;
				}
			}

			if (pSpez && tmp_add) {
                if (pScoringAbtDetails) {
                    if (!tmp_found_spez) {
                        pScoringAbtDetails->spezBereich[0] = (*pSpez)[0];
                    }
                }
				spezial_tage += pAufenthalt->abt[i].ABTVD;
				ngj_tagsatz = (long) simVal ((*pSpez) + 29, 6);
				if (pAufenthalt->abt[i].ABTVD == 0 && pAufenthalt->GESVD == 0) 
					 pAufenthalt->SCSPEZ += ngj_tagsatz;
				else {
			
					if (tmp_add == 2)
					{
						static const unsigned int add_palliativ_tagsatz_ab2009[20] = {462, 442, 424, 407, 391, 376, 363, 350, 339, 328, 318, 308, 299, 290, 282, 275, 267, 261, 254, 248};
                        static const unsigned int add_palliativ_tagsatz_ab2017[20] = {655, 626, 600, 576, 554, 534, 515, 497, 480, 465, 450, 437, 424, 412, 400, 389, 379, 369, 360, 351};
                        unsigned int nPalliativ_Tage_MaxPkt;
						unsigned int nPalliativ_Tage_MinPkt;
						unsigned int nPalliativ_Max_Zuschlag;
						unsigned int nPalliativ_Min_Zuschlag;
						const unsigned int *aPalliativ_Degressiv;
						unsigned int abtbd = (unsigned int) pAufenthalt->abt[i].ABTVD;
						unsigned int nPalliativDegressivPunkte;
						unsigned int nPalliativDegressivTage;

                        nPalliativ_Tage_MaxPkt = 21;
                        nPalliativ_Tage_MinPkt = 42;
                        if (modelljahr >= 2017) {
                            nPalliativ_Max_Zuschlag = 686;
                            nPalliativ_Min_Zuschlag = 343;
                            aPalliativ_Degressiv = add_palliativ_tagsatz_ab2017;
                        }
                        else {
                            nPalliativ_Max_Zuschlag = 484;
                            nPalliativ_Min_Zuschlag = 242;
                            aPalliativ_Degressiv = add_palliativ_tagsatz_ab2009;
                        }
                        

						if (palliativ_tage < nPalliativ_Tage_MaxPkt) {

							unsigned int day_count;

							if (!palliativ_tage)  day_count = (abtbd > nPalliativ_Tage_MaxPkt ? nPalliativ_Tage_MaxPkt : abtbd);
							else if (abtbd > nPalliativ_Tage_MaxPkt - palliativ_tage) day_count = nPalliativ_Tage_MaxPkt - palliativ_tage;
							else day_count = abtbd;
                            
							pAufenthalt->SCSPEZ += nPalliativ_Max_Zuschlag * day_count;

                            if (pScoringAbtDetails)
                                pScoringAbtDetails->spezPunkte = nPalliativ_Max_Zuschlag * day_count;
                            
							if (bTraceDetail)
							{
								sprintf (sTraceLine, "Max. Palliativzuschlag (%d Punkte) fr %d Tage: %d (SpezGesamt=%ld)", nPalliativ_Max_Zuschlag, day_count, nPalliativ_Max_Zuschlag * day_count, pAufenthalt->SCSPEZ);
								TRACE_DETAIL (pAufenthalt, (int) i, sTraceLine);
							}
						}
						
						abtbd += palliativ_tage;
						
						if (abtbd > nPalliativ_Tage_MaxPkt) {
							unsigned int tag_limit;
							if (palliativ_tage >= nPalliativ_Tage_MaxPkt) tag_limit = palliativ_tage;
							else tag_limit = nPalliativ_Tage_MaxPkt;
							

							if (abtbd >= nPalliativ_Tage_MinPkt) {

								abtbd -= nPalliativ_Tage_MinPkt-1;

								if (abtbd > (unsigned int) pAufenthalt->abt[i].ABTVD)
									abtbd = (unsigned int) pAufenthalt->abt[i].ABTVD;
								
								pAufenthalt->SCSPEZ += abtbd * nPalliativ_Min_Zuschlag;
                                if (pScoringAbtDetails)
                                    pScoringAbtDetails->spezPunkte += abtbd * nPalliativ_Min_Zuschlag;

								if (bTraceDetail)
								{
									sprintf (sTraceLine, "Min. Palliativzuschlag (%d Punkte) fr %d Tage: %d (SpezGesamt=%ld)", nPalliativ_Min_Zuschlag, abtbd, nPalliativ_Min_Zuschlag * abtbd, pAufenthalt->SCSPEZ);
									TRACE_DETAIL (pAufenthalt, (int) i, sTraceLine);
								}

								abtbd = nPalliativ_Tage_MinPkt - 1;
							}
							
							nPalliativDegressivTage = 0;
							nPalliativDegressivPunkte = 0;
							while (abtbd > tag_limit) {
								nPalliativDegressivPunkte += aPalliativ_Degressiv[abtbd - nPalliativ_Tage_MaxPkt - 1];
								--abtbd;
								nPalliativDegressivTage++;
							}
							pAufenthalt->SCSPEZ += nPalliativDegressivPunkte;
                            if (pScoringAbtDetails)
                                pScoringAbtDetails->spezPunkte += nPalliativDegressivPunkte;

							if (bTraceDetail && nPalliativDegressivTage > 0)
							{
								sprintf (sTraceLine, "Degressiver Palliativzuschlag fr %d Tage: %d (SpezGesamt=%ld)", nPalliativDegressivTage, nPalliativDegressivPunkte, pAufenthalt->SCSPEZ);
								TRACE_DETAIL (pAufenthalt, (int) i, sTraceLine);
							}
						}

						palliativ_tage += pAufenthalt->abt[i].ABTVD;

					}
					else if (tmp_add == 3)
					{ /* V11.0/doch nicht: Akutgeriatrie wird fuer max. 28 Tage bepunktet */
					    unsigned int nAbtVd = (unsigned int) pAufenthalt->abt[i].ABTVD + nAkutGeriatrieTage;
						if (nAbtVd > 28) nAbtVd = 28;
						if (nAbtVd > nAkutGeriatrieTage)
						{
							nAbtVd -= nAkutGeriatrieTage;
							nAkutGeriatrieTage += nAbtVd;
						}
						else
							nAbtVd = 0;

						pAufenthalt->SCSPEZ += ngj_tagsatz * nAbtVd;
                        if (pScoringAbtDetails)
                            pScoringAbtDetails->spezPunkte += ngj_tagsatz * nAbtVd;;

						if (bTraceDetail)
						{
							sprintf (sTraceLine, "Spezialzuschlag (%ld Punkte) fr %d Tage: %ld (SpezGesamt=%ld)", ngj_tagsatz, nAbtVd, ngj_tagsatz * nAbtVd, pAufenthalt->SCSPEZ);
							TRACE_DETAIL (pAufenthalt, (int) i, sTraceLine);
						}
					}
					else if (tmp_add == 4)
					{ /* Remob wird fuer max. 28 Tage bepunktet */
					    unsigned int nAbtVd = (unsigned int) pAufenthalt->abt[i].ABTVD + nRemobTage;
						if (nAbtVd > 28) nAbtVd = 28;
						if (nAbtVd > nRemobTage)
						{
							nAbtVd -= nRemobTage;
							nRemobTage += nAbtVd;
						}
						else
							nAbtVd = 0;

						pAufenthalt->SCSPEZ += ngj_tagsatz * nAbtVd;
                        if (pScoringAbtDetails)
                            pScoringAbtDetails->spezPunkte += ngj_tagsatz * nAbtVd;;
						if (bTraceDetail)
						{
							sprintf (sTraceLine, "Spezialzuschlag (%ld Punkte) fr %d Tage: %ld (SpezGesamt=%ld)", ngj_tagsatz, nAbtVd, ngj_tagsatz * nAbtVd, pAufenthalt->SCSPEZ);
							TRACE_DETAIL (pAufenthalt, (int) i, sTraceLine);
						}
					}
					else 
					{
						pAufenthalt->SCSPEZ += ngj_tagsatz * pAufenthalt->abt[i].ABTVD;
                        if (pScoringAbtDetails)
                            pScoringAbtDetails->spezPunkte += ngj_tagsatz * pAufenthalt->abt[i].ABTVD;
						if (bTraceDetail)
						{
							sprintf (sTraceLine, "Spezialzuschlag (%ld Punkte) fr %ld Tage: %ld (SpezGesamt=%ld)", ngj_tagsatz, pAufenthalt->abt[i].ABTVD, ngj_tagsatz * pAufenthalt->abt[i].ABTVD, pAufenthalt->SCSPEZ);
							TRACE_DETAIL (pAufenthalt, (int) i, sTraceLine);
						}
					}
				}
			}
		}
        
        
	}
	if (tmp_found_spez) {

		if (pAufenthalt->GESVD - spezial_tage == 0 && !m7501_ok && !m7506_ok) {
			pAufenthalt->SCGES = pAufenthalt->SCSPEZ;
			lkf_scoring_tkl (pContext, pAufenthalt);
            if (pScoringDetails) {
                unsigned int i;
                for (i = 0; i < pAufenthalt->mel_anz; i++) {
                    pScoringDetails->pGroup[i].inactive = 1;
                }
            }
			return 0;
		}
		else {
			pAufenthalt->GESVD -= spezial_tage;
			pAufenthalt->GESVD_KORR += spezial_tage;
			teil_akutpat = -1;
		}
	}
    
    if (pScoringDetails) {
        pScoringDetails->vd = pAufenthalt->GESVD;
    }

	/* Check auf MEL 7501 bzw. MEL 7506 */

	kjnp_missing_fc = 0;
	psych_missing_fc = 0;
	stroke_missing_fc = 0;
	alk_missing_fc = 0;

	if (pAufenthalt->mel_anz)
	{

		tmp_mel = (SCORING_MELSTRING *) LKF_CALLOC (sizeof(SCORING_MELSTRING), pAufenthalt->mel_anz);
		tmp_melanz = (SCORING_STRING4 *) LKF_CALLOC (4, pAufenthalt->mel_anz);
	
		for (i = 0; i < pAufenthalt->mel_anz; i++) 
		{
			memcpy (tmp_mel[i], pAufenthalt->mel[i].MEL, LKF_AUFENTHALT_NEWMEL_LEN);
			memcpy (tmp_melanz[i], pAufenthalt->mel[i].MELANZ, 4);
		}
	}
	
	m7591_pkt = m7592_pkt = 0;

	for (i = 0; i < pAufenthalt->mel_anz; i++) {

		if (!quickscore_enabled && mel_during_abt19 (pAufenthalt, (int)i)) {
			InactivateMel (pAufenthalt, i);
		}
		else if ((pAufenthalt->mel[i].MEL_RELEVANT == 'N' || pAufenthalt->mel[i].MEL_RELEVANT == 'K')  && pAufenthalt->mel[i].nTagMelAnzRelevant == 0)
		{
			InactivateMel (pAufenthalt, i);
		}
		else if (pAufenthalt->mel[i].nIgnoreMel)
		{
			InactivateMel (pAufenthalt, i);
		}
		else
		{
			int nMelIndex = pAufenthalt->mel[i].nMelIndex;
			if (pAufenthalt->psych_mode) {
				if ((pAufenthalt->psych_mode == 1 && (!IsConstMEL (pContext, CONSTMEL_AM060_7596, nMelIndex) && (modelljahr < 2018 || (!IsConstMEL (pContext, CONSTMEL_AM070_7597, nMelIndex) && !IsConstMEL (pContext, CONSTMEL_AM080_7598, nMelIndex)))))  ||
					(pAufenthalt->psych_mode == 2 && !IsConstMEL (pContext, CONSTMEL_AM070_7597, nMelIndex)) ||
					(pAufenthalt->psych_mode == 3 && !IsConstMEL (pContext, CONSTMEL_AM080_7598, nMelIndex)) ||
					(pAufenthalt->psych_mode == 23 && !IsConstMEL (pContext, CONSTMEL_AM070_7597, nMelIndex) && !IsConstMEL (pContext, CONSTMEL_AM080_7598, nMelIndex)) || /* V17.1 */
					(pAufenthalt->psych_mode == 4 && !IsConstMEL (pContext, CONSTMEL_AM090_7601, nMelIndex)) ||
                    (pAufenthalt->psych_mode == 7 && !IsConstMEL (pContext, CONSTMEL_AM130, nMelIndex)) ||
                    (pAufenthalt->psych_mode == 6 && !IsConstMEL (pContext, CONSTMEL_AM120, nMelIndex) && (modelljahr < 2017 || !IsConstMEL (pContext, CONSTMEL_AM130, nMelIndex))) ||
                    (pAufenthalt->psych_mode == 5 && !IsConstMEL (pContext, CONSTMEL_AM110, nMelIndex)) ||
                    (pAufenthalt->psych_mode == 8 && !IsConstMEL (pContext, CONSTMEL_AM150, nMelIndex))) {

					if (bTraceDetail)
					{
						sprintf (sTraceLine, "Aufgrund des Psych-Modus wird MEL #%d (%5.5s) fuer die Punkteberechnung deaktiviert", i+1, pAufenthalt->mel[i].MEL);
						TRACE_DETAIL(pAufenthalt, -1, sTraceLine);
					}

					InactivateMel (pAufenthalt, i);
				}
			}
			else if ((!m7501_ok && IsConstMEL (pContext, CONSTMEL_AM030_7501, nMelIndex)) || 
					 (!m7506_ok && IsConstMEL (pContext, CONSTMEL_AM020_7506, nMelIndex))) 
			{
				InactivateMel (pAufenthalt, i);
				kjnp_missing_fc = -1;
			}
			else if ((!m7316_ok && IsConstMEL (pContext, CONSTMEL_AM040_7316, nMelIndex)) ||
					 (!m7321_ok && IsConstMEL (pContext, CONSTMEL_AM050_7321, nMelIndex)))
			{
				InactivateMel (pAufenthalt, i);
				alk_missing_fc = -1;
			}
			else if (modelljahr >= 2013 && !mAM120_ok && IsConstMEL (pContext, CONSTMEL_AM120, nMelIndex))
			{
				InactivateMel (pAufenthalt, i);
			}
            else if (modelljahr >= 2017 && IsConstMEL (pContext, CONSTMEL_AM110, nMelIndex))
            {
                InactivateMel (pAufenthalt, i);
            }
            else if (modelljahr >= 2022 && IsConstMEL (pContext, CONSTMEL_AM150, nMelIndex))
            {
                InactivateMel (pAufenthalt, i);
            }
            else if (modelljahr >= 2017 && !mAM130_ok && IsConstMEL (pContext, CONSTMEL_AM130, nMelIndex))
            {
                InactivateMel (pAufenthalt, i);
            }
            else if (!mAM100_ok && IsConstMEL (pContext, CONSTMEL_AM100, nMelIndex))
			{
				InactivateMel (pAufenthalt, i);
			}
            else if (!mAM140_ok && IsConstMEL (pContext, CONSTMEL_AM140, nMelIndex))
            {
                InactivateMel (pAufenthalt, i);
            }
			else if (!m7401_ok && IsConstMEL (pContext, CONSTMEL_AA040_7401, nMelIndex))
			{
				InactivateMel (pAufenthalt, i);
				stroke_missing_fc = -1;
			}
			else if (MELInIndexList(pContext, CONSTMEL_LIST2, nMelIndex) && pAufenthalt->psych_mode == 0) {
				InactivateMel (pAufenthalt, i);
			}
			else if (!m1255_ok && IsConstMEL (pContext, CONSTMEL_LH020_1255, nMelIndex) && !(pAufenthalt->acceptedMEL & ACCEPTED_MEL_LH020))
			{
				InactivateMel (pAufenthalt, i);
			}
			else if (!mDB020_ok && IsConstMEL (pContext, CONSTMEL_DB020, nMelIndex) && !(pAufenthalt->acceptedMEL & ACCEPTED_MEL_DB020))
			{
				InactivateMel (pAufenthalt, i);
			}
			else if (!mAJ140_ok && IsConstMEL (pContext, CONSTMEL_AJ140, nMelIndex) && !(pAufenthalt->acceptedMEL & ACCEPTED_MEL_AJ140))
			{
				InactivateMel (pAufenthalt, i);
			}
			else if (!mAK010_ok && IsConstMEL (pContext, CONSTMEL_AK010, nMelIndex) && !(pAufenthalt->acceptedMEL & ACCEPTED_MEL_AK010))
			{
				InactivateMel (pAufenthalt, i);
			}
			else if (!mLH010_ok && IsConstMEL (pContext, CONSTMEL_LH010, nMelIndex) && !(pAufenthalt->acceptedMEL & ACCEPTED_MEL_LH010))
			{
				InactivateMel (pAufenthalt, i);
			}
			else if (!mLH021_ok && IsConstMEL (pContext, CONSTMEL_LH021, nMelIndex) && !(pAufenthalt->acceptedMEL & ACCEPTED_MEL_LH021))
			{
				InactivateMel (pAufenthalt, i);
			}
		}
        if (pScoringDetails && !memcmp (pAufenthalt->mel[i].MEL, "00000", 5)) {
            pScoringDetails->pGroup[i].inactive = 1;
        }
	}

	/* Intensiv-Zuschlge */

	if (SpezAnz (pContext)) for (i = 0; i < pAufenthalt->abt_anz; i++) {
		
		if (!IsEmpty (pAufenthalt->abt[i].ABTFC, 8)) {
			
			sprintf (tmp_spez_buf, "Ixxxx%4.4s%8.8s", pAufenthalt->KANR, pAufenthalt->abt[i].ABTFC);
			pSpez = SpezInArray (pContext, tmp_spez_buf);

			if (pSpez && !ABTFC_Spez_Check (pAufenthalt, (int) i, pSpez, 0)) pSpez = 0;

			if (pSpez && (*pSpez)[0] == 'I') {
				long int_zuschlag;
				int_zuschlag = (long) simVal ((*pSpez) + 29, 6);

				if (!pAufenthalt->abt[i].ABTVD && (pAufenthalt->ENTART == 'S' || pAufenthalt->ENTART == 'T' || (pAufenthalt->ENTART == LKF_ENTART_REVERS && modelljahr < 2019 /* V2019.2 */))) { /* Intensiv 0-Tage: wenn Intensivabteilung */
                    if (i == pAufenthalt->abt_anz - 1 || IsEmpty (pAufenthalt->abt[i+1].ABTFC, 8)) {
                        pAufenthalt->INTZU += int_zuschlag;
                        if (pScoringDetails) {
                            pScoringDetails->pAbt[i].spezPunkte += int_zuschlag;
                            strcpy (pScoringDetails->pAbt[i].spezBereich, "INTENSIV");
                        }

                    } /* entlassene Abteilung und Patient */
					else if (bTraceDetail)
					{
                        sprintf (sTraceLine, "Abteilungs-Intensivzuschlag %ld wird nicht addiert (Entlassungsabteilung mit Verweildauer = 0 Tage, ENTART S/T)", int_zuschlag);
                        if (pScoringDetails) {
                            lkf_scoringdetails_addComment(pAufenthalt, sTraceLine);
                        }
						TRACE_DETAIL(pAufenthalt, -1, sTraceLine);
					}
				}
                else {
                    pAufenthalt->INTZU += int_zuschlag * pAufenthalt->abt[i].ABTVD;
                    if (pScoringDetails) {
                        pScoringDetails->pAbt[i].spezPunkte += int_zuschlag * pAufenthalt->abt[i].ABTVD;
                        strcpy (pScoringDetails->pAbt[i].spezBereich, "INTENSIV");
                    }
                }
				if (bTraceDetail && int_zuschlag)
				{
					sprintf (sTraceLine, "Abteilungs-Intensivzuschlag: %ld, Gesamtzuschlag: %ld", int_zuschlag, pAufenthalt->INTZU);
					TRACE_DETAIL(pAufenthalt, (int) i, sTraceLine);
				}
			}
		}
	}

	/* Scoring_Filter */
	lkf_free_aufenthalt_subgruppen (pAufenthalt);

	if (pAufenthalt->mel_anz) {
		lkf_create_aufenthalt_subgruppen (pAufenthalt, pAufenthalt->mel_anz); /* V13.0 */
        plMaxMELZU = (long *) LKF_CALLOC(sizeof(long), pAufenthalt->mel_anz); /* V22.1 */
        plCurMELZU = (long *) LKF_CALLOC(sizeof(long), pAufenthalt->mel_anz); /* V22.1 */
		pAufenthalt->optimizeable = 0;
	}
    else {
        plMaxMELZU = plCurMELZU = 0;
    }
    
	pAufenthalt->pat_subgruppen = 0;

	psych_7591 = psych_7592 = 0;

	for (i = 0; i < pAufenthalt->mel_anz; i++) {

        melanz = (int) pAufenthalt->mel[i].nTagMelAnzRelevant;
        if (melanz == 0) {
            if (pScoringDetails) {
                pScoringDetails->pGroup[i].inactive = 2;
                pScoringDetails->pGroup[i].subgruppeIndex = -1;
                pScoringDetails->pGroup[i].originalMelIndex = pAufenthalt->mel[i].mel_nr;
            }
            continue;
        }
            

        pMel = FindMel (pContext, pAufenthalt->mel[i].MEL);
		
        if (pMel && melanz > 0) {
			
			if (isGrpMel (pMel))
			{
                sprintf (pAufenthalt->subgruppe[pAufenthalt->pat_subgruppen].gruppe, "%-8.8s%-5.5s",
							 pMel->sGruppe, pAufenthalt->mel[i].MEL);
				pAufenthalt->subgruppe[pAufenthalt->pat_subgruppen].mel_id = (int) i;
				pAufenthalt->subgruppe[pAufenthalt->pat_subgruppen].bOp = (pMel->cOp=='J');
				pAufenthalt->subgruppe[pAufenthalt->pat_subgruppen].lMelDat = lkf_dat2int (pAufenthalt->mel[i].MELDAT);
				if (pAufenthalt->mel[i].nMelMelNotAccepted)
					pAufenthalt->subgruppe[pAufenthalt->pat_subgruppen].nMelMelGrp = (int) pAufenthalt->mel[i].nMelMelIndex;
				else 
					pAufenthalt->subgruppe[pAufenthalt->pat_subgruppen].nMelMelGrp = 0;
				if (pAufenthalt->pMelMel && (pAufenthalt->pMelMel[AUFENTHALT_MELMELINDEX(pAufenthalt,(int)i,(int)i)]))
					melanz = 1; /* MEL-MEL mit sich selbst -> anzahl = 1 */

                if (pScoringDetails) {
                    pAufenthalt->subgruppe[pAufenthalt->pat_subgruppen].originalIndex = pAufenthalt->pat_subgruppen;
                    pScoringDetails->pGroup[i].subgruppeIndex = pAufenthalt->pat_subgruppen;
                    pScoringDetails->pGroup[i].originalMelIndex = pAufenthalt->mel[i].mel_nr;
                }
				pAufenthalt->subgruppe[pAufenthalt->pat_subgruppen++].anz = (unsigned int) melanz;
			}
            else if (pScoringDetails) {
                pScoringDetails->pGroup[i].inactive = 2;
                pScoringDetails->pGroup[i].subgruppeIndex = -1;
                pScoringDetails->pGroup[i].originalMelIndex = pAufenthalt->mel[i].mel_nr;
            }
		}
	}

	bMelZuModus = 0; /* V13.0 */

	if (!pAufenthalt->pat_subgruppen) {

		int zd_hdggruppe;
		int grpcnt;

		if (pContext->flags & LKFCONTEXT_OPTIMIZER) {
		
			zd_hdggruppe = 0;

			for (i = 1; i < pAufenthalt->diag_anz; i++) {
			
				diag = pAufenthalt->diag[i].DIAG;
                pICD = FindDiag (pStamm, diag); /* V13.3 */

                if (pICD && !memcmp ((*pICD) + 9, "HDG", 3)) {
                    ++zd_hdggruppe;
                    if (pScoringDetails) {
                        pScoringDetails->pDiagGroup[i].subgruppeIndex = zd_hdggruppe;
                    }
                }
                else if (pScoringDetails) {
                    pScoringDetails->pDiagGroup[i].inactive = 1;
                }
			}

		}
        else {
            if (pScoringDetails) {
                for (i = 1; i < pAufenthalt->diag_anz; i++) {
                    pScoringDetails->pDiagGroup[i].inactive = 1;
                }
            }
            zd_hdggruppe = 0;
        }
		lkf_create_aufenthalt_subgruppen (pAufenthalt, 1 + (unsigned int) zd_hdggruppe);

		pAufenthalt->optimizeable = zd_hdggruppe;

        pICD = FindDiag (pStamm, pAufenthalt->HD); /* V13.3 */

		if (pICD && !memcmp ((*pICD) + 9, "HDG", 3)) {
			sprintf (pAufenthalt->subgruppe[0].gruppe, "%-8.8s    ", (*pICD) + 9);
			pAufenthalt->subgruppe[0].anz = 1;
			pAufenthalt->subgruppe[0].mel_id = -1; /* V13.0 */
			pAufenthalt->subgruppe[0].lMelDat = 0; /* V13.0 */
			sprintf (pAufenthalt->subgruppe[0].diag, "%-6.6sH %-4.4s",
				pAufenthalt->diag[0].DIAG, pAufenthalt->diag[0].DIAGPOS);
			pAufenthalt->pat_subgruppen = 1 + (unsigned int) pAufenthalt->optimizeable;

			if (pAufenthalt->optimizeable) {
			
				grpcnt = 0;

				for (i = 1; i < pAufenthalt->diag_anz; i++) {

					diag = pAufenthalt->diag[i].DIAG;
                    pICD = FindDiag (pStamm, diag);

					if (pICD && !memcmp ((*pICD) + 9, "HDG", 3)) {
						++grpcnt;
						sprintf (pAufenthalt->subgruppe[grpcnt].gruppe, "%-8.8s      ", (*pICD) + 9);
						pAufenthalt->subgruppe[grpcnt].anz = 1;
						pAufenthalt->subgruppe[grpcnt].mel_id = -1; /* V13.0 */
						sprintf (pAufenthalt->subgruppe[grpcnt].diag, "%-6.6sZ %-4.4s",
							diag, pAufenthalt->diag[i].DIAGPOS);

					}
				}
			}
		}
        if (pScoringDetails) {
            pScoringDetails->isDIAGGroup = 1;
            pScoringDetails->pDiagGroup[0].subgruppeIndex = 0;
            for (i = 1; i < pAufenthalt->hdiag_anz; i++) {
                pScoringDetails->pDiagGroup[i].inactive = 1;
            }
        }
	}
	else
	{
        if (pScoringDetails) {
            unsigned int i, j, currentIndex;
            
            int *originalIndexes = (int *) LKF_CALLOC (sizeof(int), pAufenthalt->mel_anz);
            
            for (i = 0; i < pAufenthalt->mel_anz; i++) {
                originalIndexes[i] = pScoringDetails->pGroup[i].subgruppeIndex;
            }
            
            SortSubgruppeMelDat (pAufenthalt->pat_subgruppen, pAufenthalt->subgruppe);
            
            for (i = 0; i < pAufenthalt->mel_anz; i++) {
                currentIndex = originalIndexes[i];
                if (currentIndex >= 0) {
                    for (j = 0; j < pAufenthalt->pat_subgruppen; j++) {
                        if (pAufenthalt->subgruppe[j].originalIndex == currentIndex) {
                            pScoringDetails->pGroup[i].subgruppeIndex = (int) j;
                            break;
                        }
                    }
                }
            }
            LKF_FREE (originalIndexes);
        }
        else {
            SortSubgruppeMelDat (pAufenthalt->pat_subgruppen, pAufenthalt->subgruppe);
        }
        bMelZuModus = 1;
	}

    for (i = 0; i < pAufenthalt->pat_subgruppen; i++) {

		pSubgruppe = pAufenthalt->subgruppe + i;

		pSubgruppe->pMelZu = (MELZU_DATA *) LKF_CALLOC (pAufenthalt->mel_anz, sizeof (MELZU_DATA));

		if (i && pAufenthalt->optimizeable) diag = pSubgruppe->diag;
		else diag = pAufenthalt->HD;

        if (pScoringDetails) {
            pScoringDetails->pCurrentGroup = pScoringDetails->isDIAGGroup ? diagDetailsForSubgruppe(pScoringDetails, i) : pScoringDetails->pGroup + pSubgruppe->mel_id;
        }
        
		if ((pKnoten = GetLKFKnoten (pStamm, pSubgruppe->gruppe, errbuf))) {
		
			if (bTraceDetail)
			{
				if (pSubgruppe->mel_id < 0)
				{
					sprintf (sTraceLine, "Subgruppenberechnung #%d, Gruppe = %-8.8s, Diag = %-6.6s", i+1, pSubgruppe->gruppe, diag);
				}
				else
				{
					sprintf (sTraceLine, "Subgruppenberechnung #%d, Gruppe = %-8.8s, Diag = %-6.6s, MEL=%5.5s (%d), op=%d", i+1, pSubgruppe->gruppe, diag, pAufenthalt->mel[pSubgruppe->mel_id].MEL, pSubgruppe->mel_id, pSubgruppe->bOp ? 1 : 0);
				}
				TRACE_DETAIL (pAufenthalt, -1, sTraceLine);
			}

			act_knoten = 0;
			is_agr01 = 0;

			nSplitSteps = 0;

			while (pKnoten->nfl[act_knoten] != -99)
			{
				if (++nSplitSteps > 20)
				{ 
					/* Nach 20 Schritten abbrechen - vermutlich Fehler im Stammdaten-File */
					sprintf (pContext->errortext, "FEHLER beim Scoring-Split von '%8.8s', Abbruch nach 20 Schritten, vermutlich Fehler im Stammdaten-File",
							pAufenthalt->subgruppe[i].gruppe);
					if (pAufenthalt->mel_anz && tmp_mel) {
						LKF_FREE (tmp_mel);
						LKF_FREE (tmp_melanz);
					}
					return -2;
				}

                scoring_split_result = bModell2017p ? scoring_split_2017 (pAufenthalt, pKnoten, act_knoten, pSubgruppe->gruppe, &is_agr01, diag, pSubgruppe->mel_id, bTraceDetail)
                                                    : scoring_split_2009 (pAufenthalt, pKnoten, act_knoten, pSubgruppe->gruppe, &is_agr01, diag, pSubgruppe->mel_id, bTraceDetail);

				if (scoring_split_result) 
					 act_knoten = pKnoten->nfl[act_knoten]-1;
				else act_knoten = pKnoten->nfl[act_knoten];

				if (bTraceDetail)
				{
					sprintf (sTraceLine, "Scoring-Split: Ergebnis=%d, Neuer Knoten=%d", scoring_split_result, act_knoten);
					TRACE_DETAIL (pAufenthalt, -1, sTraceLine);
				}
			}

			pSubgruppe->knoten_char = pKnoten->knoten_char[act_knoten];
			pSubgruppe->vd_u  = pKnoten->vd_u[act_knoten];
			pSubgruppe->vd_o  = pKnoten->vd_o[act_knoten];
			pSubgruppe->vd_mw = pKnoten->vd_mw[act_knoten];

			if (pSubgruppe->mel_id >= 0 && !MELInIndexList (pContext, MEL_NICHTREDUZIERT, pAufenthalt->mel[pSubgruppe->mel_id].nMelIndex))
			{
				pSubgruppe->melzu_voll = 0;
				pSubgruppe->melzu_reduziert = pKnoten->melzu_reduziert[act_knoten];
				if (pSubgruppe->melzu_reduziert == 0 && modelljahr < 2010)
					pSubgruppe->melzu_voll = 1; /* Fehlerkorrektur reduzierte MEL 0 erst ab Modell 2010 */
			}
			else {
                if (pScoringDetails && pSubgruppe->mel_id >= 0) {
                    sprintf (sTraceLine, "Voller MEL-Zuschlag fuer %5.5s", pAufenthalt->mel[pSubgruppe->mel_id].MEL);
                    lkf_scoringdetails_addComment(pAufenthalt, sTraceLine);
                }
				pSubgruppe->melzu_reduziert = 0;
				pSubgruppe->melzu_voll = 1;
			}
			pSubgruppe->mel_median = pKnoten->median[act_knoten];

	/**************************************************/
	/*' Baum wurde abgearbeitet, merke das Maximum !! */
	/**************************************************/
            
            if (modelljahr >= 2017)
            {
                pSubgruppe->leistungs_anteil = simCLng(pKnoten->lk_anteil[act_knoten]);
            }
            else
            {
                calc = pKnoten->median[act_knoten]*pKnoten->lk_anteil[act_knoten]/100;
                pSubgruppe->leistungs_anteil = simCLng(calc);
            }
            

	/*************************************************************************************/
	/*' wenn gleiche Leistung mehrfach erbracht wurde, dann merke Leistungsanteile mit !!*/
	/*************************************************************************************/
			if (pSubgruppe->mel_id >= 0 && pAufenthalt->pMelMel && pAufenthalt->pMelMel[AUFENTHALT_MELMELINDEX(pAufenthalt,pSubgruppe->mel_id,pSubgruppe->mel_id)])
				pSubgruppe->MELZU = 0;
			else if (pSubgruppe->anz > 1) 
			{
                if (bMelZuModus && !pSubgruppe->melzu_voll) {
					/* reduzierter MEL-Zuschlag bei Mehrfachnennung */
				
                    if (modelljahr >= 2017) {
                        pSubgruppe->MELZU = simCLng(pSubgruppe->melzu_reduziert) * (pSubgruppe->anz - 1);
                        if (plCurMELZU) plCurMELZU[pSubgruppe->mel_id] = simCLng(pSubgruppe->melzu_reduziert) * (pSubgruppe->anz - 1);
                        
                    }
                    else {
                        pSubgruppe->MELZU = simCLng(pSubgruppe->mel_median * pSubgruppe->melzu_reduziert/100) * (pSubgruppe->anz - 1);
                        if (plCurMELZU) plCurMELZU[pSubgruppe->mel_id] = simCLng(pSubgruppe->mel_median * pSubgruppe->melzu_reduziert/100) * (pSubgruppe->anz - 1);
                    }
                    
                }
                else {
					pSubgruppe->MELZU = pSubgruppe->leistungs_anteil * (pSubgruppe->anz - 1);
                    if (plCurMELZU) plCurMELZU[pSubgruppe->mel_id] = pSubgruppe->leistungs_anteil * (pSubgruppe->anz - 1);
                }
			}
			else pSubgruppe->MELZU = 0;

            pSubgruppe->MELZU_DIRECT = pSubgruppe->MELZU;
            
			if (pSubgruppe->mel_id >= 0)
				pSubgruppe->pMelZu[pSubgruppe->mel_id].lMelZu = pSubgruppe->MELZU;

			pSubgruppe->LDFPLK = pSubgruppe->LDFPTK = pSubgruppe->VDZU = 0;
	/********************************/
	/*' wenn VD-Ausreier nach unten*/
	/********************************/

			if (pAufenthalt->GESVD == 0 || pAufenthalt->GESVD < pKnoten->vd_u[act_knoten]) {
                if (modelljahr >= 2017) {
                    if (pAufenthalt->GESVD == 0) {
                        if (pAufenthalt->ENTART == 'S' || pAufenthalt->ENTART == 'T')
                        {
                            pSubgruppe->SCUTK = 490;
                            pSubgruppe->SCUTK_DOUBLE = pSubgruppe->SCUTK;
                            pSubgruppe->SCULK = simCLng (pKnoten->lk_anteil[act_knoten]);
                        }
						else {
                            unsigned int k;
                            MEL_ARRAY_STRUC *pTklMel;
                            int TKLMel;
                            char tmp_mel_ka[12];
                            
                            TKLMel = 0;
                            pSubgruppe->TKLModus = 0;
                            
                            for (k = 0; k < pAufenthalt->mel_anz && !TKLMel; k++) {
                                    
                                pTklMel = FindMel (pContext, pAufenthalt->mel[k].MEL);
                                    
                                if (pTklMel) {
                                    if (pTklMel->cTKL== 'J') {
                                        sprintf (tmp_mel_ka, "%4.4s %5.5s", pAufenthalt->KANR, pAufenthalt->mel[k].MEL);
                                        if (!GetMELKATKLContext (pContext, tmp_mel_ka)) TKLMel = 1;
                                    }
                                    else if (pContext->modelljahr >= 2020 && pTklMel->cBdgr0 == 'W') {
                                        if (pAufenthalt->mel[k].melPlausFlags & LKF_MELCHECKFLAG_WX3P_ACCEPTED) {
                                            TKLMel = 1;
                                        }
                                    }
                                }
                            }
                            if (TKLMel)
                            {
                                pSubgruppe->SCUTK_DOUBLE = pKnoten->tk_anteil[act_knoten] / pKnoten->vd_u[act_knoten];

                                if (pContext->modelljahr >= 2020) {
                                    double maxpkt;
                                    double max_median_faktor = 1.5;
                                    maxpkt = pKnoten->tk_anteil[act_knoten] / pKnoten->vd_mw[act_knoten] * max_median_faktor;
                                    if (maxpkt < pSubgruppe->SCUTK_DOUBLE) {
                                        pSubgruppe->SCUTK_DOUBLE = maxpkt;
                                    }
                                }
                                pSubgruppe->SCUTK = simCLng (pSubgruppe->SCUTK_DOUBLE);

                                pSubgruppe->SCULK = simCLng (pKnoten->lk_anteil [act_knoten]);
                                pSubgruppe->TKLModus = 2;
                            }
                            else  if (!bMelZuModus) {
                                int found50mel = 0;
								pSubgruppe->SCUTK = 30;
								pSubgruppe->SCUTK_DOUBLE = pSubgruppe->SCUTK;
                                
                                for (k = 0; k < pAufenthalt->mel_anz; k++) {
                                    if (MELInIndexList(pAufenthalt->pScoringContext, MEL_LEIST50_LIST, pAufenthalt->mel[k].nMelIndex)) { /* V21.2 */
                                        found50mel = 1;
                                        break;
                                    }
                                }
                                
                                if (found50mel) {
                                    pSubgruppe->SCULK = simCLng (pKnoten->lk_anteil [act_knoten]*0.5);
                                }
                                else
                                    pSubgruppe->SCULK = 0;
							}
							else if (!memcmp(pSubgruppe->gruppe, "MEL22", 5)) {
								pSubgruppe->SCUTK = 235;
								pSubgruppe->SCUTK_DOUBLE = pSubgruppe->SCUTK;
								if (pSubgruppe->melzu_voll) {
									pSubgruppe->SCULK = simCLng (pKnoten->lk_anteil [act_knoten]);
								}
								else
                                    pSubgruppe->SCULK = simCLng (pSubgruppe->melzu_reduziert);
							}
							else {
                                if (pKnoten->tk_anteil[act_knoten] > 0) {
                                    pSubgruppe->SCUTK = 30;
                                    pSubgruppe->SCUTK_DOUBLE = pSubgruppe->SCUTK;
                                }
                                else {
                                    pSubgruppe->SCUTK = 0;
                                    pSubgruppe->SCUTK_DOUBLE = 0;
                                }
                                if (pSubgruppe->melzu_voll) {
                                    pSubgruppe->SCULK = simCLng (pKnoten->lk_anteil [act_knoten]);
                                }
                                else
                                    pSubgruppe->SCULK = simCLng (pSubgruppe->melzu_reduziert);
                                pSubgruppe->TKLModus = 1;
                            }
                        }
                    }
                    else {
                        pSubgruppe->SCUTK = simCLng (pKnoten->tk_anteil[act_knoten] * pAufenthalt->GESVD / pKnoten->vd_u[act_knoten]);
                        pSubgruppe->SCULK = simCLng (pKnoten->lk_anteil [act_knoten]);
                    }
                }
                else
                { /* vor Modell 2017 */
                    calc = pKnoten->median[act_knoten] * pKnoten->tk_anteil[act_knoten] / 100 * (pAufenthalt->GESVD + 1);
                    calc /= pKnoten->vd_u[act_knoten] + 1;
                    pSubgruppe->SCUTK_DOUBLE = calc;
                    pSubgruppe->SCUTK = simCLng (calc);
                    calc = pKnoten->median[act_knoten] * pKnoten->lk_anteil [act_knoten] / 100;
                    pSubgruppe->SCULK = simCLng (calc);
                }
            }
			else
	/********************************/
	/*' der ganz normale Patient    */
	/********************************/
			{
				pSubgruppe->SCUTK = 0;
				pSubgruppe->SCULK = 0;
				pSubgruppe->SCUTK_DOUBLE = 0.0;

                if (pContext->modelljahr >= 2017) {
                    pSubgruppe->LDFPLK = simCLng (pKnoten->lk_anteil[act_knoten]);
                    pSubgruppe->LDFPTK = simCLng (pKnoten->tk_anteil[act_knoten]);
                }
                else {
                    calc = pKnoten->median[act_knoten] * pKnoten->lk_anteil[act_knoten] / 100;
                    pSubgruppe->LDFPLK = simCLng (calc);
                    pSubgruppe->LDFPTK = simCLng(pKnoten->median[act_knoten]) - pSubgruppe->LDFPLK;
                }

	/********************************/
	/*' VD-Ausreier nach oben      */
	/********************************/

				if (!pAufenthalt->psych_mode && pAufenthalt->GESVD > pKnoten->vd_o[act_knoten]) {
				
					short vdo_max = pKnoten->vd_o[act_knoten] * 2;

                    if (pContext->modelljahr >= 2017) {
                        calc = pKnoten->tk_anteil[act_knoten];
                    }
                    else {
                        calc = pKnoten->median[act_knoten] * pKnoten->tk_anteil[act_knoten] / 100;
                    }
                    calc /= pKnoten->vd_mw[act_knoten];
                    day_mean = calc;


					for (j = (unsigned int) pKnoten->vd_o[act_knoten] + 1; j <= (unsigned int) pAufenthalt->GESVD; j++) {
					
						calc = pKnoten->vd_o[act_knoten] * day_mean;
						
						if (pContext->modelljahr < 2009)					
							calc /= j;
						else
						{
							calc /= (j>(unsigned int)vdo_max)?vdo_max:j;
						}
                        pSubgruppe->VDZU += simCLng (calc);
					}
				}
			}
			
			if (bTraceDetail)
			{
				sprintf (sTraceLine, "Knoten '%c' (vd_u=%3d, vd_o=%3d, vd_mw=%6.2f), SCUTK=%5ld, SCULK=%5ld, VDZU=%5ld, LDFPTK=%5ld, LKFPLK=%5ld, MELZU=%5ld",
					pSubgruppe->knoten_char, pSubgruppe->vd_u, pSubgruppe->vd_o, pSubgruppe->vd_mw,
					pSubgruppe->SCUTK, pSubgruppe->SCULK, pSubgruppe->VDZU, pSubgruppe->LDFPTK, pSubgruppe->LDFPLK, pSubgruppe->MELZU);
				TRACE_DETAIL (pAufenthalt, -1, sTraceLine);
			}

		}
		else {
			sprintf (pContext->errortext, "FEHLER beim Einlesen des Files '%8.8s': %s",
					 pAufenthalt->subgruppe[i].gruppe, errbuf);
			if (pAufenthalt->mel_anz && tmp_mel) {
				LKF_FREE (tmp_mel);
				LKF_FREE (tmp_melanz);
			}
			return -2;
		}
	}

	/* Bestimme das Maximum */

	max_score = 0;
    max_scges = 0;
    
	for (i = 0; i < pAufenthalt->pat_subgruppen; i++) 
	{
		pSubgruppe = pAufenthalt->subgruppe + i;
        
        if (pScoringDetails) {
            if (pScoringDetails->isDIAGGroup) {
                pScoringDetails->pCurrentGroup = diagDetailsForSubgruppe(pScoringDetails, i);
            }
            else {
                pScoringDetails->pCurrentGroup = pScoringDetails->pGroup + pSubgruppe->mel_id;
            }
        }
        
        if (plCurMELZU) {
            memset (plCurMELZU, 0, sizeof(long) * pAufenthalt->mel_anz);
            if (pSubgruppe->mel_id >= 0) {
                plCurMELZU[pSubgruppe->mel_id] = pSubgruppe->MELZU;
            }
        }
        
		bIgnoreSubgruppe = 0;
		if (!pAufenthalt->optimizeable) 
		{
			if (bMelZuModus)
			{
				int nActIndex;
				int nActIndexOp;
				long lCurrMelZu;
				long lCurrMelZuAnz;
				long lMelZu;
				long lActIndexOptValue, lActIndexOptValueOp, lCurrOptValue;
				char bOp;
				unsigned int nNextDynMelIndex = 0;

				lAktDatum = 0;
				lAktMaxZuschlag = 0;
				lAktMaxZuschlagOp = 0;
				lActIndexOptValueOp = 0;
                lActIndexOptValue = 0;


				if (pAufenthalt->pMelMel)
				{
					for (j = 1; !bIgnoreSubgruppe && j <= pAufenthalt->nMelMelCount; j++)
					{
						if (!lkf_aufenthalt_optimize_melmel (pAufenthalt, pSubgruppe, (int) j, 1, i, plCurMELZU))
							bIgnoreSubgruppe = 1;
					}
				}

				if (!bIgnoreSubgruppe)
				{
					/* Speziallogik, wenn die aktuelle MEL derart ist: nur die aktuelle MEL bekommt den Zuschlag */
					if (!pSubgruppe->melzu_voll) /* V14.2 */
						lGrpMelDat = (unsigned int) pSubgruppe->lMelDat;
					else 
						lGrpMelDat = 0;

					nCurrMelGrpIndex = pAufenthalt->subgruppe[i].nMelMelGrp;
					bOp = pAufenthalt->subgruppe[i].bOp; /* V14.0 */

					nActIndex = -1;
					nActIndexOp = -1;

					for (j = 0; j < pAufenthalt->pat_subgruppen; j++) 
					{	
						if (i != j)
						{
							LKF_AUFENTHALT_SUBGRUPPE *pGrp = pAufenthalt->subgruppe + j;

							if (pSubgruppe->pMelZu[pGrp->mel_id].lMelZu == -1)
							{  /* V13.5 */
								lCurrMelZu = 0;
								lCurrMelZuAnz = 0;
								pSubgruppe->pMelZu[pGrp->mel_id].lMelZu = 0;
							}
							else if (!pGrp->melzu_voll)
							{
								if (pContext->modelljahr >= 2010)
								{
                                    if (pContext->modelljahr >= 2017)
                                        lCurrMelZu = simCLng(pGrp->melzu_reduziert);
                                    else
                                        lCurrMelZu = simCLng(pGrp->mel_median*pGrp->melzu_reduziert/100);
									
                                    if (pGrp->anz > 1)
										lCurrMelZuAnz = lCurrMelZu * (pGrp->anz-1);
									else
										lCurrMelZuAnz = 0;

									pSubgruppe->pMelZu[pGrp->mel_id].lDynMelDiff = pGrp->leistungs_anteil - lCurrMelZu;
									pSubgruppe->pMelZu[pGrp->mel_id].nDynMelId = 0;

									if (pGrp->lMelDat != lGrpMelDat || pGrp->bOp != bOp)
									{
										lMelZu = pGrp->leistungs_anteil + lCurrMelZuAnz;
										lCurrOptValue = pGrp->leistungs_anteil - lCurrMelZu;
										if (pGrp->lMelDat != lAktDatum)
										{
											if (nActIndex >= 0)
											{
												pSubgruppe->MELZU -= pSubgruppe->pMelZu[nActIndex].lMelZu;
                                                if (plCurMELZU) {
                                                    plCurMELZU[nActIndex] += lAktMaxZuschlag - pSubgruppe->pMelZu[nActIndex].lMelZu;
                                                }
												pSubgruppe->pMelZu[nActIndex].lMelZu = lAktMaxZuschlag;
												pSubgruppe->pMelZu[nActIndex].lDynMelDiff = 0;							
												pSubgruppe->MELZU += lAktMaxZuschlag;
												nActIndex = -1;
												lAktMaxZuschlag = 0;
												lActIndexOptValue = 0;
											}											
											if (nActIndexOp >= 0)
											{
												pSubgruppe->MELZU -= pSubgruppe->pMelZu[nActIndexOp].lMelZu;
                                                if (plCurMELZU) {
                                                    plCurMELZU[nActIndexOp] += lAktMaxZuschlagOp - pSubgruppe->pMelZu[nActIndexOp].lMelZu;
                                                }
												pSubgruppe->pMelZu[nActIndexOp].lMelZu = lAktMaxZuschlagOp;
												pSubgruppe->pMelZu[nActIndexOp].lDynMelDiff = 0;							
												pSubgruppe->MELZU += lAktMaxZuschlagOp;
												nActIndexOp = -1;
												lAktMaxZuschlagOp = 0;
												lActIndexOptValueOp = 0;
											}

											lAktDatum = (unsigned int) pGrp->lMelDat;
										}

										if (pGrp->bOp)
										{
											if (nActIndexOp < 0)
												pSubgruppe->pMelZu[pGrp->mel_id].nDynMelId = (int) ++nNextDynMelIndex;
											else
												pSubgruppe->pMelZu[pGrp->mel_id].nDynMelId = pSubgruppe->pMelZu[nActIndexOp].nDynMelId;

											if (lCurrOptValue > lActIndexOptValueOp)
											{
												nActIndexOp = pGrp->mel_id;
												lAktMaxZuschlagOp = lMelZu;
												lActIndexOptValueOp = lCurrOptValue;
											}
										}
										else
										{
											if (nActIndex < 0)
												pSubgruppe->pMelZu[pGrp->mel_id].nDynMelId = (int) ++nNextDynMelIndex;
											else
												pSubgruppe->pMelZu[pGrp->mel_id].nDynMelId = pSubgruppe->pMelZu[nActIndex].nDynMelId;

											if (lCurrOptValue > lActIndexOptValue)
											{
												if (!lActIndexOptValue)
													pSubgruppe->pMelZu[pGrp->mel_id].nDynMelId = (int) ++nNextDynMelIndex;
												nActIndex = pGrp->mel_id;
												lAktMaxZuschlag = lMelZu;
												lActIndexOptValue = lCurrOptValue;
											}
										}
									}
								}
								else
								{ /* Modell 2009 */
									lCurrMelZu = simCLng(pGrp->mel_median*pGrp->melzu_reduziert/100);
									if (pGrp->anz > 1)
										lCurrMelZuAnz = simCLng(pGrp->mel_median*pGrp->melzu_reduziert/100) * (pGrp->anz-1);
									else
										lCurrMelZuAnz = 0;

									pSubgruppe->pMelZu[pGrp->mel_id].lDynMelDiff = pGrp->leistungs_anteil - lCurrMelZu;
									pSubgruppe->pMelZu[pGrp->mel_id].nDynMelId = 0;

									if (pGrp->lMelDat != lGrpMelDat)
									{
										lMelZu = pGrp->leistungs_anteil + lCurrMelZuAnz;
										lCurrOptValue = pGrp->leistungs_anteil - lCurrMelZu;
										if (pGrp->lMelDat != lAktDatum)
										{
											if (nActIndex >= 0)
											{
												pSubgruppe->MELZU -= pSubgruppe->pMelZu[nActIndex].lMelZu;
                                                if (plCurMELZU) {
                                                    plCurMELZU[nActIndex] += lAktMaxZuschlag - pSubgruppe->pMelZu[nActIndex].lMelZu;
                                                }

												pSubgruppe->pMelZu[nActIndex].lMelZu = lAktMaxZuschlag;
												pSubgruppe->pMelZu[nActIndex].lDynMelDiff = 0;							
												pSubgruppe->MELZU += lAktMaxZuschlag;
												nActIndex = -1;
												lAktMaxZuschlag = 0;
												lActIndexOptValue = 0;
											}
											lAktDatum = (unsigned int) pGrp->lMelDat;
										}

										if (nActIndex < 0)
											pSubgruppe->pMelZu[pGrp->mel_id].nDynMelId = (int) ++nNextDynMelIndex;
										else
											pSubgruppe->pMelZu[pGrp->mel_id].nDynMelId = pSubgruppe->pMelZu[nActIndex].nDynMelId;
										if (lCurrOptValue > lActIndexOptValue)
										{
											nActIndex = pGrp->mel_id;
											lAktMaxZuschlag = lMelZu;
											lActIndexOptValue = lCurrOptValue;
										}
									}
								}
							}
							else
							{
								lCurrMelZu = pGrp->leistungs_anteil * pGrp->anz;
								lCurrMelZuAnz = 0;
								pSubgruppe->pMelZu[pGrp->mel_id].nDynMelId = 0;
							}

							pSubgruppe->pMelZu[pGrp->mel_id].lMelZu = lCurrMelZu + lCurrMelZuAnz;
							pSubgruppe->MELZU += pSubgruppe->pMelZu[pGrp->mel_id].lMelZu;
                            if (plCurMELZU) {
                                plCurMELZU[pGrp->mel_id] += pSubgruppe->pMelZu[pGrp->mel_id].lMelZu;
                            }

						}
					}
					if (nActIndex >= 0)
					{
						pSubgruppe->MELZU -= pSubgruppe->pMelZu[nActIndex].lMelZu;
                        if (plCurMELZU) {
                            plCurMELZU[nActIndex] += lAktMaxZuschlag - pSubgruppe->pMelZu[nActIndex].lMelZu;
                        }

						pSubgruppe->pMelZu[nActIndex].lMelZu = lAktMaxZuschlag;
						pSubgruppe->pMelZu[nActIndex].lDynMelDiff = 0;							
						pSubgruppe->MELZU += lAktMaxZuschlag;
					}
					if (pContext->modelljahr >= 2010)
					{
						if (nActIndexOp >= 0)
						{
							pSubgruppe->MELZU -= pSubgruppe->pMelZu[nActIndexOp].lMelZu;
                            if (plCurMELZU) {
                                plCurMELZU[nActIndexOp] += lAktMaxZuschlagOp - pSubgruppe->pMelZu[nActIndexOp].lMelZu;
                            }

							pSubgruppe->pMelZu[nActIndexOp].lMelZu = lAktMaxZuschlagOp;
							pSubgruppe->pMelZu[nActIndexOp].lDynMelDiff = 0;							
							pSubgruppe->MELZU += lAktMaxZuschlagOp;
						}					
					}

					if (bTraceDetail && pSubgruppe && pSubgruppe->pMelZu)
					{

						TRACE_DETAIL (pAufenthalt, -1, "MEL    MELZU DYNDIFF DYNID");

						for (j = 0; j < pAufenthalt->mel_anz; j++)
						{
							MELZU_DATA *pData = pSubgruppe->pMelZu + j;
							sprintf (sTraceLine, "%5.5s  %5ld   %5ld %5d", pAufenthalt->mel[j].MEL, pData->lMelZu, pData->lDynMelDiff, pData->nDynMelId);
							TRACE_DETAIL (pAufenthalt, -1, sTraceLine);
						}
					}

					if (pAufenthalt->pMelMel)
					{
						for (j = 1; !bIgnoreSubgruppe && j <= pAufenthalt->nMelMelCount; j++)
						{
							if (!lkf_aufenthalt_optimize_melmel (pAufenthalt, pSubgruppe, (int) j, 0, i, plCurMELZU))
								bIgnoreSubgruppe = 1;
						}
					}
				}
			}
			else
			{
				for (j = 0; j < pAufenthalt->pat_subgruppen; j++) 
				{	
					if (i != j)
					{
						pSubgruppe->pMelZu[pAufenthalt->subgruppe[j].mel_id].lMelZu = pAufenthalt->subgruppe[j].leistungs_anteil * pAufenthalt->subgruppe[j].anz;
						pSubgruppe->MELZU += pSubgruppe->pMelZu[pAufenthalt->subgruppe[j].mel_id].lMelZu;
                        if (plCurMELZU) {
                            plCurMELZU[pAufenthalt->subgruppe[j].mel_id] += pSubgruppe->pMelZu[pAufenthalt->subgruppe[j].mel_id].lMelZu;
                        }

					}
				}
			}
		}

		if (bIgnoreSubgruppe)
			pSubgruppe->SCGES = 0;
		else
		{
			pSubgruppe->SCGES = pSubgruppe->LDFPTK + pSubgruppe->LDFPLK + pSubgruppe->SCUTK + pSubgruppe->SCULK + 
								pSubgruppe->VDZU + pSubgruppe->MELZU + pAufenthalt->INTZU + pAufenthalt->SCSPEZ;
			if (bSubgruppeTKL)
				lkf_scoring_tkl_subgruppe (pContext, pAufenthalt, pSubgruppe);
		}


		if (bTraceDetail)
		{

			sprintf (sTraceLine, "Subgruppe #%02d, GRP/MEL=%14.14s, Knt=%c, SCGES=%08ld, MELZU=%08ld, VDZU=%08ld, melid=%2d", i, pSubgruppe->gruppe, pSubgruppe->knoten_char, pSubgruppe->SCGES, pSubgruppe->MELZU, pSubgruppe->VDZU, pSubgruppe->mel_id);
			TRACE_DETAIL (pAufenthalt, -1, sTraceLine);
			
			if (pSubgruppe->mel_id >= 0)
			{
				sprintf (sTraceLine, "Subgruppe #%02d, MEL=%5.5s, MELDAT=%8.8s, melmel=%d, tagmelanz=%d/%d", i, pAufenthalt->mel[pSubgruppe->mel_id].MEL, pAufenthalt->mel[pSubgruppe->mel_id].MELDAT, 
																							 pAufenthalt->mel[pSubgruppe->mel_id].nMelMelIndex,
																							 pAufenthalt->mel[pSubgruppe->mel_id].nTagMelAnz, pAufenthalt->mel[pSubgruppe->mel_id].nTagMelAnzRelevant);
				TRACE_DETAIL (pAufenthalt, -1, sTraceLine);				
			}
			
			if (pSubgruppe->pMelZu)
			{
                for (j = 0; j < pAufenthalt->pat_subgruppen; j++)
				{
                    int k = pAufenthalt->subgruppe[j].mel_id;
                    if (k >= 0)
					{
						sprintf (sTraceLine, "#%02d: MEL=%5.5s MELDAT=%8.8s Zuschlag=%5ld Anz=%d/%d", j,
							pAufenthalt->mel[k].MEL,
							pAufenthalt->mel[k].MELDAT,
							pSubgruppe->pMelZu[k].lMelZu,
							pAufenthalt->mel[k].nTagMelAnz,
							pAufenthalt->mel[k].nTagMelAnzRelevant);
						TRACE_DETAIL (pAufenthalt, -1, sTraceLine);
					}
				}
			}
		}
        
        if (!pAufenthalt->optimizeable && ((pAufenthalt->pScoringContext->modelljahr >= 2017 && !(pContext->flags & LKFCONTEXT_SIM3)) ||
                                           (pAufenthalt->pScoringContext->modelljahr < 2017 && (pContext->flags & LKFCONTEXT_SIM3))))
        {
            long leist = pSubgruppe->leistungs_anteil;

            if (pSubgruppe->SCGES > 0 && (leist > max_score || (leist==max_score && pSubgruppe->SCGES > max_scges))) {
				if (i < pAufenthalt->pat_subgruppen - (unsigned int) pAufenthalt->optimizeable) {
					max_score = leist;
					max_scges = pSubgruppe->SCGES;
					pAufenthalt->max_index = (short) i;
                    pAufenthalt->optimized_subgruppe = (int) i; /* V21.0 */
                    if (plCurMELZU)
                        memcpy (plMaxMELZU, plCurMELZU, sizeof(long) * pAufenthalt->mel_anz);
                }
			}
		}
		else if (pSubgruppe->SCGES > max_score) {

			if (i < pAufenthalt->pat_subgruppen - (unsigned int) pAufenthalt->optimizeable) {
				max_score = pSubgruppe->SCGES;
				pAufenthalt->max_index = (short) i;
			}
			pAufenthalt->optimized_subgruppe = (int) i;
            if (plCurMELZU)
                memcpy (plMaxMELZU, plCurMELZU, sizeof(long) * pAufenthalt->mel_anz);
		}
	}
    
    pSubgruppe = pAufenthalt->subgruppe + pAufenthalt->max_index;
        
    
    if (plMaxMELZU) {
            
        for (i = 0; i < pAufenthalt->mel_anz; i++) {
            pAufenthalt->mel[i].melzu = plMaxMELZU[i];
            if (plMaxMELZU[i])
                pAufenthalt->mel[i].punkteRelevant = 1;
        }
        if (!pAufenthalt->optimizeable) {
            int maxmelId = pAufenthalt->subgruppe[pAufenthalt->max_index].mel_id;
            if (maxmelId >= 0) pAufenthalt->mel[maxmelId].punkteRelevant = 2;
        }

        lkf_adjust_melzu_pkt (pAufenthalt);
 /*
        for (i = 0; i < pAufenthalt->mel_anz; i++) {
            mz += plMaxMELZU[i];
        }

        if (pSubgruppe->MELZU != mz) {
            mz += 0;
        }
*/
        LKF_FREE (plMaxMELZU);
        LKF_FREE (plCurMELZU);
    }
    
    if (pAufenthalt->nMelMelCount > 0) {
        for (j = 0; j < pAufenthalt->mel_anz; j++) {
            lkf_setMelMelInfo (pAufenthalt, j);
        }
    }

	if (pAufenthalt->optimizeable) 
		pAufenthalt->optimized_percentage = (pAufenthalt->subgruppe[0].SCGES)*1.0 / pAufenthalt->subgruppe[pAufenthalt->optimized_subgruppe].SCGES;


	/* Setze Gesamttage wieder auf ursprnglichew Wert bei NEURO-Akutpatienten */

	if (teil_akutpat) 
	{
		pAufenthalt->GESVD += spezial_tage;
		pAufenthalt->GESVD_KORR -= spezial_tage;
	}

	pAufenthalt->LDFPLK = pSubgruppe->LDFPLK;
	pAufenthalt->LDFPTK = pSubgruppe->LDFPTK;
	pAufenthalt->SCUTK = pSubgruppe->SCUTK;
	pAufenthalt->SCUTK_DOUBLE = pSubgruppe->SCUTK_DOUBLE;
	pAufenthalt->SCULK = pSubgruppe->SCULK;
	pAufenthalt->VDZU = pSubgruppe->VDZU;
	pAufenthalt->MELZU = pSubgruppe->MELZU;
	pAufenthalt->SCGES = pSubgruppe->SCGES;
	pAufenthalt->VDO = pSubgruppe->vd_o;
	pAufenthalt->VDU = pSubgruppe->vd_u;
	pAufenthalt->VDMW = pSubgruppe->vd_mw;

	memcpy (pAufenthalt->GRP, pSubgruppe->gruppe, 8);
    pAufenthalt->maxMelIndex = pSubgruppe->mel_id;
	pAufenthalt->KNT[0] = pSubgruppe->knoten_char;


	if (bSubgruppeTKL)
	{
		pAufenthalt->TKLModus = pSubgruppe->TKLModus;
		pAufenthalt->SCGES_TKL = pAufenthalt->SCGES;
		pAufenthalt->SCUTK_TKL = pSubgruppe->SCUTK;
		pAufenthalt->SCUTK_NOTKL = simCLng(pSubgruppe->SCUTK_DOUBLE);
	}
    else if (bModell2017p) {
        pAufenthalt->TKLModus = pSubgruppe->TKLModus;
        pAufenthalt->SCGES_TKL = pAufenthalt->SCGES;
        pAufenthalt->SCUTK_TKL = pSubgruppe->SCUTK;
        pAufenthalt->SCUTK_NOTKL = simCLng(pSubgruppe->SCUTK_DOUBLE);
    }
    else
	{
		lkf_scoring_tkl (pContext, pAufenthalt);
		pAufenthalt->SCUTK_NOTKL = pAufenthalt->SCUTK;
		pAufenthalt->SCUTK = pAufenthalt->SCUTK_TKL;
	}

	pAufenthalt->SCGES = pAufenthalt->SCGES_TKL;

    for (i = 0; i < pAufenthalt->mel_anz; i++) {
        memcpy (pAufenthalt->mel[i].MEL, tmp_mel[i], 5);
        memcpy (pAufenthalt->mel[i].MELANZ, tmp_melanz[i], 4);
    }

	if (pAufenthalt->mel_anz && tmp_mel && tmp_melanz) {
		LKF_FREE (tmp_mel);
		LKF_FREE (tmp_melanz);
	}
    
    if (pAufenthalt->isAmb == 2) {
        if (pAufenthalt->abt_anz == 1)
         {
             pAufenthalt->abt[0].lp100 = (pAufenthalt->LDFPLK + pAufenthalt->SCULK + pAufenthalt->MELZU + pAufenthalt->SCSPEZ /* V2019.2 */) * 100;
             pAufenthalt->abt[0].tp100 = (pAufenthalt->LDFPTK + pAufenthalt->SCUTK) * 100;
         }
        else if (pAufenthalt->abt_anz > 1) {
            lkf_scoring_amb_divide(pAufenthalt, pAufenthalt->LDFPTK + pAufenthalt->SCUTK, 0);
            lkf_scoring_tkl_adjust(pAufenthalt, pAufenthalt->LDFPLK + pAufenthalt->MELZU + pAufenthalt->SCULK);
         }
    }
	return 0;
}
